<?php
/* Copyright (c) 1998-2016 ILIAS open source, Extended GPL, see docs/LICENSE */

include_once("./Services/Export/classes/class.ilXmlImporter.php");
include_once("./Modules/TestQuestionPool/classes/class.ilObjQuestionPool.php");
require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/classes/class.ilQuestionSetPoolPlugin.php";


/**
 * Class ilFormATestImporter
 * @author Nadia Matuschek <nmatuschek@databay.de>
 */
class ilFormATestImporter extends ilXmlImporter
{
	// for QPL import handling
	const SINGLE_IMPORT_NEW_POOL = -1;
	const SINGLE_IMPORT_NO_POOL  = -2;
	
	const IMPORT_FAT_NO_POOLS = 'xqsp_sel_0';
	const IMPORT_INTO_SELECTED_POOLS = 'xqsp_sel_1';
	
	/**
	 * @var ilObjQuestionPool
	 */
	protected $qpl_obj = NULL;
	/**
	 * @var ilObjQuestionSetPool
	 */
	public $xqsp_obj = NULL;
	
	/**
	 * @var ilObjFormATest
	 */
	public $xtsf_obj = NULL;

	protected $is_single_import = true;
	
	protected $import_into_xqsp = false;
	protected $import_into_qpl = false;
	
	/**
	 * @var int[]
	 */
	protected static $pool_by_parent_id = array();
	
	protected $current_parent_ref_id = 0;
	
	/**
	 * @var null
	 */
	protected $xml_file = NULL;
	/**
	 * @var null
	 */
	protected $qti_file = NULL;
	
	/**
	 * @var array
	 */
	protected $fat_qst_mapping = array();
	
	protected $xqsp_qst_mapping = array();
	
	/**
	 * @param $xml_file
	 */
	private function setXmlFile($xml_file)
	{
		$this->xml_file = $xml_file;
	}
	
	public function getXmlFile()
	{
		return $this->xml_file;
	}
	
	/**
	 * @param $qti_file
	 */
	public function setQtiFile($qti_file)
	{
		$this->qti_file = $qti_file;
	}
	
	public function getQtiFile()
	{
		return $this->qti_file;
	}
	
	/**
	 * ilQuestionSetPoolImporter constructor.
	 */
	public function __construct()
	{
		$this->plugin_object = ilFormATestPlugin::getInstance();
		
		$this->xqsp_plugin_object = ilQuestionSetPoolPlugin::getInstance();
		$this->xqsp_plugin_object->includeClass('models/class.ilQuestionSetPoolSet.php');
		$this->xqsp_plugin_object->includeClass('models/class.ilQuestionSetPoolPath.php');
		$this->xqsp_plugin_object->includeClass('models/class.ilQuestionSetPoolNode.php');
		$this->xqsp_plugin_object->includeClass('models/class.ilQuestionSetPoolJumpCondition.php');
		$this->xqsp_plugin_object->includeClass('class.ilQuestionSetPoolImporter.php');
	}
	
	public function init()
	{
		$qti_path = $this->getImportDirectory().'/Plugins/xtsf/set_1/expDir_1';
		$qti_file = $qti_path.'/xtsf_qti.xml';
		$xml_file = $this->getImportDirectory().'/Plugins/xtsf/set_1/export.xml';
	
		$this->validateXmlFile($xml_file);
		$this->setQtiFile($qti_file);
	}
	
	/**
	 * @param $xml_file
	 */
	public function validateXmlFile($xml_file)
	{
		$this->plugin_object->includeClass('class.ilFormATestXmlValidator.php');
		$validator = new ilFormATestXmlValidator();
		$validator->setXsdFile('/xml/xtsf_5_1.xsd');
		$validator->setXmlFile($xml_file);
		$validator->validate();
		$this->setXmlFile($xml_file);
		return true;
	}
	
	/**
	 * 
	 */
	public function createImportDirectory()
	{
		global $ilErr;
		
		include_once "./Services/Utilities/classes/class.ilUtil.php";
		
		$xtsf_data_dir = ilUtil::getDataDir()."/xtsf_data";
		ilUtil::makeDir($xtsf_data_dir);
		if(!is_writable($xtsf_data_dir))
		{
			$ilErr->raiseError("QuestionSetPool Data Directory (".$xtsf_data_dir.") not writeable.", $ilErr->error_obj->FATAL);
		}
		
		// create QuestionSetPool directory (data_dir/xtsf_data/xtsf_import)
		$import_dir = $xtsf_data_dir."/xtsf_import";
		ilUtil::makeDir($import_dir);
		if(!@is_dir($import_dir))
		{
			$ilErr->raiseError("Creation of QuestionSetPool import directory failed.", $ilErr->error_obj->FATAL);
		}
		
		$this->setImportDirectory($import_dir);
	}
	
	public function initializeQplObject($obj_id)
	{
		if($obj_id <= 0)
		{
			$this->qpl_obj = $this->createQplObject();
		}	
		else
		{
			$this->qpl_obj = new ilObjQuestionPool($obj_id, false);
			$this->qpl_obj->read(true);
		}
	}
	
	/**
	 * @param $obj_id
	 */
	public function initializeXqspObject($obj_id)
	{
		if($obj_id <= 0)
		{
			$this->xqsp_obj = $this->createXqspObject();
		}
		else
		{
			$ref_ids  = ilObject::_getAllReferences($obj_id);
			$this->xqsp_obj = new ilObjQuestionSetPool(current($ref_ids));
			$this->xqsp_obj->read(true);
		}
	}
	
	/**
	 * Creates a new questionpool object
	 * 
	 * copied from ilObjQuestionPoolGUI->importVerifiedFileObject()
	 */
	public function createQplObject()
	{
		// create new questionpool object
		$qpl_obj = new ilObjQuestionPool(0, true);
		// set type of questionpool object
		$qpl_obj->setType('qpl');
		// set title of questionpool object to "dummy"
		$qpl_obj->setTitle("dummy");
		// set description of questionpool object
		$qpl_obj->setDescription("questionpool creation");
		// set online
		$qpl_obj->setOnline(true);
		// create the questionpool class in the ILIAS database (object_data table)
		$qpl_obj->create(true);
		
		$qpl_obj->update();
		$qpl_obj->saveToDb();
		return $qpl_obj;
	}
	
	/**
	 * @return ilObjQuestionSetPool
	 */
	public function createXqspObject()
	{
		// create new questionsetpool object
		$xqsp_obj = new ilObjQuestionSetPool();
		// set type of questionsetpool object
		$xqsp_obj->setType('xqsp');
		// set title of questionsetpool object to "dummy"
		$xqsp_obj->setTitle("dummy");
		// set description of questionsetpool object
		$xqsp_obj->setDescription("questionsetpool creation");
		// set online
		$xqsp_obj->setOnline(true);
		// create the questionsetpool class in the ILIAS database (object_data table)
		$xqsp_obj->create(true);
		
		$xqsp_obj->update();
		return $xqsp_obj;
	}
	
	
	/**
	 * Import questions to QPL questionpool
	 */
	protected function parseQtiFile()
	{
		include_once "./Modules/TestQuestionPool/classes/class.ilObjQuestionPool.php";
		$tmp_dir = ilObjQuestionPool::_getImportDirectory();
		ilObjQuestionPool::_setImportDirectory($this->getImportDirectoryBase());

		$tmp_subdir = $_SESSION["qpl_import_subdir"];
		$_SESSION["qpl_import_subdir"] = basename($this->getImportDirectory());

		// start parsing of QTI files
		include_once "./Services/QTI/classes/class.ilQTIParser.php";
		$qtiParser = new ilQTIParser($this->getQtiFile(), IL_MO_PARSE_QTI, $this->xtsf_obj->getId());
		$result = $qtiParser->startParsing();
		$this->addQuestionMapping($qtiParser->getImportMapping());

		ilObjQuestionPool::_setImportDirectory($tmp_dir);
		$_SESSION["qpl_import_subdir"] = $tmp_subdir;
	}
	
	protected function addQuestionMapping($qtiParser_import_mapping)
	{
		foreach($qtiParser_import_mapping as $question_ident => $pool_qst_ident)
		{
			$exp_ident = explode('_qst_', $question_ident);
			$old_qst_id = end($exp_ident);
			$new_qst_id = $pool_qst_ident['pool'];
			$this->fat_qst_mapping[$old_qst_id] = $new_qst_id; 
		}
	}

	/**
	 * @param $set_data
	 * @param $set_questions
	 * @param $set_paths
	 * @param $a_mapping
	 * @param $container_id   FormATest | QuestionSetPool
	 */
	public function importQuestionSet($set_data, $set_questions, $set_paths, $a_mapping, $container_id)
	{
		
		$container_obj_type = ilObject::_lookupType($container_id);
		
		$set = new ilQuestionSetPoolSet();
	
		$set->setContainerObjId($container_id);
		$set->setTitle($set_data['SetTitle']);
		$set->setDescription($set_data['SetDescription']);
		
//		$org_set_id  = $this->set_mapping[$set_data['SetId']] ? $this->set_mapping[$set_data['SetId']] : $set_data['OriginalSetId'];
		
		
		$set->setOriginalId($set_data['OriginalSetId']);

		$set->update();

		$this->addSetMapping($set_data['SetId'], $set->getId());

		foreach($set_questions as $set_question)
		{
			if($container_obj_type == 'xqsp')
			{
				$original_question_id = $a_mapping->getMapping('Modules/TestQuestionPool', 'quest', $set_question['QuestionOriginalId']);
				
				if($original_question_id)
				{
					if(!assQuestion::_questionExistsInPool($original_question_id)  )
					{
						$original_question_id = NULL;
					}
				}
				
				$copy_id = $set->getQuestionList()->import($this->fat_qst_mapping[$set_question['QuestionId']], $original_question_id);
				if($original_question_id == 0 || $this->is_single_import )
				{
					$this->fat_qst_mapping['new_fat'][$set_question['QuestionId']] = $copy_id;
					if($set_question['QuestionOriginalId'])
					{
						$a_mapping->addMapping('Plugins/xtsf', 'org_qst_id', $set_question['QuestionOriginalId'], $copy_id);
					}
				}
				else
				{
					$this->fat_qst_mapping['new_fat'][$set_question['QuestionId']] = $original_question_id;
				}
			}
			else if($container_obj_type == 'xtsf')
			{
				$original_question_id = $a_mapping->getMapping('Plugins/xtsf', 'org_qst_id', $set_question['QuestionOriginalId']);
				$copy_id = $set->getQuestionList()->import($this->fat_qst_mapping[$set_question['QuestionId']], $original_question_id);
			}
			$this->fat_qst_mapping[$set_question['QuestionId']] = $copy_id;
		}

		foreach($set_paths as $set_path)
		{
			$path = new ilQuestionSetPoolPath();
			$path->setSet($set);
			$path->setTitle($set_path[0]['PathTitle']);
			$path->import();

			$this->addPathMapping($set_data[0]['PathId'], $path->getId());

			$nodes = $set_path['nodes'];
			$this->importNodes($nodes, $path);
		}
	}
	
	/**
	 * @param array                 $nodes
	 * @param ilQuestionSetPoolPath $path
	 */
	protected function importNodes($nodes, ilQuestionSetPoolPath $path)
	{
		foreach($nodes as $node_data)
		{
			$node = new ilQuestionSetPoolNode();
			$node->setQuestion(assQuestion::_instantiateQuestion($this->fat_qst_mapping[$node_data[0]['NodeQuestionId']]));
			$node->setPath($path);
			$node->setInitialCondition($node_data[0]['NodeInitialCondition']);
			if(strlen($node_data[0]['NodeDominantScoring']))
			{
				$node->setDominantScoring($node_data[0]['NodeDominantScoring']);
			}
			$node->setTrueFeedback($node_data[0]['NodeTrueFeedback']);
			$node->setFalseFeedback($node_data[0]['NodeFalseFeedback']);
			
			$node->import();
			
			$this->addNodeMapping($node_data[0]['NodeId'], $node->getId());
			
			$jump_conditions = $node_data['jump_conditions'];
			if(count($jump_conditions) == 0)
			{
				continue 1;
			}
			$this->importJumpConditions($jump_conditions, $node);
		}
	}
	
	/**
	 * @param array                 $jump_conditions
	 * @param ilQuestionSetPoolNode $node
	 */
	protected function importJumpConditions($jump_conditions, ilQuestionSetPoolNode $node)
	{
		foreach($jump_conditions as $jump_condition_data)
		{
			$jump_con = new ilQuestionSetPoolJumpCondition();
			$jump_con->setNode($node);
			$jump_con->setQuestion(
				$this->fat_qst_mapping[$jump_condition_data['JumpConditionQuestionId']] ?
				assQuestion::_instantiateQuestion($this->fat_qst_mapping[$jump_condition_data['JumpConditionQuestionId']]):
				NULL
			);
			$jump_con->setJumpCondition($jump_condition_data['JumpConditionText']);
			if(strlen($jump_condition_data['JumpConditionRepetitions']))
			{
				$jump_con->setRepititions($jump_condition_data['JumpConditionRepetitions']);
			}

			$jump_con->setTrueFeedback($jump_condition_data['JumpConditionTrueFeedback']);
			$jump_con->setTrueJumpFeedback($jump_condition_data['JumpConditionTrueJumpFeedback']);
			$jump_con->setFalseFeedback($jump_condition_data['JumpConditionFalseFeedback']);
			
			$jump_con->import();
		}
	}
	
	/**
	 * @param $old_set_id
	 * @param $new_set_id
	 */
	protected function addSetMapping($old_set_id, $new_set_id)
	{
		if(!isset($this->set_mapping[$old_set_id]))
		{
			$this->set_mapping[$old_set_id] = $new_set_id;
		}
	}
	
	public function getSetMapping()
	{
		return $this->set_mapping;
	}
	
	/**
	 * @param $old_path_id
	 * @param $new_path_id
	 */
	protected function addPathMapping($old_path_id, $new_path_id)
	{
		$this->path_mapping[$old_path_id] = $new_path_id;
	}
	
	/**
	 * @param $old_node_id
	 * @param $new_node_id
	 */
	protected function addNodeMapping($old_node_id, $new_node_id)
	{
		$this->node_mapping[$old_node_id] = $new_node_id;
	}
	
	/**
	 * Import XML
	 * @param $a_entity
	 * @param $a_id
	 * @param $a_xml
	 * @param $a_mapping
	 * @return bool
	 */
	function importXmlRepresentation($a_entity, $a_id, $a_xml, $a_mapping)
	{
		/**
		 * @var $tree ilTree
		 */
		global $tree;

		$this->plugin_object->includeClass('class.ilObjFormATest.php');
		$this->plugin_object->includeClass('class.ilObjFormATestXMLParser.php');
		
		$parent_ref_id = 0;

		if($new_id = $a_mapping->getMapping('Services/Container', 'objs', $a_id))
		{
			$ref_ids = ilObject::_getAllReferences($new_id);
			$ref_id  = current($ref_ids);
			
			$parent_ref_id = $tree->getParentId($ref_id);
			
			$this->xtsf_obj = ilObjectFactory::getInstanceByObjId($new_id, false);
			$this->xtsf_obj->setRefId($ref_id);
			$this->is_single_import = false;
		}
		else
		{
			// das ist eigentlich immer der Fall ...? 
			
			$this->xtsf_obj = new ilObjFormaTest();
			$this->xtsf_obj->create(true);

			$this->is_single_import = true;
		}
		
		$this->current_parent_ref_id = $parent_ref_id;
		
		if(isset($_POST['xqsp_selection']))
		{
			// container_id ermittle Basis-Container für questionsets
			if($_POST['xqsp_selection'] == self::IMPORT_FAT_NO_POOLS)
			{
				
			}
			else if($_POST['xqsp_selection'] == self::IMPORT_INTO_SELECTED_POOLS) 
			{
				$this->import_into_xqsp = true;
			    if((int)$_POST['xqsp'] > 0)
				{
					// verwende existierenden QuestionSetPool
					$this->initializeXqspObject((int)$_POST['xqsp']);
				}
				else
				{
					// erstelle neuen QuestionSetPool
					$this->initializeXqspObject(-1);
					$this->xqsp_obj->createReference();
					$this->xqsp_obj->putInTree($a_mapping->getTargetId());
					$this->xqsp_obj->setPermissions($a_mapping->getTargetId());
				}
		
				if((int)$_POST['qpl'] == self::SINGLE_IMPORT_NO_POOL)
				{
					// kein QuestionPool verwenden 
					$this->import_into_qpl = false;
				}
				else if((int)$_POST['qpl'] > 0)
				{
					$this->import_into_qpl = true;
					// verwende existierenden QuestionPool
					$this->initializeQplObject((int)$_POST['qpl']);
				}
				else
				{
					$this->import_into_qpl = true;
					// erstelle neuen QuestionPool
					$this->initializeQplObject(self::SINGLE_IMPORT_NEW_POOL);
					$this->qpl_obj->createReference();
					$this->qpl_obj->putInTree($a_mapping->getTargetId());
					$this->qpl_obj->setPermissions($a_mapping->getTargetId());
				}
			}
		}	
		
		// import questions into selected main container
		$this->parseQtiFile();

		$parser = new ilObjFormATestXMLParser($this->xtsf_obj, $this->getXmlFile());
		$parser->setImportDirectory($this->getImportDirectory());
		$parser->startParsing();
		$this->xtsf_obj = $parser->getFormATestObject();
		$this->xtsf_obj->update();
		
		// create sets
		$sets = $parser->getSetsArray();
		
		if($this->import_into_xqsp)
		{
			$xqsp_importer = new ilQuestionSetPoolImporter();
			$xqsp_importer->xqsp_obj = $this->xqsp_obj;
			$xqsp_importer->setExternalQplQuestionMapping($this->fat_qst_mapping);
			foreach($sets as $set_data)
			{
				$xqsp_importer->importQuestionSet($set_data[0], $set_data['questions'],  $set_data['paths'], $a_mapping);
			}
			
			$this->xqsp_qst_mapping = $xqsp_importer->getExternalQplQuestionMapping();
			$xqsp_setlist = $this->xqsp_obj->getSetList();
			$set_data = $xqsp_setlist->getData();
			
			foreach($set_data as $item)
			{
				// Fragen in den FormATest clonen
				$this->xtsf_obj->getSetList()->insert(array($item->getId()));
			}
		}
		else 
		{
			foreach($sets as $set_data)
			{
				$this->importQuestionSet($set_data[0], $set_data['questions'],  $set_data['paths'], $a_mapping, $this->xtsf_obj->getId());
			}
			$xtsf_setlist = $this->xtsf_obj->getSetList();
			$xtsf_setlist->insert($this->getSetMapping(), false);
		}
		
		$a_mapping->addMapping('Plugins/xtsf', 'xtsf', $a_id, $this->xtsf_obj->getId());
	}
	
	function finalProcessing($a_mapping)
	{
		// Container import
		// Prüfen, ob in einer statischen Variable (z.B. self::$pool_by_parent[$this->current_parent_ref_id] schon eine Pool ID existiert
		// Wenn ja, diesen Pool nehmen (sofern Bedarf für neue Originale)
		// Wenn nein, neuen Pool auf dieser Ebene -> mit putInTree($this->current_parent_ref_id) einhängen,
		// und dann dessen  ID hier merken: self::$pool_by_parent[$this->current_parent_ref_id]
		
		// Einzel-Import mit (ggf. erfolgter Wahl eines bestehenden Fragenpools als Ziel für Originale unserer Fragen)
		
		if($this->import_into_xqsp)
		{
			// fragen wurden schon in einen xqsp importiert
			$org_question_mapping = $this->xqsp_qst_mapping['new_qpl'];
			
			// die obj-id muss vorher auf die xqsp obj-id gesetzt werden
			foreach($org_question_mapping as $new_id)
			{
				$source_qst = assQuestion::_instantiateQuestion($new_id);
				$source_qst->setObjId($this->xqsp_obj->getId());
				$source_qst->saveToDb();
				
				if($this->import_into_qpl)
				{
					// jetzt müssen sie in den neu angelegten dummy pool (QPL) kopiert werden und die org_id im xqsp-pool angepasst werden.
					$qst_container_obj_type = ilObject::_lookupType($source_qst->getObjId());
					$is_xqsp_pool           = ($qst_container_obj_type == 'xqsp') ? true : false;
					// This is the intended behaviour, see http://www.ilias.de/mantis/view.php?id=18325
					if(($source_qst->getOriginalId() == NULL || $this->is_single_import) && $is_xqsp_pool)
					{
						// Dies ist nur für Fragen "ohne pool" relevant!!!!
						$clone_id   = $source_qst->copyObject($this->qpl_obj->getId());
						$source_qst = assQuestion::_instantiateQuestion($new_id);
						$source_qst->setNewOriginalId($clone_id);
						$source_qst->saveToDb();
						
						$this->qpl_qst_mapping[$new_id] = $clone_id;
					}
				}
			}
		}
		
		return parent::finalProcessing($a_mapping);
	}
	
	/**
	 * Create qti and xml file name
	 * @return array 
	 */
	protected function parseXmlFileNames()
	{
		$GLOBALS['ilLog']->write(__METHOD__.': '.$this->getImportDirectory());
		
		$basename = basename($this->getImportDirectory());

		$xml = $this->getImportDirectory().'/'.$basename.'.xml';
		$qti = $this->getImportDirectory().'/'.preg_replace('/xqsp/', 'qti', $basename).'.xml';
		
		return array($xml,$qti);
	}

	private function getImportDirectoryBase()
	{
		$dir = $this->getImportDirectory();
		$dir = dirname($dir);
		return $dir;
	}
}
