<?php
/* Copyright (c) 1998-2016 ILIAS open source, Extended GPL, see docs/LICENSE */
/**
 * Class ilFormATestXmlValidator
 * author Nadia Matuschek <nmatuschek@databay.de>
 */
class ilFormATestXmlValidator
{
	/**
	 *
	 */
	const MODE_VALIDATION = 1;
	/**
	 *
	 */
	const XSD_FILE = './Customizing/global/plugins/Services/Repository/RepositoryObject/FormATest/xml/xtsf_5_1.xsd';
	/**
	 *
	 */
	const NAME_SPACE_URI = 'http://www.ilias.de/Customizing/global/plugins/Services/Repository/RepositoryObject/FormATest';
	/**
	 *
	 */
	const DOM_MISSING_NS_CODE = 1845;
	
	/**
	 * @var
	 */
	private $mode;
	/**
	 * @var string
	 */
	private $schema;
	/**
	 * @var
	 */
	private $xml;
	
	/**
	 * @var array
	 */
	private $assignments = array();
	/**
	 * @var array
	 */
	private $errors = array();
	/**
	 * @var bool
	 */
	private $has_errors = false;
	
	/**
	 * @var null
	 */
	protected $xml_file = NULL;
	
	/**
	 * @var null
	 */
	protected $xsd_file = NULL;
	
	/**
	 * @var string
	 */
	protected $error_message = '';
	
	/**
	 * ilQuestionSetPoolXmlValidator constructor.
	 */
	public function __construct()
	{
		$this->schema = self::XSD_FILE;
	}
	
	/**
	 * @return mixed
	 */
	public function getMode()
	{
		return $this->mode;
	}
	
	/**
	 * @param mixed $mode
	 */
	public function setMode($mode)
	{
		$this->mode = $mode;
	}
	
	/**
	 * @return string
	 */
	public function getSchema()
	{
		return $this->schema;
	}
	
	/**
	 * @param string $schema
	 */
	public function setSchema($schema)
	{
		$this->schema = $schema;
	}
	
	/**
	 * @return mixed
	 */
	public function getXml()
	{
		return $this->xml;
	}
	
	/**
	 * @param mixed $xml
	 */
	public function setXml($xml)
	{
		$this->xml = $xml;
	}
	
	/**
	 * @return array
	 */
	public function getAssignments()
	{
		return $this->assignments;
	}
	
	/**
	 * @param array $assignments
	 */
	public function setAssignments($assignments)
	{
		$this->assignments = $assignments;
	}
	
	/**
	 * @return array
	 */
	public function getErrors()
	{
		return $this->errors;
	}
	
	/**
	 * @param array $errors
	 */
	public function setErrors($errors)
	{
		$this->errors = $errors;
	}
	
	/**
	 * @return boolean
	 */
	public function isHasErrors()
	{
		return $this->has_errors;
	}
	
	/**
	 * @param boolean $has_errors
	 */
	public function setHasErrors($has_errors)
	{
		$this->has_errors = $has_errors;
	}
	
	/**
	 * @return null
	 */
	public function getXmlFile()
	{
		return $this->xml_file;
	}
	
	/**
	 * @param null $xml_file
	 */
	public function setXmlFile($xml_file)
	{
		$this->xml_file = $xml_file;
	}
	
	/**
	 * @return null
	 */
	public function getXsdFile()
	{
		return $this->xsd_file;
	}
	
	/**
	 * @param null $xsd_file
	 */
	public function setXsdFile($xsd_file)
	{
		$this->xsd_file = $xsd_file;
	}
	
	/**
	 * @return string
	 */
	public function getErrorMessage()
	{
		return $this->error_message;
	}
	
	/**
	 * @param string $error_message
	 */
	public function setErrorMessage($error_message)
	{
		$this->error_message = $error_message;
	}
	
	/**
	 * @return bool
	 */
	public function validate()
	{
		// validate against schema
		$this->setXml(@file_get_contents($this->getXmlFile()));
		
		if(!$this->validateXml($this->getXml(), false))
		{
			return true;
		}
		
		if(count($this->errors))
		{
			return false;
		}
		return true;
	}
	
	/**
	 * validate xml string
	 * @param string     $a_xml
	 * @param bool|false $a_add_default_namespace
	 * @return bool
	 */
	protected function validateXml($a_xml, $a_add_default_namespace = false)
	{
		ilLoggerFactory::getLogger('xtsf')->debug($a_xml);
		
		libxml_use_internal_errors(true);
		$dom  = new DOMDocument();
		$dom->loadXML($a_xml);
		$dom->schemaValidate($this->getSchema());
		
		foreach(libxml_get_errors() as $error)
		{
			ilLoggerFactory::getLogger('xtsf')->warning('XML schema validation failed: ' . $error->line.': '. $error->message);
			
			/* @var $error LibXMLError */
			if($error->code == self::DOM_MISSING_NS_CODE)
			{
				if(!$a_add_default_namespace)
				{
					// add default namespace to xml
					$xml = simplexml_load_string($a_xml);
					$xml->addAttribute('xmlns', self::NAME_SPACE_URI);
					$xml->saveXml();
					ilLoggerFactory::getLogger('xtsf')->debug('Starting second validation with default namespace');
					libxml_clear_errors();
					return $this->validateXml($xml->asXML(), true);
				}
			}
			else
			{
				$this->errors[] = $error->line.': '.$error->message;
			}
		}
		libxml_clear_errors();
		if(count($this->errors))
		{
			return false;
		}
		return true;
		
	}
}