<?php
/* Copyright (c) 1998-2013 ILIAS open source, Extended GPL, see docs/LICENSE */

require_once 'Services/Repository/classes/class.ilObjectPlugin.php';
require_once 'Services/Tracking/classes/class.ilLPStatus.php';
require_once 'Services/Tracking/interfaces/interface.ilLPStatusPlugin.php';
require_once 'models/class.ilFormATestSetList.php';
require_once 'utils/class.ilObjTestGateway.php';
require_once 'Modules/TestQuestionPool/interfaces/interface.ilQuestionEditingFormConsumer.php';
require_once 'models/class.ilFormATestParticipantsList.php';
require_once "models/interface.ilXTSFModelInterface.php";
require_once "Modules/Test/classes/class.ilObjAssessmentFolder.php";
require_once 'Modules/Test/classes/class.ilObjTestAccess.php';
require_once "Modules/Test/classes/class.ilObjTest.php";
require_once 'Modules/TestQuestionPool/classes/class.assQuestion.php';
require_once 'Services/Tracking/classes/class.ilLPStatusFactory.php';
require_once 'Modules/Test/interfaces/interface.ilMarkSchemaAware.php';
require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/FormATest/interfaces/interface.ilConditionEvaluationObserver.php";
require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/FormATest/interfaces/interface.ilFormATestFeedbackConstants.php";

/**
 * Class ilObjFormATest
 */
class ilObjFormATest extends ilObjectPlugin implements ilLPStatusPluginInterface, ilXTSFModelInterface, ilQuestionEditingFormConsumer, ilMarkSchemaAware, ilConditionEvaluationObserver, ilFormATestFeedbackConstants
{

	const SCORE_REPORTING_FINISHED = 1;
	const SCORE_REPORTING_ALWAYS = 2;
	const SCORE_REPORTING_DATE = 3;
	const SCORE_REPORTING_NEVER = 4;

	/**
	 * @var int
	 */
	const LP_MODE_DEACTIVATED = 0;

	/**
	 * @var int
	 */
	const LP_MODE_TEST_PASSED = 8;

	/**
	 * @var ilFormATestSetList
	 */
	protected $set_list;

	/**
	 * @var ilFormATestParticipantsList
	 */
	protected $participants;

	/**
	 * @var int
	 */
	protected $test_id;

	/**
	 * @var int
	 */
	protected $learning_progress_mode = self::LP_MODE_DEACTIVATED;

	/**
	 * @var ilDB
	 */
	protected $db;

	/**
	 * @var ilObjUser
	 */
	protected $user;

	/**
	 * @var ilLog
	 */
	protected $log;

	/**
	 * @var boolean
	 */
	protected $online;

	/**
	 * @var integer
	 */
	protected $max_passes;

	/**
	 * @var integer
	 */
	protected $process_time;

	/**
	 * @var boolean
	 */
	protected $allow_walk_back;

	/**
	 * @var boolean
	 */
	protected $show_current_track;

	/**
	 * @var boolean
	 */
	protected $show_questions_overview;

	/**
	 * @var boolean
	 */
	protected $allow_interruption;

	/**
	 * @var string
	 */
	protected $intro_statement;

	/**
	 * @var string
	 */
	protected $final_statement;

	/**
	 * @var boolean
	 */
	protected $show_final_statement;

	/**
	 * @var boolean
	 */
	protected $show_intro_statement;

	/**
	 * @var boolean
	 */
	protected $start_time_enabled;

	/**
	 * @var boolean
	 */
	protected $end_time_enabled;

	/**
	 * @var ilDateTime
	 */
	protected $start_time;

	/**
	 * @var ilDateTime
	 */
	protected $end_time;

	/**
	 * @var ilDateTime
	 */
	protected $reporting_date;

	/**
	 * @var int
	 */
	protected $score_reporting;

	/**
	 * @var array
	 */
	protected static $online_status_map = array();

	/**
	 * @var ASS_MarkSchema
	 */
	public $mark_schema;

	/** @var mixed availability of selector for special characters */
	protected $char_selector_availability;

	/** @var string definition of selector for special characters */
	protected $char_selector_definition;

	/**
	 * @var integer $answer_feedback_points 
	 */
	private $answer_feedback_points;
	/**
	 * @var integer $instant_verification
	 */
	private $instant_verification;
	/**
	 * @var integer $answer_feedback
	 */
	private $answer_feedback;
	/**
	 * @var integer $specific_answer_feedback
	 */
	private $specific_answer_feedback;

	/**
	 * @var integer $node_feedback_author
	 */
	private $node_feedback_author = 0;

	/**#
	 * @var integer $node_feedback_participant
	 */
	private $node_feedback_participant = 0;
	/**
	 * @var integer $node_feedback_participant_res
	 */
	private $node_feedback_participant_res = 0;

	/**
	 * @var bool
	 */
	private $skillServiceEnabled = false;

	/**
	 * @var null|bool
	 */
	private static $isSkillManagementGloballyActivated = null;
	
	/**
	 * @param int $ref_id
	 */
	public function __construct($ref_id = 0)
	{
		/**
		 * @var $ilDB   ilDB
		 * @var $ilLog  ilLog
		 * @var $ilUser ilObjUser
		 * @var $lng    ilLanguage
		 */
		global $ilDB, $ilLog, $ilUser, $lng;

		$this->db              = $ilDB;
		$this->log             = $ilLog;
		$this->user            = $ilUser;
		$this->score_reporting = self::SCORE_REPORTING_ALWAYS;
		$this->max_passes      = 0;

		require_once "./Modules/Test/classes/class.assMarkSchema.php";
		$lng->loadLanguageModule('assessment');
		$this->mark_schema = new ASS_MarkSchema();
		$this->mark_schema->createSimpleSchema(
			$lng->txt("failed_short"),
			$lng->txt("failed_official"),
			0,
			0,
			$lng->txt("passed_short"),
			$lng->txt("passed_official"),
			50,
			1
		);
		parent::__construct($ref_id);
	}

	/**
	 * @return string
	 */
	public function getQuestionEditingFormBackTargetLabel()
	{
		return $this->plugin->txt('back_to_question_set_qlist');
	}

	/**
	 * @param $context
	 * @return string
	 */
	public function getQuestionEditingFormBackTarget($context)
	{
		/**
		 * @var $ilCtrl ilCtrl
		 */
		global $ilCtrl;
		$params = @json_decode(base64_decode($context), true);

		$ilCtrl->setParameterByClass('ilObjPluginDispatchGUI', 'ref_id', $this->getRefId());
		$ilCtrl->setParameterByClass('ilObjPluginDispatchGUI', 'set_id', $_GET['set_id']);
		$url = $ilCtrl->getLinkTargetByClass(array('ilObjPluginDispatchGUI', 'ilObjFormATestGUI', 'ilObjFormATestQuestionSetPoolGUI'), '', '', false, false);
		foreach((array)$params as $key => $value)
		{
			$url = ilUtil::appendUrlParameterString($url, $key . '=' . $value);
		}

		return $url;
	}

	/**
	 * @return ilFormATestSetList
	 */
	public function getSetList()
	{
		return $this->set_list;
	}

	/**
	 * @return int
	 */
	public function getTestId()
	{
		return $this->test_id;
	}

	/**
	 * Returns the full name of a test user according to the anonymity status
	 * @param int     $user_id             The database ID of the user
	 * @param boolean $overwrite_anonymity Indicates if the anonymity status should be ignored
	 * @return string The full name of the user or UNKNOWN if the anonymity status is affected
	 * @access public
	 */
	public function userLookupFullName($user_id, $overwrite_anonymity = FALSE, $sorted_order = FALSE, $suffix = "")
	{
		$uname = ilObjUser::_lookupName($user_id);
		if(strlen($uname["firstname"] . $uname["lastname"]) == 0) $uname["firstname"] = $this->lng->txt("deleted_user");
		if($sorted_order)
		{
			return trim($uname["lastname"] . ", " . $uname["firstname"]) . $suffix;
		}
		else
		{
			return trim($uname["firstname"] . " " . $uname["lastname"]) . $suffix;
		}
	}

	/**
	 *
	 */
	protected function initType()
	{
		$this->setType('xtsf');
	}

	/**
	 * @throws ilException
	 */
	public function doCreate()
	{
		$this->set_list     = new ilFormATestSetList($this);
		$this->participants = new ilFormATestParticipantsList($this);

		try
		{
			$this->db->insert(
				'rep_robj_xtsf_data',
				array(
					'id'         => array('integer', $this->getId()),
					"max_passes" => array("integer", $this->getMaxPasses())
				)
			);

			$testId = $this->db->nextId('tst_tests');
			$this->db->insert(
				'tst_tests',
				array(
					'test_id'         => array('integer', $testId),
					'obj_fi'          => array('integer', $this->getId()),
					'author'          => array('integer', $this->user->getFirstname() . ' ' . $this->user->getLastname()),
					"score_reporting" => array("integer", self::SCORE_REPORTING_ALWAYS)
				)
			);

			$this->mark_schema->saveToDb($testId);

			$this->createMetaData();
			$this->logAction($this->plugin->txt("log_create_new_test"));
		}
		catch(Exception $e)
		{
			$this->log->write($e->getMessage());
			$this->log->logStack();

			$this->delete();

			// If you throw an exception here, ILIAS does not create an entry in object_data / object_reference / tree ...
			throw new ilException('<PLUGIN_LANGUAGE_VARIABLE>');
		}
	}

	/**
	 * Read data from db
	 */
	public function doRead()
	{
		$this->set_list     = new ilFormATestSetList($this);
		$this->participants = new ilFormATestParticipantsList($this);

		$res = $this->db->query('SELECT rep_robj_xtsf_data.*, tst_tests.* FROM rep_robj_xtsf_data INNER JOIN tst_tests ON obj_fi = id WHERE id = ' . $this->db->quote($this->getId(), 'integer'));
		$row = $this->db->fetchAssoc($res);

		$this->setLearningProgressMode($row['lp_mode']);
		$this->test_id = $row["test_id"];
		$this->setOnline($row["is_online"]);
		$this->setAllowInterruption($row["allow_interruption"]);
		$this->setAllowWalkBack($row["allow_walk_back"]);
		$this->setMaxPasses($row["max_passes"]);
		$this->setProcessTime($row["process_time"]);
		$this->setShowCurrentTrack($row["show_current_track"]);
		$this->setIntroStatement($row["intro_statement"]);
		$this->setFinalStatement($row["final_statement"]);
		$this->setShowIntroStatement($row["show_intro_statement"]);
		$this->setShowFinalStatement($row["show_final_statement"]);
		$this->setShowQuestionsOverview($row["show_questions_overview"]);
		$this->setScoreReporting($row["score_reporting"]);
		$this->setCharSelectorAvailability((int)$row['char_selector_availability']);
		$this->setCharSelectorDefinition($row['char_selector_definition']);
		$this->setSkillServiceEnabled($row['skill_service']);

		$this->setSpecificAnswerFeedback($row['specific_feedback']);
		$this->setGenericAnswerFeedback($row['answer_feedback']);
		$this->setAnswerFeedbackPoints($row['answer_feedback_points']);
		$this->setInstantFeedbackSolution($row['instant_verification']);
		
		$this->setNodeFeedbackAuthor($row['node_fb_author']);
		$this->setNodeFeedbackParticipant($row['node_fb_part']);
		$this->setNodeFeedbackParticipantRes($row['node_fb_part_res']);

		if($row["starting_time"])
		{
			$this->setStartTime(new ilDateTime($row["starting_time"], IL_CAL_TIMESTAMP));
			$this->setStartTimeEnabled(true);
		}
		else
		{
			$this->setStartTime(null);
			$this->setStartTimeEnabled(false);
		}
		if($row["ending_time"])
		{
			$this->setEndTime(new ilDateTime($row["ending_time"], IL_CAL_TIMESTAMP));
			$this->setEndTimeEnabled(true);
		}
		else
		{
			$this->setEndTime(null);
			$this->setEndTimeEnabled(false);
		}
		if($row["reporting_date"])
		{
			$this->setReportingDate(new ilDateTime($row["reporting_date"], IL_CAL_TIMESTAMP));
		}
		else
		{
			$this->setReportingDate(null);
		}

		$this->mark_schema->flush();
		$this->mark_schema->loadFromDb($this->test_id);
	}

	/**
	 * Update data
	 */
	public function doUpdate()
	{
		$this->db->update(
			"tst_tests",
			array(
				"starting_time"              => array("text", ilFormat::dateDB2timestamp($this->getStartTime())),
				"ending_time"                => array("text", ilFormat::dateDB2timestamp($this->getEndTime())),
				"reporting_date"             => array("text", ilFormat::dateDB2timestamp($this->getReportingDate())),
				"score_reporting"            => array("integer", $this->getScoreReporting()),
				'char_selector_availability' => array('integer', (int)$this->getCharSelectorAvailability()),
				'char_selector_definition'   => array('text', (string)$this->getCharSelectorDefinition()),
				'specific_feedback'          => array('integer', (int)$this->getSpecificAnswerFeedback()),
				'answer_feedback'            => array('text', $this->getGenericAnswerFeedback()),
				'answer_feedback_points'     => array('text', $this->getAnswerFeedbackPoints()),
				'instant_verification'       => array('text', $this->getInstantFeedbackSolution()),
				'skill_service'              => array('integer', $this->isSkillServiceEnabled())
			),
			array(
				'obj_fi' => array('integer', $this->getId())
			)
		);
		
		$this->db->update(
			'rep_robj_xtsf_data',
			array(
				'lp_mode'                 => array('integer', $this->getLearningProgressMode()),
				"is_online"               => array("integer", $this->isOnline()),
				"allow_walk_back"         => array("integer", $this->isAllowWalkBack()),
				"allow_interruption"      => array("integer", $this->isAllowInterruption()),
				"show_current_track"      => array("integer", $this->getShowCurrentTrack()),
				"max_passes"              => array("integer", $this->getMaxPasses()),
				"process_time"            => array("integer", $this->getProcessTime()),
				"intro_statement"         => array("text", $this->getIntroStatement()),
				"final_statement"         => array("text", $this->getFinalStatement()),
				"show_intro_statement"    => array("integer", $this->getShowIntroStatement()),
				"show_final_statement"    => array("integer", $this->getShowFinalStatement()),
				"show_questions_overview" => array("integer", $this->getShowQuestionsOverview()),
				"node_fb_author"		  => array("integer", $this->getNodeFeedbackAuthor()),
				"node_fb_part"		  	  => array("integer", $this->getNodeFeedbackParticipant()),
				"node_fb_part_res"		  	  => array("integer", $this->getNodeFeedbackParticipantRes())
			),
			array(
				'id' => array('integer', $this->getId())
			)
		);

		$this->updateMetaData();
		$this->logAction($this->plugin->txt("log_modified_test"));
	}

	/**
	 * @return bool
	 */
	public function beforeDelete()
	{
		$this->getParticipants()->removeAll();
		$this->getSetList()->removeAll();

		return true;
	}

	/**
	 * Delete data from database
	 */
	public function doDelete()
	{
		try
		{
			$this->db->manipulate('DELETE FROM rep_robj_xtsf_data WHERE id = ' . $this->db->quote($this->getId(), 'integer'));
			$this->db->manipulate('DELETE FROM tst_tests WHERE obj_fi = ' . $this->db->quote($this->getId(), 'integer'));
			$this->mark_schema->flush();
			$this->mark_schema->saveToDb($this->getTestId());
		}
		catch(Exception $e)
		{
			$this->log->write($e->getMessage());
			$this->log->logStack();
		}
	}

	/**
	 * @param ilObjFormATest $new_obj
	 * @param int            $a_target_id
	 * @param int            $a_copy_id
	 */
	public function doCloneObject(ilObjFormATest $new_obj, $a_target_id, $a_copy_id)
	{
		$res = $this->db->queryF(
			'SELECT * FROM rep_robj_xtsf_data WHERE id = %s',
			array('integer'),
			array($this->getId())
		);
		$row = $this->db->fetchAssoc($res);

		try
		{
			$res_tmp = $this->db->queryF('SELECT * FROM rep_robj_xtsf_data WHERE id = %s',
				array('integer'), array($new_obj->getId()));
			
			$numrows = $this->db->numRows($res_tmp);
			if($numrows == 1)
			{
				$this->db->update(
					'rep_robj_xtsf_data',
					array(
						'lp_mode'                 => array('integer', $this->getLearningProgressMode()),
						"is_online"               => array("integer", $this->isOnline()),
						"allow_walk_back"         => array("integer", $this->isAllowWalkBack()),
						"allow_interruption"      => array("integer", $this->isAllowInterruption()),
						"show_current_track"      => array("integer", $this->getShowCurrentTrack()),
						"max_passes"              => array("integer", $this->getMaxPasses()),
						"process_time"            => array("integer", $this->getProcessTime()),
						"intro_statement"         => array("text", $this->getIntroStatement()),
						"final_statement"         => array("text", $this->getFinalStatement()),
						"show_intro_statement"    => array("integer", $this->getShowIntroStatement()),
						"show_final_statement"    => array("integer", $this->getShowFinalStatement()),
						"show_questions_overview" => array("integer", $this->getShowQuestionsOverview()),
						"node_fb_author"		  => array("integer", $this->getNodeFeedbackAuthor()),
						"node_fb_part"		  	  => array("integer", $this->getNodeFeedbackParticipant()),
						"node_fb_part_res"		  	  => array("integer", $this->getNodeFeedbackParticipantRes())
					),
					array('id' => array('integer', $new_obj->getId()))
				);
			}	
			else if($numrows == 0)
			{
				$this->db->insert(
					'rep_robj_xtsf_data',
					array(
						'id'                      => array('integer', $new_obj->getId()),
						'lp_mode'                 => array('integer', $row['lp_mode']),
						"is_online"               => array("integer", $row['is_online']),
						"allow_walk_back"         => array("integer", $row['allow_walk_back']),
						"allow_interruption"      => array("integer", $row['allow_interruption']),
						"show_current_track"      => array("integer", $row['show_current_track']),
						"max_passes"              => array("integer", $row['max_passes']),
						"process_time"            => array("integer", $row['process_time']),
						"intro_statement"         => array("text", $row['intro_statement']),
						"final_statement"         => array("text", $row['final_statement']),
						"show_intro_statement"    => array("integer", $row['show_intro_statement']),
						"show_final_statement"    => array("integer", $row['show_final_statement']),
						"show_questions_overview" => array("integer", $this->getShowQuestionsOverview())
					)
				);
			}

			$res = $this->db->queryF(
				'SELECT * FROM tst_tests WHERE obj_fi = %s',
				array('integer'),
				array($this->getId())
			);
			$row = $this->db->fetchAssoc($res);

			$testId = $this->db->nextId('tst_tests');
			$this->db->insert(
				'tst_tests',
				array(
					'test_id'                    => array('integer', $testId),
					'obj_fi'                     => array('integer', $new_obj->getId()),
					'author'                     => array('integer', $this->user->getFirstname() . ' ' . $this->user->getLastname()),
					"starting_time"              => array("text", $row['starting_time']),
					"ending_time"                => array("text", $row['ending_time']),
					"reporting_date	"         => array("text", $row['reporting_date']),
					"score_reporting"            => array("text", $row['score_reporting']),
					'char_selector_availability' => array('integer', (int)$this->getCharSelectorAvailability()),
					'char_selector_definition'   => array('text', (string)$this->getCharSelectorDefinition()),

					'specific_feedback'          => array('integer', (int)$this->getSpecificAnswerFeedback()),
					'answer_feedback'            => array('text', $this->getGenericAnswerFeedback()),
					'answer_feedback_points'     => array('text', $this->getAnswerFeedbackPoints()),
					'instant_verification'       => array('text', $this->getInstantFeedbackSolution()),
					'skill_service'              => array('integer', $this->isSkillServiceEnabled())
				)
			);

			$set_data = $this->getSetList()->getData();
			
			foreach($set_data as $item)
			{
				$new_obj->getSetList()->insert(array($item["id"]));
			}

			// @todo: Clone mark schema
			$this->cloneMetaData($new_obj);
		}
		catch(Exception $e)
		{
			//@todo: Handle Exception
			$this->log->write($e->getMessage());
			$this->log->logStack();
		}
	}

	/**
	 * Get all user ids with LP status completed
	 * @return array
	 */
	public function getLPCompleted()
	{
		if(in_array($this->getLearningProgressMode(), array(self::LP_MODE_DEACTIVATED)))
		{
			return array();
		}

		$user_ids     = array();
		$passed_users = ilObjTestAccess::_getPassedUsers($this->getId());
		foreach((array)$passed_users as $user_data)
		{
			if($user_data['passed'])
			{
				$user_ids[] = $user_data['user_id'];
			}
		}
		return $user_ids;
	}

	/**
	 * Get all user ids with LP status not attempted
	 * @return array
	 */
	public function getLPNotAttempted()
	{
		return array();
	}

	/**
	 * Get all user ids with LP status failed
	 * @return array
	 */
	public function getLPFailed()
	{
		if(in_array($this->getLearningProgressMode(), array(self::LP_MODE_DEACTIVATED)))
		{
			return array();
		}

		$user_ids     = array();
		$passed_users = ilObjTestAccess::_getPassedUsers($this->getId());
		foreach((array)$passed_users as $user_data)
		{
			if($user_data['failed'])
			{
				$user_ids[] = $user_data['user_id'];
			}
		}
		return $user_ids;
	}

	/**
	 * Get all user ids with LP status in progress
	 * @return array
	 */
	public function getLPInProgress()
	{
		if(in_array($this->getLearningProgressMode(), array(self::LP_MODE_DEACTIVATED)))
		{
			return array();
		}

		require_once 'Modules/Test/classes/class.ilObjTestAccess.php';
		$test_fi = ilObjTestAccess::_getTestIDFromObjectID($this->getId());
		$query   = 'SELECT DISTINCT(user_fi) FROM tst_active WHERE test_fi = ' . $this->db->quote($test_fi, 'integer');

		$user_ids = array();
		$res      = $this->db->query($query);
		while($row = $this->db->fetchAssoc($res))
		{
			$user_ids[] = $row['user_fi'];
		}

		$users = array_diff((array)$user_ids, $this->getLPCompleted());
		$users = array_diff((array)$users, $this->getLPFailed());
		return $users ? $users : array();
	}

	/**
	 * Get current status for given user
	 * @param int $a_user_id
	 * @return int
	 */
	public function getLPStatusForUser($a_user_id)
	{
		$status = ilLPStatus::LP_STATUS_NOT_ATTEMPTED_NUM;

		$res = $this->db->query(
			'SELECT tries
			FROM tst_active
			WHERE user_fi = ' . $this->db->quote($a_user_id, 'integer') . ' AND test_fi = ' . $this->db->quote(ilObjTestAccess::_getTestIDFromObjectID($this->getId()), 'integer')
		);

		assQuestion::setResultGateway(new ilObjTestGateway());
		if($row = $this->db->fetchAssoc($res))
		{
			if(ilObjTestAccess::_isPassed($a_user_id, $this->getId()))
			{
				$status = ilLPStatus::LP_STATUS_COMPLETED_NUM;
			}
			else
			{
				$status = ilLPStatus::LP_STATUS_FAILED_NUM;
			}
		}
		assQuestion::setResultGateway(null);
		return $status;
	}

	/**
	 * @param int $a_user_id
	 * @return int
	 */
	public function getPercentageForUser($a_user_id)
	{
		/**
		 * @var $ilDB iLDB
		 */
		global $ilDB;

		$set = $ilDB->query(
			"SELECT tst_result_cache.*, tst_active.user_fi FROM " .
			"tst_result_cache JOIN tst_active ON (tst_active.active_id = tst_result_cache.active_fi)" .
			" JOIN tst_tests ON (tst_tests.test_id = tst_active.test_fi) " .
			" WHERE tst_tests.obj_fi = " . $ilDB->quote($this->getId(), "integer") .
			" AND tst_active.user_fi = " . $ilDB->quote($a_user_id, "integer"));
		$per = 0;
		if($rec = $ilDB->fetchAssoc($set))
		{
			if($rec["max_points"] > 0)
			{
				$per = min(100, 100 / $rec["max_points"] * $rec["reached_points"]);
			}
			else
			{
				// According to mantis #12305
				$per = 0;
			}
		}
		return (int)$per;
	}

	/**
	 * @param int $learning_progress_mode
	 */
	public function setLearningProgressMode($learning_progress_mode)
	{
		$this->learning_progress_mode = $learning_progress_mode;
	}

	/**
	 * @return int
	 */
	public function getLearningProgressMode()
	{
		return $this->learning_progress_mode;
	}

	/**
	 * @return array
	 */
	public function getLPValidModes()
	{
		return array(
			self::LP_MODE_DEACTIVATED,
			self::LP_MODE_TEST_PASSED
		);
	}

	/**
	 * @return int
	 */
	public function getDefaultMode()
	{
		return self::LP_MODE_DEACTIVATED;
	}

	/**
	 * @param boolean $allow_interruption
	 */
	public function setAllowInterruption($allow_interruption)
	{
		$this->allow_interruption = $allow_interruption;
	}

	/**
	 * @return boolean
	 */
	public function isAllowInterruption()
	{
		return $this->allow_interruption;
	}

	/**
	 * @param boolean $allow_walk_back
	 */
	public function setAllowWalkBack($allow_walk_back)
	{
		$this->allow_walk_back = $allow_walk_back;
	}

	/**
	 * @return boolean
	 */
	public function isAllowWalkBack()
	{
		return $this->allow_walk_back;
	}

	/**
	 * @param int $max_passes
	 */
	public function setMaxPasses($max_passes)
	{
		$this->max_passes = $max_passes;
	}

	/**
	 * @return int
	 */
	public function getMaxPasses()
	{
		return $this->max_passes;
	}

	/**
	 * @param boolean $online
	 */
	public function setOnline($online)
	{
		$this->online = $online;
	}

	/**
	 * @return boolean
	 */
	public function isOnline()
	{
		return $this->online;
	}

	/**
	 * @param int $process_time
	 */
	public function setProcessTime($process_time)
	{
		$this->process_time = $process_time;
	}

	/**
	 * @return int
	 */
	public function getProcessTime()
	{
		return $this->process_time;
	}

	/**
	 * @param boolean $show_current_track
	 */
	public function setShowCurrentTrack($show_current_track)
	{
		$this->show_current_track = $show_current_track;
	}

	/**
	 * @return boolean
	 */
	public function getShowCurrentTrack()
	{
		return $this->show_current_track;
	}

	/**
	 * @param string $final_statement
	 */
	public function setFinalStatement($final_statement)
	{
		$this->final_statement = $final_statement;
	}

	/**
	 * @return string
	 */
	public function getFinalStatement()
	{
		return $this->final_statement;
	}

	/**
	 * @param string $intro_statement
	 */
	public function setIntroStatement($intro_statement)
	{
		$this->intro_statement = $intro_statement;
	}

	/**
	 * @return string
	 */
	public function getIntroStatement()
	{
		return $this->intro_statement;
	}

	/**
	 * @param boolean $show_final_statement
	 */
	public function setShowFinalStatement($show_final_statement)
	{
		$this->show_final_statement = $show_final_statement;
	}

	/**
	 * @return boolean
	 */
	public function getShowFinalStatement()
	{
		return $this->show_final_statement;
	}

	/**
	 * @param boolean $show_intro_statement
	 */
	public function setShowIntroStatement($show_intro_statement)
	{
		$this->show_intro_statement = $show_intro_statement;
	}

	/**
	 * @return boolean
	 */
	public function getShowIntroStatement()
	{
		return $this->show_intro_statement;
	}

	/**
	 * @param ilDateTime $end_time
	 */
	public function setEndTime($end_time)
	{
		$this->end_time = $end_time;
	}

	/**
	 * @return ilDateTime
	 */
	public function getEndTime()
	{
		return $this->end_time;
	}

	/**
	 * @param boolean $end_time_enabled
	 */
	public function setEndTimeEnabled($end_time_enabled)
	{
		$this->end_time_enabled = $end_time_enabled;
	}

	/**
	 * @return boolean
	 */
	public function isEndTimeEnabled()
	{
		return $this->end_time_enabled;
	}

	/**
	 * @param ilDateTime $start_time
	 */
	public function setStartTime($start_time)
	{
		$this->start_time = $start_time;
	}

	/**
	 * @return ilDateTime
	 */
	public function getStartTime()
	{
		return $this->start_time;
	}

	/**
	 * @param boolean $start_time_enabled
	 */
	public function setStartTimeEnabled($start_time_enabled)
	{
		$this->start_time_enabled = $start_time_enabled;
	}

	/**
	 * @return boolean
	 */
	public function isStartTimeEnabled()
	{
		return $this->start_time_enabled;
	}

	/**
	 * @return \ilFormATestParticipantsList
	 */
	public function getParticipants()
	{
		return $this->participants;
	}

	/**
	 * @param boolean $show_questions_overview
	 */
	public function setShowQuestionsOverview($show_questions_overview)
	{
		$this->show_questions_overview = $show_questions_overview;
	}

	/**
	 * @return boolean
	 */
	public function getShowQuestionsOverview()
	{
		return $this->show_questions_overview;
	}

	/**
	 * @param int $score_reporting
	 */
	public function setScoreReporting($score_reporting)
	{
		$this->score_reporting = $score_reporting;
	}

	/**
	 * @return int
	 */
	public function getScoreReporting()
	{
		return $this->score_reporting;
	}

	/**
	 * @param \ilDateTime $reporting_date
	 */
	public function setReportingDate($reporting_date)
	{
		$this->reporting_date = $reporting_date;
	}

	/**
	 * @return \ilDateTime
	 */
	public function getReportingDate()
	{
		return $this->reporting_date;
	}

	/**
	 * @param int availability of the special character selector
	 */
	public function setCharSelectorAvailability($availability)
	{
		$this->char_selector_availability = (int)$availability;
	}

	/**
	 * @return int    availability of the special character selector
	 */
	public function getCharSelectorAvailability()
	{
		return (int)$this->char_selector_availability;
	}

	/**
	 * @param string    definition of the special character selector
	 */
	public function setCharSelectorDefinition($definition = '')
	{
		$this->char_selector_definition = $definition;
	}

	/**
	 * @return string    definition of the special character selector
	 */
	public function getCharSelectorDefinition()
	{
		return $this->char_selector_definition;
	}

	/**
	 * @return array
	 */
	public function toArray()
	{
		require_once 'Services/UIComponent/CharSelector/classes/class.ilCharSelectorGUI.php';
		$char_selector = new ilCharSelectorGUI(ilCharSelectorConfig::CONTEXT_TEST);
		$char_selector->getConfig()->setDefinition($this->getCharSelectorDefinition());
		$values = array(
			"title"                      => $this->getTitle(),
			"description"                => $this->getDescription(),
			"online"                     => $this->isOnline(),
			"allow_walk_back"            => $this->isAllowWalkBack(),
			"allow_interruption"         => $this->isAllowInterruption(),
			"max_passes"                 => $this->getMaxPasses(),
			"process_time"               => $this->getProcessTime(),
			"show_current_track"         => $this->getShowCurrentTrack(),
			"intro_statement"            => $this->getIntroStatement(),
			"final_statement"            => $this->getFinalStatement(),
			"show_intro_statement"       => $this->getShowIntroStatement(),
			"show_final_statement"       => $this->getShowFinalStatement(),
			"enable_start_time"          => $this->isStartTimeEnabled(),
			"enable_end_time"            => $this->isEndTimeEnabled(),
			"show_questions_overview"    => $this->getShowQuestionsOverview(),
			"score_reporting"            => $this->getScoreReporting(),
			'char_selector_availability' => (int)$this->getCharSelectorAvailability(),
			'char_selector_blocks' => $char_selector->getConfig()->getAddedBlocks(),
			'char_selector_custom_items' => $char_selector->getConfig()->getCustomItems(),
			'skill_service'        => $this->isSkillServiceEnabled(),
		);

		if($this->getAnswerFeedbackPoints())
		{	
			$values['instant_feedback'][] = 'instant_feedback_points';
		}
		if($this->getGenericAnswerFeedback())
		{
			$values['instant_feedback'][] = 'instant_feedback_generic' ;
		}
		if($this->getInstantFeedbackSolution())
		{
			$values['instant_feedback'][] = 'instant_feedback_solution';
		}
		if($this->getSpecificAnswerFeedback())
		{
			$values['instant_feedback'][] = 'instant_feedback_specific';
		}
		
		if($this->getNodeFeedbackAuthor())
		{
			$values['node_feedback'][] ='node_feedback_author'; 
		}
		if($this->getNodeFeedbackParticipant())
		{
			$values['node_feedback'][] ='node_feedback_part';
		}

		if($this->getNodeFeedbackParticipantRes())
		{
			$values['node_feedback'][] ='node_feedback_part_res';
		}
		
		if($this->getStartTime() != null)
		{
			$date                 = explode(' ', $this->getStartTime()->get(IL_CAL_DATETIME));
			$values["start_time"] = array(
				"date" => $date[0],
				"time" => $date[1]
			);
		}
		if($this->getEndTime() != null)
		{
			$date               = explode(' ', $this->getEndTime()->get(IL_CAL_DATETIME));
			$values["end_time"] = array(
				"date" => $date[0],
				"time" => $date[1]
			);
		}
		if($this->getReportingDate() != null)
		{
			$date                     = explode(' ', $this->getReportingDate()->get(IL_CAL_DATETIME));
			$values["reporting_date"] = array(
				"date" => $date[0],
				"time" => $date[1]
			);
		}

		return $values;
	}

	/**
	 * @param ilPropertyFormGUI $form
	 */
	public function bindForm(ilPropertyFormGUI $form)
	{
		if($form->getItemByPostVar("title"))
		{
			$this->setTitle($form->getInput('title'));
			$this->setDescription($form->getInput('description'));
			$this->setOnline($form->getInput('online'));
			$this->setMaxPasses($form->getInput('max_passes'));
			$this->setProcessTime($form->getInput('process_time'));
			$this->setAllowInterruption($form->getInput('allow_interruption'));
			$this->setAllowWalkBack($form->getInput('allow_walk_back'));
			$this->setIntroStatement($form->getInput('intro_statement'));
			$this->setFinalStatement($form->getInput('final_statement'));
			$this->setShowFinalStatement((bool)$form->getInput('show_final_statement'));
			$this->setShowIntroStatement((bool)$form->getInput('show_intro_statement'));
			$this->setSkillServiceEnabled((bool)$form->getInput('skill_service'));

			// Selector for uicode characters
			global $ilSetting;
			if($ilSetting->get('char_selector_availability') > 0)
			{
				require_once 'Services/UIComponent/CharSelector/classes/class.ilCharSelectorGUI.php';
				$char_selector = new ilCharSelectorGUI(ilCharSelectorConfig::CONTEXT_TEST);
				$char_selector->addFormProperties($form);
				$char_selector->getFormValues($form);
				$this->setCharSelectorAvailability($char_selector->getConfig()->getAvailability());
				$this->setCharSelectorDefinition($char_selector->getConfig()->getDefinition());
			}
		}
		
		$options = array();
		if(is_array( $form->getInput('instant_feedback')))
		{
			$options = $form->getInput('instant_feedback');
		}
		$this->setScoringFeedbackOptionsByArray($options);

		$node_options = array();
		if(is_array($form->getInput('node_feedback')))
		{
			$node_options = $form->getInput('node_feedback');
		}
		$this->setNodeFeedbackOptionsByArray($node_options);
		
		if($form->getItemByPostVar("show_current_track"))
		{
			$this->setShowCurrentTrack($form->getInput('show_current_track'));
		}
		if($form->getItemByPostVar("show_questions_overview"))
		{
			$this->setShowQuestionsOverview($form->getInput('show_questions_overview'));
		}

		if($form->getItemByPostVar("enable_start_time"))
		{
			$this->setStartTimeEnabled($form->getInput("enable_start_time"));
			if($form->getInput("enable_start_time"))
			{
				$this->setStartTime($form->getItemByPostVar('start_time')->getDate());
			}
			else
			{
				$this->setStartTime(null);
			}
		}

		if($form->getItemByPostVar("enable_end_time"))
		{
			$this->setEndTimeEnabled($form->getInput("enable_end_time"));
			if($form->getInput("enable_end_time"))
			{
				$this->setEndTime($form->getItemByPostVar('end_time')->getDate());
			}
			else
			{
				$this->setEndTime(null);
			}
		}

		if($form->getItemByPostVar("reporting_date"))
		{
			if($form->getInput("reporting_date"))
			{
				$this->setReportingDate($form->getItemByPostVar('reporting_date')->getDate());
			}
			else
			{
				$this->setReportingDate(null);
			}
		}

		if($form->getItemByPostVar("score_reporting"))
		{
			$this->setScoreReporting($form->getInput("score_reporting"));
		}
	}

	/**
	 * @param int $obj_id
	 * @return bool
	 */
	public static function getOnlineStatusByObjId($obj_id)
	{
		if(!isset(self::$online_status_map[$obj_id]))
		{
			self::preloadDataForLists(array($obj_id));
		}

		return (bool)self::$online_status_map[$obj_id];

	}

	/**
	 * @param array $obj_ids
	 * @param array $ref_ids
	 */
	public static function preloadDataForLists(array $obj_ids, array $ref_ids = array())
	{
		/**
		 * @var $ilDB iLDB
		 */
		global $ilDB;

		if(count($obj_ids) == 1)
		{
			$in = ' id = ' . $ilDB->quote(current($obj_ids), 'integer') . ' ';
		}
		else
		{
			$in = $ilDB->in('id', $obj_ids, false, 'integer');
		}

		$query = 'SELECT id, is_online FROM rep_robj_xtsf_data WHERE ' . $in;
		$res   = $ilDB->query($query);

		while($row = $ilDB->fetchAssoc($res))
		{
			self::$online_status_map[$row['id']] = $row['is_online'];
		}
	}

	/**
	 * @param string     $logtext
	 * @param string|int $set_id
	 */
	public function logAction($logtext = "", $set_id = "")
	{
		if(ilObjAssessmentFolder::_enabledAssessmentLogging())
		{
			global $ilUser;

			$original_id = "";
			if(strcmp($set_id, "") != 0)
			{
				include_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/classes/models/class.ilQuestionSetPoolSet.php";
				$set = new ilQuestionSetPoolSet($set_id);
				$set->read();
				$original_id = $set->getOriginalId();
			}
			ilObjAssessmentFolder::_addLog($ilUser->getId(), $this->getId(), $logtext, $set_id, $original_id, TRUE, $this->getRefId());
		}
	}

	/**
	 * @return bool
	 */
	final public function isComplete()
	{
		if(!count($this->mark_schema->mark_steps))
		{
			return false;
		}

		return count((array)$this->getSetList()->getData()) > 0;
	}

	public function setSpecificAnswerFeedback($specific_answer_feedback)
	{
		switch($specific_answer_feedback)
		{
			case 1:
				$this->specific_answer_feedback = 1;
				break;
			default:
				$this->specific_answer_feedback = 0;
				break;
		}
	}

	public function getSpecificAnswerFeedback()
	{
		switch($this->specific_answer_feedback)
		{
			case 1:
				return 1;
			default:
				return 0;
		}
	}

	function setGenericAnswerFeedback($generic_answer_feedback = 0)
	{
		switch($generic_answer_feedback)
		{
			case 1:
				$this->answer_feedback = 1;
				break;
			default:
				$this->answer_feedback = 0;
				break;
		}
	}

	public function getGenericAnswerFeedback()
	{
		return ($this->answer_feedback) ? $this->answer_feedback : 0;
	}

	function setAnswerFeedbackPoints($answer_feedback_points = 0)
	{
		switch($answer_feedback_points)
		{
			case 1:
				$this->answer_feedback_points = 1;
				break;
			default:
				$this->answer_feedback_points = 0;
				break;
		}
	}

	function getAnswerFeedbackPoints()
	{
		return ($this->answer_feedback_points) ? $this->answer_feedback_points : 0;
	}

	function setInstantFeedbackSolution($instant_feedback = 0)
	{
		switch($instant_feedback)
		{
			case 1:
				$this->instant_verification = 1;
				break;
			default:
				$this->instant_verification = 0;
				break;
		}
	}

	function getInstantFeedbackSolution()
	{
		return ($this->instant_verification) ? $this->instant_verification : 0;
	}

	public function setScoringFeedbackOptionsByArray($options)
	{
		if (is_array($options))
		{
			$this->setGenericAnswerFeedback(	in_array('instant_feedback_generic',  $options) ? 1 : 0);
			$this->setSpecificAnswerFeedback(	in_array('instant_feedback_specific', $options) ? 1 : 0);
			$this->setAnswerFeedbackPoints(		in_array('instant_feedback_points',   $options) ? 1 : 0);
			$this->setInstantFeedbackSolution(	in_array('instant_feedback_solution', $options) ? 1 : 0);
		}
		else
		{
			$this->setGenericAnswerFeedback(0);
			$this->setSpecificAnswerFeedback(0);
			$this->setAnswerFeedbackPoints(0);
			$this->setInstantFeedbackSolution(0);
		}
	}

	public function getInstantFeedbackSessionKey()
	{
		return 'xtsf_' . $this->getId() . '_is_instant_fb_requested';
	}

	/**
	 * @return ASS_MarkSchema
	 */
	public function getMarkSchema()
	{
		return $this->mark_schema;
	}

	/**
	 * @return boolean|string True or an error string which can be used for display purposes
	 */
	public function checkMarks()
	{
		return $this->mark_schema->checkMarks();
	}

	/**
	 * @return boolean
	 */
	public function canEditMarks()
	{
		return !$this->getParticipants()->hasParticipants();
	}

	/**
	 * @return int
	 */
	public function getMarkSchemaForeignId()
	{
		return $this->getTestId();
	}

	/**
	 *
	 */
	public function onMarkSchemaSaved()
	{
	}

	/**
	 * @return int
	 */
	public function getNodeFeedbackAuthor()
	{
		return (int)$this->node_feedback_author;
	}

	/**
	 * @param int $node_feedback_author
	 */
	public function setNodeFeedbackAuthor($node_feedback_author)
	{
		$this->node_feedback_author = $node_feedback_author;
	}

	/**
	 * @return int
	 */
	public function getNodeFeedbackParticipant()
	{
		return (int)$this->node_feedback_participant;
	}

	/**
	 * @param int $node_feedback_participants
	 */
	public function setNodeFeedbackParticipant($node_feedback_participant)
	{
		$this->node_feedback_participant = (int)$node_feedback_participant;
	}

	/**
	 * @return int
	 */
	public function getNodeFeedbackParticipantRes()
	{
		return $this->node_feedback_participant_res;
	}

	/**
	 * @param int $node_feedback_participant_res
	 */
	public function setNodeFeedbackParticipantRes($node_feedback_participant_res)
	{
		$this->node_feedback_participant_res = $node_feedback_participant_res;
	}
	
	/**
	 * @param $node_options
	 */
	protected function setNodeFeedbackOptionsByArray($node_options)
	{
		if (is_array($node_options))
		{
			$this->setNodeFeedbackAuthor(	in_array('node_feedback_author',  $node_options) ? 1 : 0);
			$this->setNodeFeedbackParticipant(	in_array('node_feedback_part', $node_options) ? 1 : 0);
			$this->setNodeFeedbackParticipantRes(	in_array('node_feedback_part_res', $node_options) ? 1 : 0);
		}
		else
		{
			$this->setNodeFeedbackAuthor(0);
			$this->setNodeFeedbackParticipant(0);
			$this->setNodeFeedbackParticipantRes(0);
		}
	}
	
	public static function deleteFeedback($node_index, $set_id, $session_id)
	{
		global $ilDB;
		
		$ilDB->manipulateF('DELETE FROM rep_robj_xtsf_fbt 
			WHERE step >= %s
			 AND set_id = %s 
			 AND session_id = %s',
			array('integer', 'integer', 'integer'), array($node_index, $set_id, $session_id));
	}
	
	/**
	 * @param array $data
	 */
	public static function updateConditionEvaluation(array $data)
	{
		global $ilDB;
		
		$node = $data['node'];
		$node_index = $data['node_index'];
		
		$session = $data['session'];
		$feedback_type = $data['feedback_type'];
		$condition_id = $data['condition_id'];		
		$set_id = $session->getSetId();

		if($condition_id > 0)
		{
			// jump == true
			$res = $ilDB->queryF('
			SELECT * FROM rep_robj_xtsf_fbt 
			WHERE session_id = %s
			AND step = %s
			AND feedback_type = %s
			AND condition_id = %s',
				array('integer', 'integer', 'integer', 'integer'),
				array($session->getSessionId(), $node_index, $feedback_type, $condition_id)
			);
		}
		else
		{
			$res = $ilDB->queryF('
				SELECT * FROM rep_robj_xtsf_fbt 
				WHERE session_id = %s
				AND step = %s
				AND node_id = %s
				AND feedback_type = %s',
				array('integer', 'integer', 'integer', 'integer'),
				array($session->getSessionId(), $node_index, $node->getId(), $feedback_type)
			);
		}
		
		$numrows = $ilDB->numRows($res);
		
		if(!$numrows)
		{
			$fbt_time = time();
			
			$next_id = $ilDB->nextId('rep_robj_xtsf_fbt');
			$ilDB->insert('rep_robj_xtsf_fbt',
				array(
					'fbt_id'		=> array('integer', $next_id),	
					'session_id'    => array('integer', $session->getSessionId()),
					'step'          => array('integer', $node_index),
					'node_id'       => array('integer', $node->getId()),
					'set_id'        => array('integer', $set_id),
					'condition_id'  => array('integer', $condition_id),
					'feedback_type' => array('integer', $feedback_type),
					'fbt_time'		=> array('integer', $fbt_time)
				));
		}
//		// lösche feebacks die wegen einer Sprungcondition nicht mehr benötigt werden.  
		/*$ilDB->manipulateF('DELETE FROM rep_robj_xtsf_fbt
			WHERE step >= %s 
			 AND node_id <> %s 
			 AND session_id = %s',
			array('integer', 'integer', 'integer'), array($node_index, $node->getId(), $session->getSessionId()));*/
	}

	/**
	 * @param integer $session_id
	 * @param integer $set_id
	 * @param integer $node_id
	 * @return array  with strings
	 */
	public function getNodeFeedbacks($session_id,  $set_id, $node_id, $step)
	{
		global $ilDB;

		$feedbacks = array();

		$res = $ilDB->queryF("
		   (SELECT fbt.fbt_id, 
				fbt.step, 
				 fbt.feedback_type,
				 node.false_feedback fb_0,
				 node.true_feedback fb_1,
				 jmp_con.false_feedback fb_2,
				 jmp_con.true_feedback fb_3,
				 jmp_con.true_jmp_feedback fb_4,
				 fbt.fbt_time
				 
		   FROM rep_robj_xtsf_fbt fbt
		   INNER JOIN rep_robj_xqsp_node node ON node.id = fbt.node_id
		   INNER JOIN rep_robj_xqsp_jmp_con jmp_con ON node_fi = fbt.node_id AND jmp_con.id = fbt.condition_id
		   WHERE fbt.condition_id IS NOT NULL
		   AND session_id = %s 
		   AND set_id = %s
		   AND step = %s
		   )
		   UNION
		   (SELECT fbt.fbt_id, fbt.step, 
				 fbt.feedback_type,
				 node.false_feedback fb_0,
				 node.true_feedback fb_1,
				 'dummy' fb_2,
				 'dummy' fb_3,
				 'dummy' fb_4,
				 fbt.fbt_time
				 
		   FROM rep_robj_xtsf_fbt fbt
		   INNER JOIN rep_robj_xqsp_node node ON node.id = fbt.node_id
		   WHERE fbt.condition_id IS NULL
		   AND session_id = %s 
		   AND set_id = %s
		   AND step = %s
		   )
		   ORDER BY fbt_time, fbt_id ASC",
			array('integer', 'integer', 'integer', 'integer', 'integer', 'integer'),
			array($session_id,  $set_id, $step, $session_id,  $set_id, $step)
		);

		while($row = $ilDB->fetchAssoc($res))
		{
			$feedbacks[] = ilRTE::_replaceMediaObjectImageSrc($row['fb_'.$row['feedback_type']], 1);
		}

		return $feedbacks;
	}
	
	/**
	 * @return bool
	 */
	public function isFeedbackEnabled()
	{
		global $ilDB, $ilUser, $ilAccess;
		
		$res = $ilDB->queryF('SELECT node_fb_part FROM rep_robj_xtsf_data WHERE id = %s',
			array('integer'), array($this->getId()));
		
		$row = $ilDB->fetchAssoc($res);
	
		$fb_part = (bool)$row['node_fb_part'];
		
		if($ilAccess->checkAccessOfUser($ilUser->getId(), 'read', '', $this->getRefId()) 
		&& $fb_part)
		{
			return true;
		}
		
		return false;
	}

	/**
	 * @return bool
	 */
	public function isResultFeedbackEnabled()
	{
		global $ilDB, $ilUser, $ilAccess;

		$res = $ilDB->queryF('SELECT node_fb_author, node_fb_part, node_fb_part_res FROM rep_robj_xtsf_data WHERE id = %s',
			array('integer'), array($this->getId()));

		$row = $ilDB->fetchAssoc($res);
		$fb_author 	 = (bool)$row['node_fb_author'];
		$fb_part 	 = (bool)$row['node_fb_part'];
		$fb_part_res = (bool)$row['node_fb_part_res'];

		if($ilAccess->checkAccessOfUser($ilUser->getId(), 'write', '', $this->getRefId())
			&& $fb_author)
		{
			return true;
		}
		else if($ilAccess->checkAccessOfUser($ilUser->getId(), 'read', '', $this->getRefId())
			&& $fb_part_res)
		{
			return true;
		}

		return false;
	}

	/**
	 * @param bool $skillServiceEnabled
	 */
	public function setSkillServiceEnabled($skillServiceEnabled)
	{
		$this->skillServiceEnabled = $skillServiceEnabled;
	}

	/**
	 * @return bool
	 */
	public function isSkillServiceEnabled()
	{
		if(!self::isSkillManagementGloballyActivated())
		{
			return false;
		}
		
		return $this->skillServiceEnabled;
	}

	/**
	 * @return bool
	 */
	public static function isSkillManagementGloballyActivated()
	{
		if( self::$isSkillManagementGloballyActivated === null )
		{
			include_once './Services/Skill/classes/class.ilSkillManagementSettings.php';
			$skmgSet = new ilSkillManagementSettings();

			self::$isSkillManagementGloballyActivated = $skmgSet->isActivated();
		}
		
		return self::$isSkillManagementGloballyActivated;
	}

	/**
	 * Adapter for ilObjTest::canShowTestResults
	 *
	 * @param ilFormATestSession $testSession
	 *
	 * @return boolean
	 */
	public function canShowTestResults($testSession)
	{
		return false;
	}
}

