<?php

require_once "Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/classes/class.ilObjQuestionSetPoolGUI.php";
require_once "Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/classes/class.ilObjQuestionSetPool.php";

/**
 * Class ilObjFormATestQuestionSetPoolGUI
 *
 * Date: 14.02.14
 * Time: 14:01
 * @author Thomas Joußen <tjoussen@databay.de>
 */ 
class ilObjFormATestQuestionSetPoolGUI extends ilObjQuestionSetPoolGUI
{

	/**
	 * execute command
	 */
	function &executeCommand()
	{
		global $ilCtrl, $tpl, $lng, $ilTabs, $ilLocator, $ilToolbar;

		$available_pools = array_keys(ilObjQuestionSetPool::getAvailableQuestionSetPools());

		$set = new ilQuestionSetPoolSet((int)$_GET["set_id"]);
		$set->read();

		// get standard template (includes main menu and general layout)
		$tpl->getStandardTemplate();

		$ilLocator->clearItems();

		$plugin = $this->initFormATestPlugin();

		$tpl->setTitle($this->object->getTitle());
		$tpl->setTitleIcon($plugin->getImagePath("icon_".$this->object->getType() . ".svg"),
			$lng->txt("icon")." ".$this->txt("obj_".$this->object->getType()));

		$this->setTabs();
		$this->setLocator();

		$next_class = $this->ctrl->getNextClass($this);
		$cmd = $this->ctrl->getCmd();

		switch($next_class)
		{
			case "ilinfoscreengui":
				$this->checkPermission("visible");
				$this->infoScreen();	// forwards command
				break;

			case 'ilpermissiongui':
				include_once("Services/AccessControl/classes/class.ilPermissionGUI.php");
				$perm_gui = new ilPermissionGUI($this);
				$ilTabs->setTabActive("perm_settings");
				$ret = $ilCtrl->forwardCommand($perm_gui);
				break;

			case 'ilobjectcopygui':
				include_once './Services/Object/classes/class.ilObjectCopyGUI.php';
				$cp = new ilObjectCopyGUI($this);
				$cp->setType($this->getType());
				$this->ctrl->forwardCommand($cp);
				break;

			default:
				if (strtolower($_GET["baseClass"]) == "iladministrationgui")
				{
					$this->viewObject();
					return;
				}
				if(!$cmd)
				{
					$cmd = $this->getStandardCmd();
				}
				if ($cmd == "infoScreen")
				{
					$ilCtrl->setCmd("showSummary");
					$ilCtrl->setCmdClass("ilinfoscreengui");
					$this->infoScreen();
				}
				else
				{
					$this->performCommand($cmd);

					$reflection = new ReflectionClass(get_class($this->tpl));
					$property = $reflection->getProperty('message');
					$property->setAccessible(true);
					$values = $property->getValue($this->tpl);
					$values = (array)$values;

					if(ilObjQuestionSetPool::canSynchronizeSet($set->getOriginalId()))
					{
						if(!strlen($_SESSION['failure']) && !count($values['failure']))
						{
							$ilCtrl->setParameter($this, "set_id", $set->getId());
							$ilToolbar->setFormAction($ilCtrl->getFormAction($this));
							$ilCtrl->setParameter($this, "set_id", "");
							$ilToolbar->addFormButton($this->plugin->txt("synchronize_set"),"ilQuestionSetPoolSetsGUI.synchronize", "", true);
						}
					}
					else
					{
						if(!strlen($_SESSION['failure']) && !count($values['failure']))
						{
							ilUtil::sendInfo($this->plugin->txt("not_in_test"));
						}
					}
				}
				break;
		}
	}

	protected function redirectToDefaultPage()
	{
		$this->ctrl->redirectByClass("ilObjFormATestGUI", "ilFormATestSetsGUI.listSets");
	}

	public function initFormATestPlugin()
	{
		return ilPlugin::getPluginObject(IL_COMP_SERVICE, "Repository", "robj",
			ilPlugin::lookupNameForId(IL_COMP_SERVICE, "Repository", "robj", "xtsf"));
	}

	public function setTabs()
	{
		/**
		 * @var ilTabsGUI $ilTabs
		 */
		global $ilTabs;

		$ilTabs->clearTargets();

		$ilTabs->setBackTarget($this->txt('back_to_test'), $this->ctrl->getLinkTargetByClass("ilObjFormATestGUI", "ilFormATestSetsGUI.listSets"));
	}

	protected function addHeaderAction()
	{

	}

	/**
	 * Add object to locator
	 */
	function addLocatorItems()
	{
		global $ilLocator;

		if (!$this->getCreationMode())
		{
			$ilLocator->addItem($this->object->getTitle(),
								$this->ctrl->getLinkTargetByClass('ilObjFormATestGUI', $this->getStandardCmd()), "", $_GET["ref_id"]);
		}
	}

	public function getStandardCmd()
	{
		return "ilFormATestSetsGUI.listSets";
	}



	/**
	 * Functions that must be overwritten
	 */
	/*public function getType()
	{
		return "xtsf";
	}*/
}
 