<?php
/* Copyright (c) 1998-2016 ILIAS open source, Extended GPL, see docs/LICENSE */

require_once 'Services/Xml/classes/class.ilSaxParser.php';

/**
 * Class ilObjFormATestXMLParser
 * @author Nadia Matuschek <nmatuschek@databay.de>
 */
class ilObjFormATestXMLParser extends ilSaxParser
{
	/**
	 * @var ilObjFormATest
	 */
	private $xtsf_obj;
	
	private $inSettingsTag;
	
	private $inMetaDataTag;
	private $inMdGeneralTag;
	
	public $sets = array();
	
	private $questions = array();
	private $paths = array();
	private $nodes = array();
	private $jump_conditions = array();
	protected $importDirectory = '';
	
	/**
	 * @param ilObjFormATest $xtsf_obj
	 * @param                      $xmlFile
	 */
	public function __construct($xtsf_obj, $xmlFile)
	{
		$this->xtsf_obj = $xtsf_obj;
		
		$this->inSettingsTag  = false;
		$this->inMetaDataTag  = false;
		$this->inMdGeneralTag = false;
		
		parent::ilSaxParser($xmlFile);
	}
	
	/**
	 * Set import directory
	 *
	 * @param	string	import directory
	 */
	public function setImportDirectory($a_val)
	{
		$this->importDirectory = $a_val;
	}
	
	/**
	 * Get import directory
	 *
	 * @return	string	import directory
	 */
	public function getImportDirectory()
	{
		return $this->importDirectory;
	}
	
	/**
	 * @param $xmlParser
	 */
	public function setHandlers($xmlParser)
	{
		xml_set_object($xmlParser, $this);
		xml_set_element_handler($xmlParser, 'handlerBeginTag', 'handlerEndTag');
		xml_set_character_data_handler($xmlParser, 'handlerCharacterData');
	}
	
	/**
	 * @param $xmlParser
	 * @param $tagName
	 * @param $tagAttributes
	 */
	public function handlerBeginTag($xmlParser, $tagName, $tagAttributes)
	{
		switch($tagName)
		{
			case 'MetaData':
				$this->inMetaDataTag = true;
				break;
			
			case 'General':
				if($this->inMetaDataTag)
				{
					$this->inMdGeneralTag = true;
				}
				break;
			
			case 'Description':
				if($this->inMetaDataTag && $this->inMdGeneralTag)
				{
					$this->cdata = '';
				}
				break;
			
			case 'Settings':
				$this->inSettingsTag = true;
				break;
			
			case 'Online':
			case 'SkillService':
			case 'ShowIntroStatement':
			case 'IntroStatement':
			case 'ShowFinalStatement':
			case 'FinalStatement':
			case 'MaxPasses':
			case 'StartTimeEnabled':
			case 'StartTime':
			case 'EndTimeEnabled':
			case 'EndTime':
			case 'AllowWalkBack':
			case 'ShowCurrentTrack':
			case 'ShowQuestionsOverview':
			case 'AllowInterruption':
			case 'SkillServiceEnabled':
			case 'AnswerFeedbackPoints':
			case 'GenericAnswerFeedback':
			case 'SpecificAnswerFeedback':
			case 'InstantFeedbackSolution':
			case 'NodeFeedbackAuthor':
			case 'NodeFeedbackParticipant':
			case 'NodeFeedbackParticipantRes':
			case 'ProcessTime':
			case 'CharSelectorAvailability':
			case 'CharSelectorDefinition':
			case 'ScoreReporting':
			case 'ReportingDate':
				if($this->inSettingsTag)
				{
					$this->cdata = '';
				}
				break;
			case 'SetId':
			case 'SetTitle':
			case 'SetDescription':
			case 'OriginalId':
				$this->cdata = '';
				break;
		}
	}
	
	/**
	 * @param $xmlParser
	 * @param $tagName
	 */
	public function handlerEndTag($xmlParser, $tagName)
	{
		switch($tagName)
		{
			case 'MetaData':
				$this->inMetaDataTag = false;
				break;
			
			case 'General':
				if($this->inMetaDataTag)
				{
					$this->inMdGeneralTag = false;
				}
				break;
			
			case 'Title':
				if($this->inMetaDataTag && $this->inMdGeneralTag)
				{
					$this->xtsf_obj->setTitle(trim($this->cdata));
					$this->cdata = '';
				}
				break;
			
			case 'Description':
				if($this->inMetaDataTag && $this->inMdGeneralTag)
				{
					$this->xtsf_obj->setDescription(trim($this->cdata));
					$this->cdata = '';
				}
				break;
			
			case 'Settings':
				$this->inSettingsTag = false;
				break;
			
			case 'Online':
				$this->xtsf_obj->setOnline((bool)trim($this->cdata));
				$this->cdata = '';
				break;
			
			case 'SkillService':
				$this->xtsf_obj->setSkillServiceEnabled((bool)trim($this->cdata));
				$this->cdata = '';
				break;
			case 'ShowIntroStatement':
				$this->xtsf_obj->setShowIntroStatement(trim($this->cdata));
				$this->cdata = '';
				break;
			case 'IntroStatement':
				$this->xtsf_obj->setIntroStatement(trim($this->cdata));
				$this->cdata = '';
				break;
			case 'ShowFinalStatement':
				$this->xtsf_obj->setShowFinalStatement(trim($this->cdata));
				$this->cdata = '';
				break;
			case 'FinalStatement':
				$this->xtsf_obj->setFinalStatement(trim($this->cdata));
				$this->cdata = '';
				break;
			case 'MaxPasses':
				$this->xtsf_obj->setMaxPasses(trim($this->cdata));
				$this->cdata = '';
				break;
			case 'StartTimeEnabled':
				$this->xtsf_obj->setStartTimeEnabled(trim($this->cdata));
				$this->cdata = '';
				break;
			case 'StartTime':
				$this->xtsf_obj->setStartTime(new ilDateTime(trim($this->cdata), IL_CAL_TIMESTAMP));
				
				$this->cdata = '';
				break;
			case 'EndTimeEnabled':
				$this->xtsf_obj->setEndTimeEnabled(trim($this->cdata));
				$this->cdata = '';
				break;
			case 'EndTime':
				$this->xtsf_obj->setEndTime(new ilDateTime(trim($this->cdata), IL_CAL_TIMESTAMP));
				$this->cdata = '';
				break;
			case 'AllowWalkBack':
				$this->xtsf_obj->setAllowWalkBack(trim($this->cdata));
				$this->cdata = '';
				break;
			case 'ShowCurrentTrack':
				$this->xtsf_obj->setShowCurrentTrack(trim($this->cdata));
				$this->cdata = '';
				break;
			case 'ShowQuestionsOverview':
				$this->xtsf_obj->setShowQuestionsOverview(trim($this->cdata));
				$this->cdata = '';
				break;
			case 'AllowInterruption':
				$this->xtsf_obj->setAllowInterruption(trim($this->cdata));
				$this->cdata = '';
				break;
			case 'SkillServiceEnabled':
				$this->xtsf_obj->setSkillServiceEnabled(trim($this->cdata));
				$this->cdata = '';
				break;
			case 'AnswerFeedbackPoints':
				$this->xtsf_obj->setAnswerFeedbackPoints(trim($this->cdata));
				$this->cdata = '';
				break;
			case 'GenericAnswerFeedback':
				$this->xtsf_obj->setGenericAnswerFeedback(trim($this->cdata));
				$this->cdata = '';
				break;
			case 'SpecificAnswerFeedback':
				$this->xtsf_obj->setSpecificAnswerFeedback(trim($this->cdata));
				$this->cdata = '';
				break;
			case 'InstantFeedbackSolution':
				$this->xtsf_obj->setInstantFeedbackSolution(trim($this->cdata));
				$this->cdata = '';
				break;
			case 'NodeFeedbackAuthor':
				$this->xtsf_obj->setNodeFeedbackAuthor(trim($this->cdata));
				$this->cdata = '';
				break;
			case 'NodeFeedbackParticipant':
				$this->xtsf_obj->setNodeFeedbackParticipant(trim($this->cdata));
				$this->cdata = '';
				break;
			case 'NodeFeedbackParticipantRes':
				$this->xtsf_obj->setNodeFeedbackParticipantRes(trim($this->cdata));
				$this->cdata = '';
				break;
			case 'ProcessTime':
				$this->xtsf_obj->setProcessTime(trim($this->cdata));
				$this->cdata = '';
				break;
			case 'CharSelectorAvailability':
				$this->xtsf_obj->setCharSelectorAvailability(trim($this->cdata));
				$this->cdata = '';
				break;
			case 'CharSelectorDefinition':
				$this->xtsf_obj->setCharSelectorDefinition(trim($this->cdata));
				$this->cdata = '';
				break;
			case 'ScoreReporting':
				$this->xtsf_obj->setScoreReporting(trim($this->cdata));
				$this->cdata = '';
				break;
			case 'ReportingDate':
				$this->xtsf_obj->setReportingDate(new ilDateTime(trim($this->cdata), IL_CAL_TIMESTAMP));
				$this->cdata = '';
				break;
			
			// sets	
			case 'SetId':
			case 'SetTitle':
			case 'SetDescription':
			case 'OriginalId':
				$this->import_set[$tagName] = trim($this->cdata);
				$this->cdata                = '';
				break;
			// question
			case 'QuestionId':
			case 'QuestionIndex':
			case 'QuestionOriginalId':
				$this->import_question[$tagName] = trim($this->cdata);
				$this->cdata                     = '';
				break;
			// paths
			case 'PathId':
			case 'PathTitle':
			case 'PathQuestionSetId':
			case 'PathIndex':
				$this->import_path[$tagName] = trim($this->cdata);
				$this->cdata                 = '';
				break;
			//nodes
			case 'NodeId':
			case 'NodeQuestionId':
			case 'NodePathId':
			case 'NodeIndex':
			case 'NodeInitialCondition':
			case 'NodeDominantScoring':
			case 'NodeTrueFeedback':
			case 'NodeFalseFeedback':
				$this->import_node[$tagName] = trim($this->cdata);
				$this->cdata                 = '';
				break;
			// jump conditions 
			case 'JumpConditionId':
			case 'JumpConditionText':
			case 'JumpConditionNodeId':
			case 'JumpConditionQuestionId':
			case 'JumpConditionRepetitions':
			case 'JumpConditionSorting':
			case 'JumpConditionTrueFeedback':
			case 'JumpConditionTrueJumpFeedback':
			case 'JumpConditionFalseFeedback':
				$this->import_jump_condition[$tagName] = trim($this->cdata);
				$this->cdata                           = '';
				break;
			case 'Set':
				//'addSet';
				$this->sets[]     = array($this->import_set, 'questions' => $this->questions, 'paths' => $this->paths);
				$this->questions  = array();
				$this->paths      = array();
				$this->import_set = array();
				break;
			
			case 'Question':
				// addQuesion
				$this->questions[]     = $this->import_question;
				$this->import_question = array();
				break;
			
			case 'Path':
				// addPath
				$this->paths[]     = array($this->import_path, 'nodes' => $this->nodes);
				$this->nodes       = array();
				$this->import_path = array();
				break;
			
			case 'Node':
				// addNode
				$this->nodes[]         = array($this->import_node, 'jump_conditions' => $this->jump_conditions);
				$this->jump_conditions = array();
				$this->import_node     = array();
				break;
			
			case 'JumpCondition':
				// addJumpConditition
				$this->jump_conditions[]     = $this->import_jump_condition;
				$this->import_jump_condition = array();
				break;
		}
	}
	
	public function handlerCharacterData($xmlParser, $charData)
	{
		if($charData != "\n")
		{
			// Replace multiple tabs with one space
			$charData = preg_replace("/\t+/", " ", $charData);
			
			$this->cdata .= $charData;
		}
	}
	
	/**
	 * @return mixed
	 */
	public function getFormATestObject()
	{
		return $this->xtsf_obj;
	}
	
	
	public function getSetsArray()
	{
		return $this->sets;
	}
}