<?php

require_once "abstract.ilXTSFControllerGUI.php";

/**
 * Class ilXTSFControllerFormGUI
 *
 * Date: 02.10.13
 * Time: 15:10
 * @author Thomas Joußen <tjoussen@databay.de>
 */ 
abstract class ilXTSFControllerFormGUI extends ilXTSFControllerGUI
{

	/**
	 * @var ilPropertyFormGUI
	 */
	protected $form;

	public function edit()
	{
		$this->controller->ensurePermission("write");

		$model = $this->loadModel();
		$this->initForm($model);
		$this->bindModelToForm($model);
		$this->tpl->setContent($this->form->getHTML());
	}

	public function update()
	{
		$this->controller->ensurePermission("write");

		$model = $this->loadModel();
		$this->initForm($model);
		if($this->form->checkInput() && $this->canUpdate())
		{
			$item = $this->form->getItemByPostVar('online');
			if($item && (int)$this->form->getInput('online') && !$this->controller->object->isComplete())
			{
				ilUtil::sendFailure($this->lng->txt('form_input_not_valid'));
				$this->form->getItemByPostVar('online')->setAlert(
					$this->plugin->txt('onp_no_sets_no_mark_steps')
				);
				$this->form->setValuesByPost();
				$this->tpl->setContent($this->form->getHTML());
				return;
			}

			$model->bindForm($this->form);
			$model->update();
			ilUtil::sendSuccess($this->plugin->txt("msg_obj_modified"), true);
			$this->redirect(
				$this->getAfterUpdateRedirectCmd(),
				$this->getAfterUpdateRedirectParams($model)
			);
		}

		$this->form->setValuesByPost();
		$this->tpl->setContent($this->form->getHTML());
	}

	protected function canUpdate()
	{
		return true;
	}


	/**
	 * @param ilXTSFModelInterface $model
	 */
	protected function bindModelToForm(ilXTSFModelInterface $model)
	{
		$this->form->setValuesByArray($model->toArray());
	}

	/**
	 * Loads and returns the Model for the controller gui
	 *
	 * @return ilXTSFModel
	 */
	abstract protected function loadModel();

	/**
	 * Initialize the PropertyFormGUI for this controller
	 *
	 * @param ilXTSFModel $model
	 *
	 * @return mixed
	 */
	abstract protected function initForm($model);

	/**
	 * Get the redirect action after an update was successful
	 *
	 * @return string
	 */
	abstract protected function getAfterUpdateRedirectCmd();

	/**
	 * @param $model
	 *
	 * @return array
	 */
	abstract protected function getAfterUpdateRedirectParams($model);
}
