<?php

require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/FormATest/classes/class.ilObjFormATestGUI.php";

/**
 * Class ilXTSFControllerGUI
 *
 * Date: 23.09.13
 * Time: 14:10
 * @author Thomas Joußen <tjoussen@databay.de>
 */ 
abstract class ilXTSFControllerGUI {

	/**
	 * The main Controller of the Plugin
	 *
	 * @var ilObjFormATestGUI
	 */
	public $controller;

	/**
	 * @var ilCtrl
	 */
	protected $ctrl;

	/**
	 * @var ilToolbarGUI;
	 */
	protected $toolbar;

	/**
	 * @var ilTemplate
	 */
	protected $tpl;

	/**
	 * @var iPlugin
	 */
	protected $plugin;

	/**
	 * @var ilLanguage
	 */
	protected $lng;

	/**
	 * @var ilTabsGUI
	 */
	protected $tabs;

	/**
	 * @var ilObjUser
	 */
	protected $user;

	/**
	 * @param ilObjFormATestGUI $controller
	 */
	public function __construct(ilObjFormATestGUI $controller)
	{
		/**
		 * @var $ilCtrl ilCtrl
		 * @var $tpl ilTemplate
		 * @var $lng ilLanguage
		 * @var $ilToolbar ilToolbarGUI
		 * @var $ilTabs ilTabsGUI
		 * @var $ilUser ilObjUser
		 */
		global $ilCtrl, $tpl, $lng, $ilToolbar, $ilTabs, $ilUser;

		$this->controller = $controller;
		$this->ctrl       = $ilCtrl;
		$this->tpl        = $tpl;
		$this->plugin     = $this->controller->plugin;
		$this->lng        = $lng;
		$this->toolbar    = $ilToolbar;
		$this->tabs       = $ilTabs;
		$this->user       = $ilUser;
		$this->activateTab();
		$this->includeRequirements();
	}

	protected function initToolbar()
	{
		$this->toolbar->setFormAction($this->ctrl->getFormAction($this->controller, '', 'question_output'));
	}

	protected function addToolbarButton($text, $command, array $params = array())
	{
		if($this->toolbar == null)
		{
			$this->initToolbar();
		}

		foreach($params as $key => $param)
		{
			$this->ctrl->setParameter($this->controller, $key, $param);
		}

		$link = $this->ctrl->getLinkTarget($this->controller, $command, 'question_output');

		foreach($params as $key => $param)
		{
			$this->ctrl->setParameter($this->controller, $key, "");
		}

		$this->toolbar->addButton($this->plugin->txt($text),$link);
	}

	/**
	 * Activates the tab for the current GUI
	 */
	private function activateTab()
	{
		global $ilTabs;

		if($this->getCurrentTab() != "")
		{
			$ilTabs->activateTab($this->getCurrentTab());
		}
	}

	private function includeRequirements()
	{
		$requirements = $this->getRequirements();

		if(!empty($requirements))
		{
			foreach($requirements as $requirement)
			{
				require_once $this->controller->plugin->getDirectory().DIRECTORY_SEPARATOR."classes".DIRECTORY_SEPARATOR.$requirement;
			}
		}
	}

	protected function redirect($command, $params = array())
	{
		foreach($params as $key => $param)
		{
			$this->ctrl->setParameter($this->controller, $key, $param);
		}

		$this->ctrl->redirect($this->controller, $command, 'question_output');
	}

	/**
	 * Returns the current tab name
	 *
	 * return string
	 */
	abstract protected function getCurrentTab();

	/**
	 * @return array
	 */
	abstract protected function getRequirements();

	/**
	 * @param ilFormATestSession $session
	 * @param array              $hidden_buttons
	 */
	protected function addSessionSensitiveButton(ilFormATestSession $session, $hidden_buttons = array())
	{
		$canStartTest =
			ilFormATestPlayerAccessHelper::canDoTestAction($this->controller, $session) &&
			$this->controller->hasPermission("read");
		
		if(!$this->controller->object->isComplete())
		{
			ilUtil::sendInfo($this->plugin->txt('cs_no_sets_no_mark_steps'), true);
			return;
		}
		else if (!$session->activeIDExists() && $canStartTest)
		{
			$this->toolbar->addFormButton($this->plugin->txt("start_test"), "ilFormATestPlayerControllerGUI.startTest", "", true);
		}
		else if($canStartTest)
		{
			if (!$session->isPassFinished())
			{
				$this->toolbar->addFormButton($this->plugin->txt("resume"), "ilFormATestPlayerControllerGUI.resumeTest", "", true);
			}
			else if($this->controller->object->getMaxPasses() == 0 || $session->getPass() + 1 < $this->controller->object->getMaxPasses())
			{
				$this->toolbar->addFormButton($this->plugin->txt("start_new_pass"), "ilFormATestPlayerControllerGUI.startNewPass", "", true);
			}
			else
			{
				ilUtil::sendInfo($this->controller->lng->txt('maximum_nr_of_tries_reached'), true);
			}
		}

		if(!in_array('test_results', $hidden_buttons) && ($this->scoringReportingAlways() || $this->scoringReportingFinished($session) || $this->scoringReportingDate()))
		{
			$this->toolbar->addFormButton($this->plugin->txt("show_test_results"), "ilFormATestInfoScreenControllerGUI.showTestResults");
		}

		if($session->isPassFinished() && $this->controller->object->isSkillServiceEnabled() && ilObjFormATest::isSkillManagementGloballyActivated() )
		{
			require_once './Modules/Test/classes/class.ilTestSkillEvaluationGUI.php';
			$link = $this->ctrl->getLinkTargetByClass(
				array('ilTestSkillEvaluationGUI'),
				ilTestSkillEvaluationGUI::CMD_SHOW
			);

			$this->toolbar->addButton($this->lng->txt("tst_show_comp_results"), $link);
		}
	}

	/**
	 * @param string $cancel
	 * @param string $confirm
	 * @param array $items
	 * @param string $header
	 */
	protected function renderConfirmationScreen($cancel, $confirm, $items = array(), $header = "", $params = array())
	{
		$confirmation = new ilConfirmationGUI();
		foreach($params as $key => $value)
		{
			$this->ctrl->setParameter($this->controller, $key, $value);
		}
		$this->ctrl->setParameter($this->controller, "confirmed", true);
		$confirmation->setFormAction($this->ctrl->getFormAction($this->controller, '', 'question_output'));
		foreach($params as $key => $value)
		{
			$this->ctrl->setParameter($this->controller, $key, "");
		}
		$this->ctrl->setParameter($this->controller, "confirmed", "");
		$confirmation->setHeaderText($header);
		$confirmation->setCancel($this->lng->txt("cancel"), $cancel);
		$confirmation->setConfirm($this->lng->txt("confirm"), $confirm);

		foreach ($items as $item)
		{
			$confirmation->addItem($item["name"], $item["id"], $item["text"]);
		}

		$this->tpl->setContent($confirmation->getHTML());
	}

	protected function scoringReportingAlways()
	{
		return $this->controller->object->getScoreReporting() == ilObjFormATest::SCORE_REPORTING_ALWAYS && $this->controller->object->getParticipants()->hasParticipants();
	}

	protected function scoringReportingFinished(ilFormATestSession $session)
	{
		return $this->controller->object->getScoreReporting() == ilObjFormATest::SCORE_REPORTING_FINISHED &&
				$session->isPassFinished();
	}

	protected function scoringReportingDate()
	{
		$date = new ilDateTime(time(), IL_CAL_UNIX);

		return $this->controller->object->getScoreReporting() == ilObjFormATest::SCORE_REPORTING_DATE &&
			$date->_after($date, $this->controller->object->getReportingDate());
	}

	/**
	 * @param bool $finish_set
	 * @param ilQuestionSetPoolNode $node
	 * @param array $params
	 *
	 * @return string
	 */
	protected function generateJumpToNodeLink($finish_set, $node, $params = array())
	{
		$this->ctrl->setParameter($this->controller, "node_id", $node["id"]);
		if ($finish_set)
		{
			$this->ctrl->setParameter($this->controller, "finished", 1);
		}
		foreach($params as $key => $value)
		{
			$this->ctrl->setParameter($this->controller, $key, $value);
		}
		$href = $this->ctrl->getLinkTarget($this->controller, "ilFormATestPlayerControllerGUI.jumpToNode", 'question_output');
		foreach($params as $key => $value)
		{
			$this->ctrl->setParameter($this->controller, $key, "");
		}

		$this->ctrl->setParameter($this->controller, "node_id", "");
		$this->ctrl->setParameter($this->controller, "finished", "");

		return $href;
	}
}
