<?php

require_once "abstract.ilXTSFControllerGUI.php";

/**
 * Class ilFormATestInfoScreenControllerGUI
 *
 * Date: 12.12.13
 * Time: 14:39
 * @author Thomas Joußen <tjoussen@databay.de>
 */ 
class ilFormATestInfoScreenControllerGUI extends ilXTSFControllerGUI
{
	/**
	 * @var ilFormATestSession
	 */
	protected $session;

	/**
	 * @param ilObjFormATestGUI $controller
	 */
	public function __construct($controller)
	{
		parent::__construct($controller);

		$this->session = new ilFormATestSession($controller->object);
		$this->session->read($controller->object->getSetList()->getData());
	}

	public function show()
	{
		$this->initToolbar();
		$this->addSessionSensitiveButton($this->session);

		$info = new ilFormATestInfoScreenGUI($this->controller);
		$info->setIntro($this->controller->object->getIntroStatement());
		$this->tpl->setContent($info->getHTML());
	}

	public function showTestResults()
	{
		$this->controller->ensurePermission("read");

		$output = new ilFormATestResultOuputGUI($this->controller);
		$output->setHeader($this->plugin->txt("test_pass_overview"));

		$output->renderTestData($this->session->getUserId());
		$output->renderPassesOverview($this->session->getActiveId(), __CLASS__ .".showDetails");

		$this->initToolbar();
		$this->addSessionSensitiveButton($this->session, array('test_results'));
		$this->toolbar->addButton($this->lng->txt('print'), 'javascript:window.print();');
		$this->tpl->addCss(ilUtil::getStyleSheetLocation('output', 'test_print.css', 'Modules/Test'), 'print');

		$this->tpl->setContent($output->getHTML());
	}

	public function showDetails()
	{
		$output = new ilFormATestResultOuputGUI($this->controller);
		$output->setHeader($this->plugin->txt("test_pass_overview"));
		$output->setModus(ilFormATestResultOuputGUI::MODUS_PARTICIPANTS);
		$output->renderTestData($this->session->getUserId());
		$output->renderStepsOverview((int)$_GET["session_id"]);
		$output->renderQuestionDetails((int)$_GET["session_id"], $this->controller->object->getSetList()->getData(), true, null, true);

		$this->initToolbar();
		$this->addToolbarButton("back", __CLASS__.".showTestResults");
		$this->addSessionSensitiveButton($this->session, array('test_results'));
		$this->toolbar->addButton($this->lng->txt('print'), 'javascript:window.print();');
		$this->tpl->addCss(ilUtil::getStyleSheetLocation('output', 'test_print.css', 'Modules/Test'), 'print');

		$this->tpl->setContent($output->getHTML());
	}

	/**
	 * Returns the current tab name
	 * return string
	 */
	protected function getCurrentTab()
	{
		return "info_short";
	}

	/**
	 * @return array
	 */
	protected function getRequirements()
	{
		require_once "./Services/InfoScreen/classes/class.ilInfoScreenGUI.php";

		return array(
			"models/class.ilFormATestSession.php",
			"utils/abstract.ilFormATestPlayerAccessHelper.php",
			"utils/class.ilFormATestInfoScreenGUI.php",
			"controllers/class.ilFormATestResultOuputGUI.php"
		);
	}
}
 