<?php
require_once 'abstract.ilXTSFControllerGUI.php';
require_once 'Services/Tracking/classes/class.ilLPObjSettings.php';
require_once 'Services/Tracking/classes/class.ilLearningProgressBaseGUI.php';
require_once 'Modules/TestQuestionPool/classes/class.assQuestion.php';
require_once 'Services/Tracking/classes/class.ilLPStatusFactory.php';

/**
 * Class ilFormATestLearningProgressGUI
 */
class ilFormATestLearningProgressGUI  extends ilXTSFControllerGUI
{
	/**
	 * @param ilObjFormATestGUI $controller
	 */
	public function __construct(ilObjFormATestGUI $controller)
	{
		parent::__construct($controller);

		$this->lng->loadLanguageModule('trac');
		$this->addSubTabs();
	}

	private function addSubTabs()
	{
		if($this->controller->hasPermission('write'))
		{
			if($this->controller->object->getLearningProgressMode() != ilObjFormATest::LP_MODE_DEACTIVATED)
			{
				$this->tabs->addSubTab('lp_users', $this->plugin->txt('lp_users'), $this->ctrl->getLinkTarget($this->controller, __CLASS__ . '.showLPUsers'));
				$this->tabs->addSubTab('lp_summary', $this->plugin->txt('lp_summary'), $this->ctrl->getLinkTarget($this->controller, __CLASS__ . '.showLPSummary'));
			}
			$this->tabs->addSubTab('lp_settings', $this->lng->txt('trac_settings'), $this->ctrl->getLinkTarget($this->controller, __CLASS__ . '.showLPSettings'));
		}
		else if($this->controller->hasPermission('read') && $this->controller->object->getLearningProgressMode() != ilObjFormATest::LP_MODE_DEACTIVATED)
		{
			$this->tabs->addSubTab('lp_users', $this->plugin->txt('lp_users'), $this->ctrl->getLinkTarget($this->controller, __CLASS__ . '.showLPUserDetails'));
		}
	}
	
	/**
	 *
	 */
	public function showLPSettings(ilPropertyFormGUI $form = null)
	{
		$this->tabs->activateSubTab('lp_settings');
		$this->controller->ensurePermission('write');

		if(!($form instanceof ilPropertyFormGUI))
		{
			$form = $this->getLearningProgressSettingsForm();
		}

		$this->tpl->setContent($form->getHTML());
	}

	/**
	 * Init property form
	 *
	 * @return ilPropertyFormGUI $form
	 */
	public function getLearningProgressSettingsForm()
	{
		$form = new ilPropertyFormGUI();
		$form->setTitle($this->lng->txt('tracking_settings'));
		$form->setFormAction($this->ctrl->getFormAction($this->controller, __CLASS__ . '.saveLearningProgressSettings'));

		$mod = new ilRadioGroupInputGUI($this->lng->txt('trac_mode'), 'modus');
		$mod->setRequired(true);
		$form->addItem($mod);

		foreach($this->controller->object->getLPValidModes() as $mode)
		{
			$opt = new ilRadioOption(
				ilLPObjSettings::_mode2Text($mode),
				$mode,
				ilLPObjSettings::_mode2InfoText($mode)
			);
			$mod->addOption($opt);
		}
		$mod->setValue($this->controller->object->getLearningProgressMode());

		$form->addCommandButton(__CLASS__ . '.saveLearningProgressSettings', $this->lng->txt('save'));

		return $form;
	}

	/**
	 * Save learning progress settings
	 */
	public function saveLearningProgressSettings()
	{
		$this->controller->ensurePermission('write');
		$form = $this->getLearningProgressSettingsForm();
		if($form->checkInput())
		{
			$new_mode     = (int)$form->getInput('modus');
			$old_mode     = (int)$this->controller->object->getLearningProgressMode();
			$mode_changed = ($old_mode != $new_mode);

			$this->controller->object->setLearningProgressMode($new_mode);
			$this->controller->object->update();

			if($mode_changed)
			{
				include_once 'Services/Tracking/classes/class.ilLPStatusWrapper.php';
				ilLPStatusWrapper::_refreshStatus($this->controller->object->getId());
			}

			ilUtil::sendSuccess($this->lng->txt('trac_settings_saved'), true);
			$this->ctrl->redirect($this->controller, __CLASS__ . '.showLPSettings');
		}

		$form->setValuesByPost();
		$this->showLPSettings($form);
	}

	/**
	 *
	 */
	public function showLPUsers()
	{
		$this->tabs->activateSubTab('lp_users');
		$this->controller->ensurePermission('write');
		$this->plugin->includeClass('class.ilFormATestLPUsersTableGUI.php');
		$table = new ilFormATestLPUsersTableGUI($this->controller, __CLASS__ . '.showLPUsers', $this->controller->object->getId(), $this->controller->object->getRefId(), false);
		$this->tpl->setContent(implode('<br />', array($table->getHTML(), ilLearningProgressBaseGUI::__getLegendHTML())));
	}

	/**
	 *
	 */
	public function showLPSummary()
	{
		$this->tabs->activateSubTab('lp_summary');
		$this->controller->ensurePermission('write');
		$this->plugin->includeClass('class.ilFormATestLPSummaryTableGUI.php');
		$table = new ilFormATestLPSummaryTableGUI($this->controller, __CLASS__ . '.showLPSummary', $this->controller->object->getRefId(), $this->plugin);
		$this->tpl->setContent(implode('<br />', array($table->getHTML(), ilLearningProgressBaseGUI::__getLegendHTML())));
	}
	
	public function showLPUserDetails()
	{
		$this->tabs->activateSubTab('lp_summary');
		$this->controller->ensurePermission('read');
		
		if($this->controller->object->getLearningProgressMode() == ilObjFormATest::LP_MODE_DEACTIVATED)
		{
			$this->ctrl->redirect($this->controller, $this->controller->getStandardCmd());
		}

		include_once('./Services/InfoScreen/classes/class.ilInfoScreenGUI.php');
		$cloned_controller = clone $this->controller;
		$cloned_controller->object = null;
		$info = new ilInfoScreenGUI($cloned_controller);
		$info->setFormAction($this->ctrl->getFormAction($this->controller, __CLASS__ . '.editUser'));
		$info->addSection($this->lng->txt('trac_learning_progress'));
		include_once("./Services/Tracking/classes/class.ilLearningProgressBaseGUI.php");
		$status = ilLearningProgressBaseGUI::__readStatus($this->controller->object->getId(), $this->user->getId());
		$status_path = ilLearningProgressBaseGUI::_getImagePathForStatus($status);
		$status_text = ilLearningProgressBaseGUI::_getStatusText($status);
		$info->addProperty($this->lng->txt('trac_status'), ilUtil::img($status_path, $status_text)." ".$status_text);
		include_once 'Services/Tracking/classes/class.ilLPMarks.php';
		if(strlen($mark = ilLPMarks::_lookupMark($this->user->getId(), $this->controller->object->getId())))
		{
			$info->addProperty($this->lng->txt('trac_mark'),$mark);
		}
		if(strlen($comment = ilLPMarks::_lookupComment($this->user->getId(), $this->controller->object->getId())))
		{
			$info->addProperty($this->lng->txt('trac_comment'),$comment);
		}

		$this->tpl->setContent(implode('<br />', array($info->getHTML(), ilLearningProgressBaseGUI::__getLegendHTML())));
	}

	/**
	 * @param ilPropertyFormGUI $form
	 */
	public function editUser(ilPropertyFormGUI $form = null)
	{
		$this->tabs->activateSubTab('lp_users');
		$this->controller->ensurePermission('write');
		
		if(!isset($_GET['user_id']))
		{
			return $this->showLPUsers();
		}
		
		$user = ilObjectFactory::getInstanceByObjId((int)$_GET['user_id'], false);
		if(!$user instanceof ilObjUser)
		{
			return $this->showLPUsers();
		}

		include_once('./Services/InfoScreen/classes/class.ilInfoScreenGUI.php');
		$cloned_controller = clone $this->controller;
		$cloned_controller->object = null;
		$info = new ilInfoScreenGUI($cloned_controller);
		$info->setFormAction($this->ctrl->getFormAction($this->controller, __CLASS__ . '.editUser'));
		$info->addSection($this->lng->txt('trac_user_data'));
		$info->addProperty($this->lng->txt('last_login'), ilDatePresentation::formatDate(new ilDateTime($user->getLastLogin(), IL_CAL_DATETIME)));
		$info->addProperty($this->lng->txt('trac_total_online'), ilFormat::_secondsToString(ilOnlineTracking::getOnlineTime($user->getId())));

		if(!$form instanceof ilPropertyFormGUI)
		{
			$form = $this->getLPMarksForm($user);

			include_once 'Services/Tracking/classes/class.ilLPMarks.php';
			$marks = new ilLPMarks($this->controller->object->getId(), $user->getId());

			$form->setValuesByArray(array(
				'comment' => $marks->getComment(),
				'mark'    => $marks->getMark()
			));
		}

		$this->tpl->setContent(implode('<br />', array($form->getHtml(), $info->getHTML())));
	}

	/**
	 * @param ilObjUser $user
	 * @return ilPropertyFormGUI
	 */
	protected function getLPMarksForm(ilObjUser $user)
	{
		require_once 'Services/Form/classes/class.ilPropertyFormGUI.php';
		$form = new ilPropertyFormGUI();
		$this->ctrl->setParameter($this->controller, 'user_id', $user->getId());
		$form->setFormAction($this->ctrl->getFormAction($this->controller, __CLASS__ . '.editUser'));
		$form->setTitle($this->lng->txt('edit'). ': '. $this->lng->txt('trac_learning_progress_tbl_header') . $user->getFullname() );
		$form->setDescription($this->lng->txt('trac_mode').': '.ilLPObjSettings::_mode2Text($this->controller->object->getLearningProgressMode()));

		$mark = new ilTextInputGUI($this->lng->txt('trac_mark'), 'mark');
		$mark->setSize(5);
		$form->addItem($mark);

		$comment = new ilTextInputGUI($this->lng->txt('trac_comment'), 'comment');
		$form->addItem($comment);

		$form->addCommandButton(__CLASS__ . '.updateLPUsers', $this->lng->txt('save'));
		$form->addCommandButton(__CLASS__ . '.showLPUsers', $this->lng->txt('cancel'));

		return $form;
	}
	
	public function updateLPUsers()
	{
		$this->tabs->activateSubTab('lp_users');
		$this->controller->ensurePermission('write');

		if(!isset($_GET['user_id']))
		{
			return $this->showLPUsers();
		}

		$user = ilObjectFactory::getInstanceByObjId((int)$_GET['user_id'], false);
		if(!$user instanceof ilObjUser)
		{
			return $this->showLPUsers();
		}
		
		$form = $this->getLPMarksForm($user);
		if($form->checkInput())
		{
			include_once 'Services/Tracking/classes/class.ilLPMarks.php';
			$marks = new ilLPMarks($this->controller->object->getId(), $user->getId());
			$marks->setMark($form->getInput('mark'));
			$marks->setComment($form->getInput('comment'));
			$marks->update();
			ilUtil::sendSuccess($this->lng->txt('trac_update_edit_user'));
			return $this->showLPUsers();
		}

		$form->setValuesByPost();
		$this->editUser($form);
	}
	
	/**
	 * Returns the current tab name
	 * return string
	 */
	protected function getCurrentTab()
	{
		return 'learning_progress';
	}

	/**
	 * @return array
	 */
	protected function getRequirements()
	{
	}
} 