<?php

require_once "abstract.ilXTSFControllerGUI.php";

/**
 * Class ilFormATestOutputGUI
 *
 * Date: 13.01.14
 * Time: 15:21
 * @author Thomas Joußen <tjoussen@databay.de>
 */ 
class ilFormATestOutputGUI extends ilXTSFControllerGUI
{

	/**
	 * @var ilFormATestSession
	 */
	protected $session;

	/**
	 * @param \ilFormATestSession $session
	 */
	public function setSession($session)
	{
		$this->session = $session;
	}

	protected function populateCharSelector()
	{
		global $ilSetting;
		if ($ilSetting->get('char_selector_availability') > 0)
		{
			require_once 'Services/UIComponent/CharSelector/classes/class.ilCharSelectorGUI.php';
			// Chat selector doesn't support plugins
			$test = new ilObjTest();
			$test->setCharSelectorAvailability($this->controller->object->getCharSelectorAvailability());
			$test->setCharSelectorDefinition($this->controller->object->getCharSelectorDefinition());

			$char_selector = ilCharSelectorGUI::_getCurrentGUI($test);
			if($char_selector->getConfig()->getAvailability() == ilCharSelectorConfig::ENABLED)
			{
				$char_selector->addToPage();
				$this->tpl->setCurrentBlock('char_selector');
				$this->tpl->setVariable("CHAR_SELECTOR_IMAGE", ilUtil::getImagePath('icon_omega_test.svg','Services/UIComponent/CharSelector'));
				$this->tpl->setVariable("CHAR_SELECTOR_TEXT", $this->lng->txt('char_selector'));
				$this->tpl->setVariable("CHAR_SELECTOR_TEMPLATE", $char_selector->getSelectorHtml());
				$this->tpl->parseCurrentBlock();
			}
		}
	}

	public function outTestPage($node)
	{
		$this->controller->ensurePermission("read");

		$this->session->trackProgress();

		$this->tpl->addCss('./Customizing/global/plugins/Services/Repository/RepositoryObject/FormATest/templates/default/xtsf_player.css');
		$this->tpl->addBlockFile("ADM_CONTENT", "adm_content", "tpl.il_xtsf_output.html", "Customizing/global/plugins/Services/Repository/RepositoryObject/FormATest");

		if(!$this->session->isFirstStep() && $this->controller->object->isAllowWalkBack())
		{
			$this->renderButton($this->tpl, "prev");
		}
		if($this->controller->object->isAllowInterruption())
		{
			$this->renderButton($this->tpl, "interrupt");
		}
		$this->renderButton($this->tpl, "interruptset");

		$this->renderButton($this->tpl, "next");
		$this->populateContentStyleBlock();
		$this->populateSyntaxStyleBlock();
		$this->populateCharSelector();
		$this->renderSidebar($this->tpl);
		if ($this->controller->object->getShowCurrentTrack())
		{
			$this->renderCurrentResults($this->tpl);
		}

		if(
			!$this->session->hasFixatedSolution($node->getId()) &&
			$this->session->isDeadlockPossible()
		)
		{
			ilFormATestUtil::addInfo($this->plugin->txt("fixation_possible_msg"));
		}

		$gui = assQuestion::instantiateQuestionGUI($node->getQuestion()->getId());
		$gui->object->setStep($this->session->getStep());
		$gui->setTargetGuiClass("ilObjFormATestGUI");
		$gui->setQuestionHeaderBlockBuilder(new ilFormATestParticipantQuestionHeaderBuilder($this->plugin, $node, $this->session));

		if(!$this->session->hasFixatedSolution($node->getId()))
		{
			// This controls if the solution should be shown.
			// It gets the parameter "Scoring and Results" -> "Instant Feedback" -> "Show Solutions"
			
			$is_instant_feedback = (bool)$this->controller->object->getInstantFeedbackSolution() && ilSession::get($this->controller->object->getInstantFeedbackSessionKey());
			ilSession::set($this->controller->object->getInstantFeedbackSessionKey(), false);
			
			// This controls if the instant feedback solution should be shown.
			// It gets the parameter "Scoring and Results" -> "Instant Feedback" -> "Show best solution"				
			if($is_instant_feedback)
			{
				$show_question_inline_score = $this->determineInlineScoreDisplay();
				
				// Notation of the params prior to getting rid of this crap in favor of a class
				$solutionoutput = $gui->getSolutionOutput(
						$this->session->getActiveId(),    #active_id
						NULL,                                                #pass
						TRUE,                                                #graphical_output
						$show_question_inline_score,                        #result_output
						FALSE,                                                #show_question_only
						FALSE,                                                #show_feedback
						TRUE,                                                #show_correct_solution
						FALSE,                                                #show_manual_scoring
						FALSE                                                #show_question_text
				);
				$this->populateSolutionBlock($solutionoutput);
			}

			// This controls if the score should be shown.
			// It gets the parameter "Scoring and Results" -> "Instant Feedback" -> "Show Results (Only Points)"				
			if($this->controller->object->getAnswerFeedbackPoints())
			{
				$reachedPoints = $gui->object->getAdjustedReachedPoints($this->session->getActiveId(), NULL);
				$maxPoints     = $gui->object->getMaximumPoints();

				$this->populateScoreBlock($reachedPoints, $maxPoints);
			}

			// This controls if the generic feedback should be shown.
			// It gets the parameter "Scoring and Results" -> "Instant Feedback" -> "Show Solutions"				
			if($this->controller->object->getGenericAnswerFeedback())
			{
				$this->populateGenericFeedbackBlock($gui);
			}

			// This controls if the specific feedback should be shown.
			// It gets the parameter "Scoring and Results" -> "Instant Feedback" -> "Show Answer-Specific Feedback"
			if($this->controller->object->getSpecificAnswerFeedback())
			{
				$this->populateSpecificFeedbackBlock($gui);
			}
		}
		$this->fillQuestionRelatedNavigation($gui);
		
		ilFormATestUtil::send();
		$user_solutions = false;
		if(array_key_exists('previouspost', $_SESSION))
		{
			$user_solutions = $_SESSION['previouspost'];
			unset($_SESSION['previouspost']);
		}
		$gui->outQuestionForTest($this->getQuestionPageFormAction(), $this->session->getActiveId(), $this->session->getPass(), false, $user_solutions, $is_instant_feedback);
	}

	private function showFeedbackButton()
	{
		global $tpl, $lng;
		
		$tpl->setCurrentBlock("direct_feedback");
		$tpl->setVariable("CMD_SHOW_INSTANT_RESPONSE", 'showInstantResponse');
		$tpl->setVariable("TEXT_SHOW_INSTANT_RESPONSE", $lng->txt("check"));
		$tpl->parseCurrentBlock();
	}
	
	protected function populateSpecificFeedbackBlock($question_gui)
	{
		$this->tpl->setCurrentBlock( "specific_feedback" );
		$this->tpl->setVariable( "SPECIFIC_FEEDBACK",
			$question_gui->getSpecificFeedbackOutput(
				$this->session->getActiveId(),
				NULL
			)
		);
		$this->tpl->parseCurrentBlock();
	}

	protected function populateGenericFeedbackBlock($question_gui)
	{
		$this->tpl->setCurrentBlock( "answer_feedback" );
		$this->tpl->setVariable( "ANSWER_FEEDBACK",
			$question_gui->getAnswerFeedbackOutput( $this->session->getActiveId(),
				NULL
			)
		);
		$this->tpl->parseCurrentBlock();
	}

	protected function populateScoreBlock($reachedPoints, $maxPoints)
	{
		$scoreInformation = sprintf(
			$this->lng->txt( "you_received_a_of_b_points" ), $reachedPoints, $maxPoints
		);

		$this->tpl->setCurrentBlock( "received_points_information" );
		$this->tpl->setVariable("RECEIVED_POINTS_INFORMATION", $scoreInformation);
		$this->tpl->parseCurrentBlock();
	}

	protected function populateSolutionBlock($solutionoutput)
	{
		$this->tpl->setCurrentBlock( "solution_output" );
		$this->tpl->setVariable( "CORRECT_SOLUTION", $this->lng->txt( "tst_best_solution_is" ) );
		$this->tpl->setVariable( "QUESTION_FEEDBACK", $solutionoutput );
		$this->tpl->parseCurrentBlock();
	}
	
	protected function fillQuestionRelatedNavigation(assQuestionGUI $questionGUI)
	{
		global $tpl;

		$parseQuestionRelatedNavigation = false;

		switch( 1 )
		{
			case $this->controller->object->getSpecificAnswerFeedback():
			case $this->controller->object->getGenericAnswerFeedback():
			case $this->controller->object->getAnswerFeedbackPoints():
			case $this->controller->object->getInstantFeedbackSolution():

			$this->showFeedbackButton();
		
			$parseQuestionRelatedNavigation = true;
		}

		if( $parseQuestionRelatedNavigation )
		{
			$tpl->setCurrentBlock("question_related_navigation");
			$tpl->parseCurrentBlock();
		}
	}
	
	public function outFinishSet()
	{
		$this->controller->ensurePermission("read");

		$this->tpl->addCss('./Customizing/global/plugins/Services/Repository/RepositoryObject/FormATest/templates/default/xtsf_player.css');
		$template = new ilTemplate("tpl.il_xtsf_finish_set.html", true, true, "Customizing/global/plugins/Services/Repository/RepositoryObject/FormATest");

		if($this->session->hasNextSet() && !($this->controller->object->isAllowWalkBack() && $this->controller->object->getShowQuestionsOverview()))
		{
			ilUtil::sendInfo($this->plugin->txt("finish_set_information"));
			$this->renderButton($template, "nextset");
		}
		elseif($this->session->allSetsFinished())
		{
			ilFormATestUtil::addInfo($this->plugin->txt("finish_test_information"));
		}
		else
		{
			ilFormATestUtil::addInfo($this->plugin->txt("finish_set_information_choose_list"));
		}
		$this->renderButton($template, "finishtest");

		if($this->controller->object->isAllowInterruption())
		{
			$this->renderButton($template, "interrupt");
		}

		if($this->controller->object->isAllowWalkBack())
		{
			$this->renderButton($template, "back");
		}

		$template->setVariable("FORMACTION", $this->getQuestionPageFormAction());

		$this->renderCurrentResults($template);
		$this->renderSidebar($template, true);

		ilFormATestUtil::send();
		$this->tpl->setContent($template->get());
	}

	/**
	 * @param $template
	 * @param $type
	 */
	protected function renderButton($template, $type)
	{
		$template->setCurrentBlock("{$type}");
		$template->setVariable("CMD", "ilFormATestPlayerControllerGUI.$type");
		$template->setVariable("BTN", $this->plugin->txt($type));
		$template->parseCurrentBlock();

		$template->setCurrentBlock("{$type}_bottom");
		$template->setVariable("CMD", "ilFormATestPlayerControllerGUI.$type");
		$template->setVariable("BTN", $this->plugin->txt($type));
		$template->parseCurrentBlock();
	}

	protected function populateContentStyleBlock()
	{
		include_once("./Services/Style/classes/class.ilObjStyleSheet.php");
		$this->tpl->setCurrentBlock( "ContentStyle" );
		$this->tpl->setVariable( "LOCATION_CONTENT_STYLESHEET",ilObjStyleSheet::getContentStylePath(0));
		$this->tpl->parseCurrentBlock();

		$this->tpl->addCss(ilUtil::getStyleSheetLocation("output", "test_print.css", "Modules/Test"), "print");
		$this->tpl->addCss(ilUtil::getStyleSheetLocation("output", "ta.css", "Modules/Test"), "screen");
	}

	protected function populateSyntaxStyleBlock()
	{
		$this->tpl->setCurrentBlock( "SyntaxStyle" );
		$this->tpl->setVariable( "LOCATION_SYNTAX_STYLESHEET",ilObjStyleSheet::getSyntaxStylePath());
		$this->tpl->parseCurrentBlock();
	}

	/**
	 * @return string
	 */
	protected function getQuestionPageFormAction()
	{
		$this->ctrl->setParameter($this->controller, "active_id", $this->session->getActiveId());
		$this->ctrl->setParameter($this->controller, "xtsf_token", $this->session->getToken());
		$formaction = $this->ctrl->getFormAction($this->controller);
		$this->ctrl->setParameter($this->controller, "active_id", "");
		$this->ctrl->setParameter($this->controller, "xtsf_token", "");

		return $formaction;
	}

	/**
	 * Returns the current tab name
	 * return string
	 */
	protected function getCurrentTab()
	{
	}

	/**
	 * @return array
	 */
	protected function getRequirements()
	{
		return array(
			"models/class.ilFormATestSession.php",
			"models/class.ilFormATestResultList.php",
			"models/class.ilFormATestSetResultList.php",
			"models/class.ilFormATestSetStepList.php",
			"utils/class.ilFormATestParticipantQuestionHeaderBuilder.php",
			"utils/class.ilFormATestParticipantQuestionHeaderBuilder.php",
			"utils/class.ilFormATestAccordionGUI.php",
			"utils/abstract.ilFormATestUtil.php",
			"tables/class.ilFormATestResultsTableGUI.php",
			"tables/class.ilFormATestSetResultTableGUI.php",
			"controllers/class.ilFormATestResultOuputGUI.php"
		);
	}

	/**
	 * @param ilTemplate $template
	 * @param bool $finish_set
	 */
	protected function renderSidebar($template, $finish_set = false)
	{
		if ($this->controller->object->getShowQuestionsOverview())
		{
			$left_template = new ilTemplate('tpl.il_xtsf_result_path.html', true, true, "Customizing/global/plugins/Services/Repository/RepositoryObject/FormATest");

			$sets = $this->controller->object->getSetList()->getData();

			$i = 0;
			foreach ($sets as $set)
			{
				$nodes_model = new ilFormATestSetStepList($this->controller->object, $set["id"], $this->session->getSessionId());
				$nodes = $nodes_model->getData();

				$this->session->reachedDeadlock();
				$step = 0;
				foreach($nodes as $node)
				{
					$title = $this->plugin->txt("header_step") . ": ". ++$step;

					if($this->session->hasFixatedSolution($node["id"]))
					{
						$title .= ' <span class="ilFormATestFixatedSolution">&otimes;</span>';
					}

					if($this->isCurrent($node))
					{
						$title .= " &otimes;";
					}

					if($this->isCurrent($node) || !$this->controller->object->isAllowWalkBack())
					{
						$left_template->setCurrentBlock("unlinked_line");
						//$template->setVariable("QUESTION_INDEX", $node["question_index"]);
						$left_template->setVariable("QUESTION_TITLE", $title);
						$left_template->parseCurrentBlock();
					}
					else
					{
						$left_template->setCurrentBlock("linked_line");
						$left_template->setVariable("LINK", $this->generateJumpToNodeLink($finish_set, $node, array("xtsf_token" => $this->session->getToken())));
						//$template->setVariable("QUESTION_INDEX", $node["question_index"]);
						$left_template->setVariable("QUESTION_TITLE", $title);
						$left_template->parseCurrentBlock();
					}
				}
				$status = $this->session->getSetStatus($set["id"]);
				if($status != null)
				{
					$status = $this->plugin->txt("status_".$status);
				}
				else
				{
					$status = $this->plugin->txt("status_not_started");
				}

				$left_template->setCurrentBlock("set");
				$left_template->setVariable("SET_TITLE", ++$i . ". " . $set["title"]);
				$left_template->setVariable("SET_STATUS", $status);
				$left_template->parseCurrentBlock();
			}
			$left_template->setCurrentBlock("result_path");
			$left_template->setVariable("QUESTIONS_OVERVIEW", $this->plugin->txt("questions_overview_title"));
			$left_template->parseCurrentBlock();

			$GLOBALS['tpl']->setLeftNavContent($left_template->get());
		}
	}

	/**
	 * @param ilTemplate $template
	 */
	protected function renderCurrentResults($template)
	{
		$accordion = new ilFormATestAccordionGUI();
		$accordion->setId('accord_xtsf_' . $this->controller->plugin->getId());

		$accordion->setOrientation(ilFormATestAccordionGUI::HORIZONTAL);
		$accordion->setBehaviour(ilFormATestAccordionGUI::ONE_OPEN_SESSION);

		$accordion->setContentHeight("300px");

		$result_overview = "";
		$i = 0;
		$template->setVariable("RESULT_OVERVIEW_TITLE", $this->plugin->txt("result_overview_title"));
		foreach($this->controller->object->getSetList()->getData() as $set)
		{
			$output = new ilFormATestResultOuputGUI($this->controller);
			$output->initTemplate("tpl.xtsf_tst_question_overview.html");
			$output->setModus(ilFormATestResultOuputGUI::MODUS_PARTICIPANTS);
			$output->renderQuestionDetails($this->session->getSessionId(), array($set), false, $this->session->getNodeId(), false);
			
			$force_open = $this->session->getSetId() == $set["id"];
			$accordion->addItem(++$i, $output->getHTML(), $force_open);
		}
		$result_overview .= $accordion->getHTML();
		$template->setVariable("RESULT_OVERVIEW", $result_overview);
	}

	/**
	 * @param $node
	 *
	 * @return bool
	 */
	protected function isCurrent($node)
	{
		$is_current_node = ($node["id"] == $this->session->getNodeId() && $this->session->getSetStatus() == ilFormATestSession::STATUS_IN_PROGRESS);

		return $is_current_node;
	}

	protected function determineInlineScoreDisplay()
	{
		$show_question_inline_score = FALSE;
		if ($this->controller->object->getAnswerFeedbackPoints())
		{
			$show_question_inline_score = TRUE;
			return $show_question_inline_score;
		}
		return $show_question_inline_score;
	}
}
 