<?php

require_once "abstract.ilXTSFControllerGUI.php";

/**
 * Class ilFormATestPlayerControllerGUI
 *
 * Date: 13.12.13
 * Time: 11:40
 * @author Thomas Joußen <tjoussen@databay.de>
 */ 
class ilFormATestPlayerControllerGUI extends ilXTSFControllerGUI
{

	/**
	 * @var ilFormATestSession
	 */
	protected $session;

	/**
	 * @var ilFormATestOutputGUI
	 */
	protected $output;

	public function __construct(ilObjFormATestGUI $controller)
	{
		parent::__construct($controller);

		$this->session = new ilFormATestSession($controller->object);
		$this->session->read($controller->object->getSetList()->getData());
		$this->output = new ilFormATestOutputGUI($this->controller);
		$this->output->setSession($this->session);
		
		$this->tabs->clearTargets();

		if(!ilFormATestPlayerAccessHelper::canDoTestAction($this->controller, $this->session))
		{
			$this->redirect("ilFormATestInfoScreenControllerGUI.show");
		}
	}

	public function startTest()
	{
		$this->controller->ensurePermission('read');

		if(!$this->session->activeIDExists())
		{
			$this->session->create();
			$this->start();
		}
		else{
			$this->redirectOutPage();
		}
	}

	public function startNewPass()
	{
		$this->controller->ensurePermission("read");

		if($this->session->isPassFinished() || !$this->controller->object->isAllowInterruption())
		{
			$this->session->createNewPass();
			$this->start();
		}
		else
		{
			$this->redirectOutPage();
		}
	}

	public function resumeTest()
	{
		$this->controller->ensurePermission("read");

		if(!$this->session->isPassFinished())
		{
			if(!$this->session->isSetFinished())
			{
				$this->session->setStatus(ilFormATestSession::STATUS_NOT_FINISHED);
				$this->session->update();
				$this->redirectOutPage();
			}
			else
			{
				$this->redirect("ilFormATestPlayerControllerGUI.finish", array(
				   "xtsf_token" => $this->session->getToken()
			   ));
			}
		}
		else
		{
			$this->redirect("ilFormATestInfoScreenControllerGUI.show");
		}
	}

	public function next()
	{
		$this->controller->ensurePermission("read");
		$this->redirectIfTokenIsInvalid();

		$node = $this->getNodeFromSession();
		$status = $this->saveQuestionSolution($node);

		$this->handleSkillTriggering();

		if($status == false)
		{
			if($this->isTokenValid())
			{
				$_SESSION["previouspost"] = $_POST;
				$this->output->outTestPage($node);
			}
			else
			{
				throw new ilException("Access Denied", 403);
			}
			return;
		}

		ilFormATestConditionEvaluation::attach($this->controller->object);
		$next_node = ilFormATestConditionEvaluation::getNextNode($node,$this->session);
		ilFormATestConditionEvaluation::detach($this->controller->object);

		$this->reworkDominantScoring($node);
		$this->session->saveHistory();

		if($next_node == null)
		{
			if($this->session->isJump())
			{
				ilFormATestUtil::addInfo($this->plugin->txt("triggered_termination"));
			}
			$this->redirect("ilFormATestPlayerControllerGUI.finish", array(
				"xtsf_token" => $this->session->getToken()
			));
		}
		if($this->session->isJump())
		{
			$this->session->removeStepsUntil($next_node->getIndex()+1);
			$this->session->updateTrackNodeId($next_node->getId());
			$this->updateTestPassResults($node);
			$this->handleSkillTriggering();
			if($node->getQuestion()->getId() == $next_node->getQuestion()->getId())
			{
				ilFormATestUtil::addInfo($this->plugin->txt("triggered_repitition"));
			}
			else
			{
				ilFormATestUtil::addInfo($this->plugin->txt("triggered_jump"));
			}
		}
		else
		{
			$this->session->step($next_node->getId());
			$this->handleDeadlock();
			$this->handleSkillTriggering();
		}

		$this->session->updateLP();
		$this->redirectOutPage();
	}

	public function prev()
	{
		$this->controller->ensurePermission("read");
		$this->redirectIfTokenIsInvalid();

		if(!$this->session->isFirstStep() && $this->controller->object->isAllowWalkBack())
		{
			$node = $this->getNodeFromSession();
			$hasSolutions = $this->session->stepHasSolution($node);
			if($_GET["confirmed"] || !$hasSolutions)
			{
				if($hasSolutions)
				{
					ilFormATestUtil::addInfo($this->plugin->txt("triggered_prev"));
				}
				else
				{
					ilFormATestUtil::addInfo($this->plugin->txt("triggered_prev_no_delete"));
				}


				$this->session->removeStepsUntil($node->getIndex());

				// delete current_step Feedback
				$current_step = $node->getIndex();
				$this->session->removeFeedbackByStep($current_step);

				$previous_step = ilFormATestConditionEvaluation::getAnchorIndex($node);
				// lösche Feedbacks des Steps, zu dem hingesprungen werden soll.
				$this->session->removeFeedbackByStep($previous_step);

				$this->session->changePath($this->getNodeFromSession(), $node);
				$this->updateTestPassResults($node);
				$this->handleSkillTriggering();
				$this->redirectOutPage();
			}
			else
			{
				$this->renderConfirmationScreen(
					"ilFormATestPlayerControllerGUI.resumeTest",
					"ilFormATestPlayerControllerGUI.prev",
					array(
						array(
							"name" => "n_id[]",
							"id" => $this->session->getNodeId(),
							"text" => $this->plugin->txt("header_step"). ": ". $this->session->getStep()
						)
					),
					$this->plugin->txt("confirm_walk_back"),
					array(
						"xtsf_token" => $this->session->getToken()
					)
				);
			}
		}
		else
		{
			$this->redirectOutPage();
		}
	}

	public function back()
	{
		$this->redirectIfTokenIsInvalid();
		$this->session->updateSetStatus(ilFormATestSession::STATUS_IN_PROGRESS);
		
		$node = $this->getNodeFromSession();
		if($node != null)
		{
			// delete current_step Feedback
			$current_step = $node->getIndex();
			$this->session->removeFeedbackByStep($current_step);

			$previous_step = ilFormATestConditionEvaluation::getAnchorIndex($node);
			// lösche Feedbacks des Steps, zu dem hingesprungen werden soll.
			$this->session->removeFeedbackByStep($previous_step);
		}
		
		$this->redirectOutPage();
	}

	public function interrupt()
	{
		$this->controller->ensurePermission("read");
		$this->redirectIfTokenIsInvalid();

		if($this->controller->object->isAllowInterruption())
		{
			if(!$this->session->isSetFinished())
			{
				$node = $this->getNodeFromSession();
				$this->saveQuestionSolution($node);
				$this->handleSkillTriggering();
				$this->reworkDominantScoring($node);
				$this->session->updateLP();
			}

			$this->session->setStatus(ilFormATestSession::STATUS_INTERRUPTED);
			$this->session->update();
			$this->redirect("ilFormATestInfoScreenControllerGUI.show");
		}
		else
		{
			$this->redirectOutPage();
		}
	}

	public function finish()
	{
		$this->controller->ensurePermission("read");
		$this->redirectIfTokenIsInvalid();
		$this->session->updateSetStatus(ilFormATestSession::STATUS_FINISHED);
		$this->handleSkillTriggering();
		$this->output->outFinishSet();
	}

	public function interruptset()
	{
		$this->controller->ensurePermission("read");
		$this->redirectIfTokenIsInvalid();
		$node = $this->getNodeFromSession();
		$this->saveQuestionSolution($node);
		$this->handleSkillTriggering();
		$this->reworkDominantScoring($node);
		$this->session->updateSetStatus(ilFormATestSession::STATUS_FINISHED);
		$this->session->updateLP();
		$this->output->outFinishSet();
	}

	public function finishTest()
	{
		global $tpl, $ilCtrl, $lng;
		
		$this->controller->ensurePermission("read");

		if((int)$_GET["confirmed"])
		{
			$this->session->setStatus(ilFormATestSession::STATUS_FINISHED);
			$this->session->update();

			ilUtil::sendSuccess($this->plugin->txt("saved_test_data"), true);
			if($this->controller->object->getShowFinalStatement())
			{
				
				$template = new ilTemplate("tpl.xtsf_final_statement.html", true, true, "Customizing/global/plugins/Services/Repository/RepositoryObject/FormATest");
				$template->setVariable("FORMACTION", $ilCtrl->getFormAction($this->controller, 'ilFormATestPlayerControllerGUI.afterTestPassFinished'));
				$template->setVariable("BUTTON_CONTINUE", $lng->txt("btn_next"));
				$template->setVariable("CMD", "ilFormATestPlayerControllerGUI.afterTestPassFinished");
				
				$template->setVariable("FINALSTATEMENT", $this->controller->object->getFinalStatement());
				
				$tpl->setContent($template->get());
			}
			else
			{
				$this->afterTestPassFinished();
			}
		}
		else
		{
			$message = $this->plugin->txt("confirm_finish_test");
			if(!$this->session->allSetsFinished())
			{
				$message = $this->plugin->txt("confirm_finish_test_not_completed");
			}
			$this->renderConfirmationScreen(
				"ilFormATestPlayerControllerGUI.finish",
				"ilFormATestPlayerControllerGUI.finishTest",
				array(),
				$message,
				array(
					"xtsf_token" => $this->session->getToken()
				)
			);
		}
	}
	
	/**
	 * 
	 */
	public function afterTestPassFinished()
	{
		if($this->scoringReportingAlways() || $this->scoringReportingFinished($this->session) || $this->scoringReportingDate())
		{
			$this->redirect("ilFormATestInfoScreenControllerGUI.showTestResults", array(
				"active_id" => $this->session->getActiveId()
			));
		}
		else
		{
			$this->redirect("ilFormATestInfoScreenControllerGUI.show");
		}
	}

	public function nextSet()
	{
		$this->controller->ensurePermission("read");
		$this->redirectIfTokenIsInvalid();

		$this->start();
	}

	public function handleQuestionAction()
	{
		$this->controller->ensurePermission("read");
		$this->redirectIfTokenIsInvalid();

		$node = $this->getNodeFromSession();
		$this->saveQuestionSolution($node);
		$this->handleSkillTriggering();
		$this->reworkDominantScoring($node);
		$this->session->updateLP();
		$this->redirectOutPage();
	}

	public function jumpToNode()
	{
		$this->controller->ensurePermission("read");
		$this->redirectIfTokenIsInvalid();

		$node = new ilQuestionSetPoolNode((int)$_GET["node_id"]);
		$node->read();

		if($_GET["confirmed"] || !$this->session->hasStepsToDelete($node))
		{
			if($this->session->hasStepsToDelete($node))
			{
				ilFormATestUtil::addInfo($this->plugin->txt("triggered_jump_to"));
			}
			else
			{
				ilFormATestUtil::addInfo($this->plugin->txt("triggered_jump_to_no_delete"));
			}

			$current_node = $this->getNodeFromSession();
			$this->session->swapCurrent($node->getPath()->getSet()->getId());
			$this->session->removeStepsUntil($node->getIndex()+1);

			// delete current_step Feedback 
			$current_step = $node->getIndex();
			$this->session->removeFeedbackByStep($current_step);
			
			$previous_step = ilFormATestConditionEvaluation::getAnchorIndex($node);
			// lösche Feedbacks des Steps, zu dem hingesprungen werden soll.  
			$this->session->removeFeedbackByStep($previous_step);
			
			$this->session->changePath($this->getNodeFromSession(), $current_node);
			
			$this->updateTestPassResults($node);
			$this->handleSkillTriggering();
			$this->redirectOutPage();
		}
		else
		{
			$steps = $this->session->getStepsToDelete($node);
			$items = array();
			foreach($steps as $step)
			{
				$items[] = array(
					"name" => "n_id[]",
					"id" => $step["id"],
					"text" => $this->plugin->txt("header_step"). ": ". $step["step"]
				);
			}
			$cancel = "ilFormATestPlayerControllerGUI.back";
			if((int)$_GET["finished"])
			{
				$cancel = "ilFormATestPlayerControllerGUI.finish";
			}

			$this->ctrl->setParameter($this->controller, "node_id", (int)$_GET["node_id"]);
			$this->renderConfirmationScreen(
				$cancel,
				"ilFormATestPlayerControllerGUI.jumpToNode",
				$items,
				$this->plugin->txt("confirm_jump_to_node"),
				array(
					"xtsf_token" => $this->session->getToken()
				)
			);
			$this->ctrl->setParameter($this->controller, "node_id", "");
		}
	}

	public function outPage()
	{
		if($this->isTokenValid())
		{
			$node = $this->getNodeFromSession();
			$this->output->outTestPage($node);
		}
		else
		{
			throw new ilException("Access Denied", 403);
		}
	}

	protected function handleDeadlock()
	{
		if($this->session->hasFixatedSolution())
		{
			$this->session->handleDeadlock();
		}
	}

	/**
	 * @param ilQuestionSetPoolNode $node
	 */
	protected function updateTestPassResults($node)
	{
		assQuestion::setResultGateway(new ilObjTestGateway());
		assQuestion::_updateTestPassResults($this->session->getActiveId(), $this->session->getPass(), false, $node->getQuestion()->getProcessLocker());
		assQuestion::setResultGateway(null);
	}

	/**
	 * @param ilQuestionSetPoolNode $node
	 */
	protected function reworkDominantScoring($node)
	{
		if($node->getDominantScoring() !== null)
		{
			assQuestion::setResultGateway(new ilObjTestGateway());
			$reworker = new ilFormATestDominantScoringReworker(
				$this->session->getActiveId(),
				$this->session->getStep(),
				$this->session->getPass()
			);
			$reworker->rework($node);
			assQuestion::setResultGateway(null);
		}
	}

	protected function redirectOutPage()
	{
		if($this->session->hasFixatedSolution())
		{
			ilFormATestUtil::addInfo($this->plugin->txt("triggered_deadlock"));
		}

		$this->redirect("ilFormATestPlayerControllerGUI.outPage", array(
			"xtsf_token" => $this->session->getToken()
		));
	}

	protected function start()
	{
		$this->session->initCurrent();
		$this->session->step($this->session->getNodeId());
		$this->redirectOutPage();
	}

	/**
	 * @param ilQuestionSetPoolNode $node
	 */
	protected function saveQuestionSolution($node)
	{
		assQuestion::setResultGateway(new ilObjTestGateway());
		$gui = assQuestion::instantiateQuestionGUI($node->getQuestion()->getId());
		$gui->object->setStep($this->session->getStep());
		$saveStatus = true;

		if(!$this->session->hasFixatedSolution())
		{
			$saveStatus = $gui->object->persistWorkingState($this->session->getActiveId(),$this->session->getPass(),false);
		}
		else
		{
			$gui->object->calculateResultsFromSolution($this->session->getActiveId(), $this->session->getPass(), false);
		}
		assQuestion::setResultGateway(null);
		return $saveStatus;
	}

	/**
	 * @return ilQuestionSetPoolNode
	 */
	protected function getNodeFromSession()
	{
		$node = new ilQuestionSetPoolNode($this->session->getNodeId());
		$node->read();

		return $node;
	}

	protected function isTokenValid()
	{
		return $this->session->getToken() == $_GET["xtsf_token"];
	}

	protected function handleSkillTriggering()
	{
		if($this->controller->object->isSkillServiceEnabled() && ilObjFormATest::isSkillManagementGloballyActivated())
		{
			$handler = new ilFormATestSkillLevelHelper();
			$handler->setTestId($this->controller->object->getTestId());
			$handler->setRefId($this->controller->object->getRefId());
			$handler->setSession($this->session);
			$handler->trigger($this->controller->object);
		}
	}

	protected function redirectIfTokenIsInvalid()
	{
		/*if(!$this->isTokenValid())
		{
			ilUtil::sendFailure($this->plugin->txt("multiple_tab_detection"),true);
			$this->redirect("ilFormATestInfoScreenControllerGUI.show");
		}*/
	}

	/**
	 * Returns the current tab name
	 * return string
	 */
	protected function getCurrentTab()
	{
		return "info_short";
	}

	/**
	 * @return array
	 */
	protected function getRequirements()
	{
		require_once "Services/Utilities/classes/class.ilConfirmationGUI.php";
		require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/classes/models/class.ilQuestionSetPoolSet.php";
		require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/classes/models/class.ilQuestionSetPoolPath.php";
		require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/classes/models/class.ilQuestionSetPoolNode.php";

		return array(
			"models/class.ilFormATestSession.php",
			"controllers/class.ilFormATestOutputGUI.php",
			"utils/abstract.ilFormATestConditionEvaluation.php",
			"utils/class.ilObjTestGateway.php",
			"utils/abstract.ilFormATestPlayerAccessHelper.php",
			"utils/class.ilFormATestDominantScoringReworker.php",
			"utils/abstract.ilFormATestUtil.php",
			'utils/class.ilFormATestSkillLevelHelper.php'
		);
	}

	public function showInstantResponse()
	{
		$this->controller->ensurePermission("read");
		
		$this->redirectIfTokenIsInvalid();

		$node = $this->getNodeFromSession();

		if(!$this->session->hasFixatedSolution($node->getQuestion()->getId()))
		{
			$this->saveQuestionSolution($node);
			ilSession::set($this->controller->object->getInstantFeedbackSessionKey(), true);
		}

		$this->session->saveHistory();
		//@todo evtl. update learning progress
//		$this->handleDeadlock();
//		$this->session->updateLP();
		
		$this->redirectOutPage();
	}
}
 
