<?php

require_once "abstract.ilXTSFControllerFormGUI.php";

/**
 * Class ilFormATestPropertiesGUI
 *
 * Date: 07.01.14
 * Time: 09:56
 * @author Thomas Joußen <tjoussen@databay.de>
 */ 
class ilFormATestPropertiesGUI extends ilXTSFControllerFormGUI
{
	/**
	 * @param ilXTSFModel $model
	 * @return mixed|void
	 */
	protected function initForm($model)
	{
		require_once "./Services/Form/classes/class.ilPropertyFormGUI.php";
		$this->form = new ilPropertyFormGUI();
		$this->form->setTitle($this->plugin->txt("edit_properties"));
		$this->form->setTableWidth("100%");
		$this->form->setId("xtsf_properties");
		$this->form->setFormAction($this->ctrl->getFormAction($this->controller));

		if($this->controller->object->getParticipants()->hasParticipants())
		{
			ilUtil::sendInfo($this->plugin->txt("unable_to_edit_settings"));
		}
		else
		{
			$this->form->addCommandButton("ilFormATestPropertiesGUI.update", $this->plugin->txt("save"));
		}

		$title = new ilTextInputGUI($this->plugin->txt("title"), "title");
		$title->setRequired(true);

		$description = new ilTextAreaInputGUI($this->plugin->txt("description"), "description");
		$description->setRequired(false);

		$availability_header = new ilFormSectionHeaderGUI();
		$this->lng->loadLanguageModule('rep');
		$availability_header->setTitle($this->lng->txt("rep_activation_availability"));

		$online = new ilCheckboxInputGUI($this->plugin->txt("online"), "online");
		$online->setDisabled($this->controller->object->getParticipants()->hasParticipants());
		$online->setValue(1);

		$information_header = new ilFormSectionHeaderGUI();
		$information_header->setTitle($this->lng->txt("tst_beginning_ending_information"));

		$sessionheader = new ilFormSectionHeaderGUI();
		$sessionheader->setTitle($this->lng->txt("tst_session_settings"));

		$max_passes = new ilNumberInputGUI($this->lng->txt("tst_nr_of_tries"), "max_passes");
		$max_passes->allowDecimals(false);
		$max_passes->setMinValue(0);
		$max_passes->setSize(3);
		$max_passes->setRequired(true);
		$max_passes->setSuffix($this->lng->txt("0_unlimited"));
		$max_passes->setValue((int)$this->controller->object->getMaxPasses());
		$max_passes->setDisabled($this->controller->object->getParticipants()->hasParticipants());

		$enable_start_time = new ilCheckboxInputGUI($this->plugin->txt("start_time"), "enable_start_time");
		$enable_start_time->setValue(1);
		$enable_start_time->setDisabled($this->controller->object->getParticipants()->hasParticipants());

		$start_time = new ilDateTimeInputGUI('', 'start_time');
		$start_time->setShowTime(true);
		$start_time->setDate(new ilDateTime(time(), IL_CAL_UNIX));
		$start_time->setDisabled($this->controller->object->getParticipants()->hasParticipants());
		$enable_start_time->addSubItem($start_time);

		$enable_end_time = new ilCheckboxInputGUI($this->plugin->txt("end_time"), "enable_end_time");
		$enable_end_time->setValue(1);
		$enable_end_time->setDisabled($this->controller->object->getParticipants()->hasParticipants());

		$end_time = new ilDateTimeInputGUI('', 'end_time');
		$end_time->setShowTime(true);
		$end_time->setDate(new ilDateTime(time(), IL_CAL_UNIX));
		$end_time->setDisabled($this->controller->object->getParticipants()->hasParticipants());
		$enable_end_time->addSubItem($end_time);

		$test_execution_header = new ilFormSectionHeaderGUI();
		$test_execution_header->setTitle($this->lng->txt("tst_test_execution"));

		$allow_walk_back = new ilCheckboxInputGUI($this->plugin->txt("allow_walk_back"), "allow_walk_back");
		$allow_walk_back->setDisabled($this->controller->object->getParticipants()->hasParticipants());
		$allow_walk_back->setValue(1);

		$show_current_track = new ilCheckboxInputGUI($this->plugin->txt("show_current_track"), "show_current_track");
		$show_current_track->setInfo($this->plugin->txt("show_current_track_info"));
		$show_current_track->setValue(1);
		$show_current_track->setDisabled($this->controller->object->getParticipants()->hasParticipants());

		$show_questions_overview = new ilCheckboxInputGUI($this->plugin->txt("show_questions_overview"), "show_questions_overview");
		$show_questions_overview->setInfo($this->plugin->txt("show_questions_overview_info"));
		$show_questions_overview->setValue(1);
		$show_questions_overview->setDisabled($this->controller->object->getParticipants()->hasParticipants());

		$allow_interruption = new ilCheckboxInputGUI($this->plugin->txt("allow_interruption"), "allow_interruption");
		$allow_interruption->setDisabled($this->controller->object->getParticipants()->hasParticipants());
		$allow_interruption->setValue(1);

		$skill_service = new ilCheckboxInputGUI($this->plugin->txt("skill_service_setting"), "skill_service");
		global $ilSetting;
		$skill_setting_enabled = $ilSetting->_lookupValue('skmg','enable_skmg');
		
		$skill_service->setDisabled($this->controller->object->getParticipants()->hasParticipants() || $skill_setting_enabled == false);
		$skill_service->setInfo($this->plugin->txt('adm_enable_skmg_first'));
		$skill_service->setValue(1);

		$this->form->addItem($title);
		$this->form->addItem($description);

		$this->form->addItem($availability_header);
		$this->form->addItem($online);

		$this->form->addItem($information_header);
		$this->createStatementField($model, "intro_statement");
		$this->createStatementField($model, "final_statement");

		$this->form->addItem($sessionheader);
		$this->form->addItem($max_passes);
		$this->form->addItem($enable_start_time);
		$this->form->addItem($enable_end_time);

		$this->form->addItem($allow_walk_back);
		$this->form->addItem($show_current_track);
		$this->form->addItem($show_questions_overview);
		$this->form->addItem($allow_interruption);

		$this->form->addItem($skill_service);
		
		$this->addQuestionBehaviourFormSection($this->form);
	}

	private function addQuestionBehaviourFormSection(ilPropertyFormGUI $form)
	{
		//sequence properties
		$seqheader = new ilFormSectionHeaderGUI();
		$seqheader->setTitle($this->lng->txt("tst_presentation_properties"));
		$this->form->addItem($seqheader);

		// instant feedback
		$instant_feedback = new ilCheckboxGroupInputGUI($this->lng->txt('tst_instant_feedback'), 'instant_feedback');

		$instant_feedback->addOption(new ilCheckboxOption(
			$this->lng->txt('tst_instant_feedback_results'), 'instant_feedback_points',
			$this->lng->txt('tst_instant_feedback_results_desc')
		));
		$instant_feedback->addOption(new ilCheckboxOption(
			$this->lng->txt('tst_instant_feedback_answer_generic'), 'instant_feedback_generic',
			$this->lng->txt('tst_instant_feedback_answer_generic_desc')
		));
		$instant_feedback->addOption(new ilCheckboxOption(
			$this->lng->txt('tst_instant_feedback_answer_specific'), 'instant_feedback_specific',
			$this->lng->txt('tst_instant_feedback_answer_specific_desc')
		));
		$instant_feedback->addOption(new ilCheckboxOption(
			$this->lng->txt('tst_instant_feedback_solution'), 'instant_feedback_solution',
			$this->lng->txt('tst_instant_feedback_solution_desc')
		));
		
		$values = array();
		
		if ($this->controller->object->getSpecificAnswerFeedback()) 
		{
			$values[] = 'instant_feedback_specific';
		}	
		if ($this->controller->object->getGenericAnswerFeedback())
		{
			$values[] = 'instant_feedback_generic';
		}
		if ($this->controller->object->getAnswerFeedbackPoints()) 
		{
			$values[] = 'instant_feedback_points';
		}
		if ($this->controller->object->getInstantFeedbackSolution()) 
		{
			$values[] = 'instant_feedback_solution';
		}
		$instant_feedback->setValue($values);
		$this->form->addItem($instant_feedback);

		// node feedback
		$node_feedback = new ilCheckboxGroupInputGUI($this->plugin->txt('tst_node_feedback'), 'node_feedback');

		// author
		$node_feedback->addOption(new ilCheckboxOption(
			$this->plugin->txt('tst_node_feedback_author'), 'node_feedback_author',
			$this->plugin->txt('tst_node_feedback_author_desc')
		));
		
		// participants
		$node_feedback->addOption(new ilCheckboxOption(
			$this->plugin->txt('tst_node_feedback_part'), 'node_feedback_part',
			$this->plugin->txt('tst_node_feedback_part_desc')
		));
		$node_feedback->addOption(new ilCheckboxOption(
			$this->plugin->txt('tst_node_feedback_part_res'), 'node_feedback_part_res',
			$this->plugin->txt('tst_node_feedback_part_res_desc')
		));
		
		$node_values = array();
	
		if ($this->controller->object->getNodeFeedbackAuthor())
		{
			$node_values[] = 'node_feedback_author';
		}	
		if ($this->controller->object->getNodeFeedbackParticipant())
		{
			$node_values[] = 'node_feedback_part';
		}
		if ($this->controller->object->getNodeFeedbackParticipantRes())
		{
			$node_values[] = 'node_feedback_part_res';
		}
		$node_feedback->setValue($node_values);
		$this->form->addItem($node_feedback);

		// selector for unicode characters
		global $ilSetting;
		if($ilSetting->get('char_selector_availability') > 0)
		{
			// If there will be more presentation settings in future, move this header creation

			require_once 'Services/UIComponent/CharSelector/classes/class.ilCharSelectorGUI.php';
			$char_selector = new ilCharSelectorGUI(ilCharSelectorConfig::CONTEXT_TEST);
			$char_selector->getConfig()->setAvailability($this->controller->object->getCharSelectorAvailability());
			$char_selector->getConfig()->setDefinition($this->controller->object->getCharSelectorDefinition());
			$char_selector->addFormProperties($this->form);
			$char_selector->setFormValues($this->form);

			$refl = new ReflectionObject($char_selector);
			$statics = $refl->getStaticProperties();
			$allowrd_guis   = $statics['allowed_guis'];
			$allowrd_guis[] = 'ilObjFormATestGUI';
			$property = $refl->getProperty('allowed_guis');
			$property->setAccessible(true);
			$property->setValue($allowrd_guis);

			$this->form->getItemByPostVar('char_selector_availability')->setDisabled($this->controller->object->getParticipants()->hasParticipants());
			$this->form->getItemByPostVar('char_selector_blocks')->setDisabled($this->controller->object->getParticipants()->hasParticipants());
			$this->form->getItemByPostVar('char_selector_custom_items')->setDisabled($this->controller->object->getParticipants()->hasParticipants());
		}
	}

	protected function canUpdate()
	{
		return !$this->controller->object->getParticipants()->hasParticipants();
	}


	/**
	 * Returns the current tab name
	 * return string
	 */
	protected function getCurrentTab()
	{
		return "properties";
	}

	/**
	 * @return array
	 */
	protected function getRequirements()
	{
		array();
	}

	/**
	 * Loads and returns the Model for the controller gui
	 * @return ilPluginModelInterface
	 */
	protected function loadModel()
	{
		return $this->controller->object;
	}

	/**
	 * Get the redirect action after an update was successful
	 * @return string
	 */
	protected function getRedirectAfterUpdate()
	{

	}

	/**
	 * Get the redirect action after an update was successful
	 * @return string
	 */
	protected function getAfterUpdateRedirectCmd()
	{
		return "ilFormATestPropertiesGUI.edit";
	}

	/**
	 * @param $model
	 *
	 * @return array
	 */
	protected function getAfterUpdateRedirectParams($model)
	{
		return array();
	}

	/**
	 * @param $model
	 * @param $statement_type
	 */
	protected function createStatementField($model, $statement_type)
	{
		$statement = new ilTextAreaInputGUI($this->plugin->txt($statement_type), $statement_type);
		$statement->setRows(10);
		$statement->setCols(89);
		$statement->setUseRte(true);
		$statement->addPlugin("latex");
		$statement->addButton("latex");
		$statement->setRTESupport($model->getId(), "tst", "assessment");
		$statement->setRteTagSet("full");
		$statement->setDisabled($this->controller->object->getParticipants()->hasParticipants());

		$show_statement = new ilCheckboxInputGUI($this->plugin->txt("{$statement_type}_show"), "show_$statement_type");
		$show_statement->setValue(1);
		$show_statement->setOptionTitle($this->plugin->txt("{$statement_type}_label"));
		$show_statement->setInfo($this->plugin->txt("{$statement_type}_desc"));
		$show_statement->setDisabled($this->controller->object->getParticipants()->hasParticipants());

		$show_statement->addSubItem($statement);
		$this->form->addItem($show_statement);
	}
}
