<?php

require_once "abstract.ilXTSFControllerGUI.php";

/**
 * Class ilFormATestResultOuputGUI
 *
 * Date: 22.01.14
 * Time: 10:38
 * @author Thomas Joußen <tjoussen@databay.de>
 */ 
class ilFormATestResultOuputGUI extends ilXTSFControllerGUI
{

	const MODUS_AUTHOR = "author";
	const MODUS_PARTICIPANTS = "participants";

	/**
	 * @var ilTemplate
	 */
	protected $template;

	/**
	 * @var string
	 */
	protected $modus;

	public function __construct($controller)
	{
		parent::__construct($controller);

		$this->initTemplate("tpl.il_xtsf_result.html");
		$this->modus = self::MODUS_AUTHOR;
	}

	/**
	 * @param string $modus
	 */
	public function setModus($modus)
	{
		$this->modus = $modus;
	}

	/**
	 * @param int $user_id
	 */
	public function renderTestData($user_id)
	{
		$this->template->setVariable("USER", ilUserUtil::getNamePresentation($user_id));
		$this->template->setVariable("DATE_OF_TEST_TITLE", $this->plugin->txt("date_of_test_title"));
		$this->template->setVariable("DATE_OF_TEST", "DATUM ES TESTS");
	}

	/**
	 * @param int $active_id
	 * @param string $detailed_action
	 */
	public function renderPassesOverview($active_id, $detailed_action = "")
	{
		$table = new ilFormATestResultsTableGUI($this->controller, $this->plugin, "");
		$table->setDetailedAction($detailed_action);
		$table->setModelList(new ilFormATestResultList($active_id));
		$table->populate();
		$this->template->setVariable("PASS_OVERVIEW_TITLE", $this->plugin->txt("test_result"));
		$this->template->setCurrentBlock("pass_overview");
		$this->template->setVariable("PASS_OVERVIEW_TABLE", $table->getHTML());
		$this->template->parseCurrentBlock();
	}

	/**
	 * @param int $session_id
	 */
	public function renderStepsOverview($session_id)
	{

		$gateway = new ilObjTestGateway();
		$points = $gateway->getMaxPointsPerSet(ilFormATestSession::getActiveIdBySessionId($session_id));
		$sets = $this->controller->object->getSetList()->getData();
		foreach($sets as $set)
		{
			$table = new ilFormATestSetResultTableGUI($this->controller, $this->plugin, true, $this->modus);
			$table->setTotalMaxPoints($points[$set["id"]]);
			$table->setTitle(sprintf($this->plugin->txt("result_path_for"), $set["title"]));
			$table->setModelList(new ilFormATestSetResultList(
					$this->controller->object,
					$set["id"],
					$session_id
				));
			$table->populate();

			$this->template->setCurrentBlock("step_overview_table");
			$this->template->setVariable("STEP_OVERVIEW_TABLE", $table->getHTML());
			$this->template->parseCurrentBlock();
		}

		$this->template->setCurrentBlock("steps_overview");
		$this->template->setVariable("STEP_OVERVIEW_TITLE", $this->plugin->txt("result_table_header"));
		$this->template->parseCurrentBlock();
	}

	/**
	 * @param int $session_id
	 * @param int $set_id
	 * @param int $node_id
	 */
	public function renderFeedback($session_id,  $set_id, $node_id, $step)
	{
		$feedbacks = $this->controller->object->getNodeFeedbacks($session_id,  $set_id, $node_id, $step);
		
		if(!is_array($feedbacks)) return;

		$this->renderFeedbacks($feedbacks);
	}

	/**
	 * @param int  $session_id
	 * @param array $sets
	 * @param bool $graphical_output
	 * @param int|null $current_node_id
	 * @param bool $is_finished    Flag für die Ausgabe des prozessbezogenen Feedbacks, zur Unterscheidung ob der Test schon abgeschlossen wurde oder nicht. 
	 */
	public function renderQuestionDetails($session_id, $sets, $graphical_output, $current_node_id = null, $is_finished = false)
	{
		$pass = ilFormATestSession::getPassBySessionId($session_id);
		$active_id = ilFormATestSession::getActiveIdBySessionId($session_id);

		$is_feedback_enabled = $this->controller->object->isFeedbackEnabled();
		$is_res_feedback_enabled = $this->controller->object->isResultFeedbackEnabled();
		
		foreach($sets as $set)
		{
			$result_list = new ilFormATestSetResultList($this->controller->object,$set["id"],$session_id);
			$data = $result_list->getData();

			$data_exist = false;

			// feedback output "before FIRST question"
			$feedback_exist =  $this->controller->object->getNodeFeedbacks($session_id,  $set['id'], null, 0);
			if($feedback_exist && $this->modus == self::MODUS_AUTHOR)
			{
				$this->renderFeedbacks($feedback_exist);
				$this->template->setCurrentBlock('path_overview_question');
				$this->template->parseCurrentBlock();
			}
			else if( $feedback_exist && $this->modus == self::MODUS_PARTICIPANTS && $is_feedback_enabled && !$is_finished)
			{
				$this->renderFeedbacks($feedback_exist);
				$this->template->setCurrentBlock('path_overview_question');
				$this->template->parseCurrentBlock();
			}
			else if($feedback_exist &&  $this->modus == self::MODUS_PARTICIPANTS && $is_res_feedback_enabled && $is_finished)
			{
				$this->renderFeedbacks($feedback_exist);
				$this->template->setCurrentBlock('path_overview_question');
				$this->template->parseCurrentBlock();
			}
			// 
			
			foreach($data as $item)
			{
				if($current_node_id == null || $current_node_id != $item["id"])
				{
					if($this->modus == self::MODUS_AUTHOR)
					{
						$this->renderFeedback($session_id, $item["set_fi"], null, $item['step']);
					}	
					else if( $this->modus == self::MODUS_PARTICIPANTS && $is_feedback_enabled && !$is_finished)
					{
						$this->renderFeedback($session_id, $item["set_fi"], $item["id"], $item['step']);
					}
					else if( $this->modus == self::MODUS_PARTICIPANTS && $is_res_feedback_enabled && $is_finished)
					{
						$this->renderFeedback($session_id, $item["set_fi"], null, $item['step']);
					}						
					
					$data_exist = true;
					$question_title = "(Q ". $item["question_index"]. ") ". $item["question_title"];
					if($this->modus == self::MODUS_PARTICIPANTS)
					{
						$question_title = $set["title"] . " - " . $this->plugin->txt("header_step"). ": ". $item["step"];
					}

					$gui = assQuestion::instantiateQuestionGUI($item["question_fi"]);
					$gui->object->setStep($item["step"]);
					$output = $gui->getSolutionOutput($active_id, $pass, $graphical_output,false,true,$graphical_output);

					if($this->modus == self::MODUS_AUTHOR)
					{
						$this->template->setCurrentBlock("condition");
						$this->template->setVariable("PATH_OVERVIEW_QUESTION_INDEX", $item["question_index"]);
						$this->template->setVariable("PATH_OVERVIEW_PATH", $item["title"]);

						if($item["matched_condition"] == "")
						{
							$item["matched_condition"] = $this->plugin->txt("empty_condition");
						}
						$type = $this->plugin->txt("condition_type_initial");
						if($item["matched_condition_id"] != null)
						{
							$type = $this->plugin->txt("condition_type_jump");
						}
						$this->template->setVariable("PATH_OVERVIEW_CONDITION", $item["matched_condition"]);
						$this->template->setVariable("TYPE", $type);
						$this->template->parseCurrentBlock();
					}

					if($graphical_output)
					{
						if($gui->object->getMaximumPoints() != $item["max_points"])
						{
							$this->template->setCurrentBlock("dominant_scoring");
							$this->template->setVariable("POINTS_INFORMATION", "Für diese Antwort wurden Ihnen alternative Punkte zugewiesen.");
							$this->template->setVariable("POSSIBLE_POINTS", sprintf($this->plugin->txt("possible_points"), $gui->object->calculateReachedPoints($active_id, $pass), $gui->object->getMaximumPoints()));
							$this->template->parseCurrentBlock();
						}
					}
					$this->template->setCurrentBlock("path_overview_question");
					$this->template->setVariable("PATH_OVERVIEW_QUESTION", $output);
					$this->template->setVariable("QUESTION_TITLE", $question_title);
					$this->template->setVariable("REACHED_POINTS", sprintf($this->plugin->txt("reached_points"), $item["reached_points"], $item["max_points"]));
					$this->template->parseCurrentBlock();
				}

			}
			$this->template->setCurrentBlock("paths_set_overview");
			$this->template->setVariable("PATH_SET_OVERVIEW_TITLE", sprintf($this->plugin->txt("results_for_set"), $set["title"]));
			$this->template->parseCurrentBlock();

			
			if(!$data_exist)
			{
				$this->template->setCurrentBlock("no_data");
				$this->template->setVariable("NO_DATA", $set["title"] . ": " . $this->plugin->txt("no_data"));
				$this->template->parseCurrentBlock();
			}
		}

		$this->populateContentStyleBlock();
		$this->template->setCurrentBlock("paths_overview");
		$this->template->setVariable("PATH_OVERVIEW_TITLE", $this->plugin->txt("test_result_details"));
		$this->template->parseCurrentBlock();

	}

	public function getHTML()
	{
		return $this->template->get();
	}

	/**
	 * @param string $header
	 */
	public function setHeader($header)
	{
		$this->template->setVariable("TITLE", $header);
	}

	/**
	 * @param string $template_name
	 */
	public function initTemplate($template_name)
	{
		$this->template = new ilTemplate($template_name,true,true,"Customizing/global/plugins/Services/Repository/RepositoryObject/FormATest");
	}

	protected function populateContentStyleBlock()
	{
		include_once("./Services/Style/classes/class.ilObjStyleSheet.php");
		$this->tpl->setCurrentBlock( "ContentStyle" );
		$this->tpl->setVariable( "LOCATION_CONTENT_STYLESHEET",ilObjStyleSheet::getContentStylePath(0));
		$this->tpl->parseCurrentBlock();

		$this->tpl->addCss(ilUtil::getStyleSheetLocation("output", "test_print.css", "Modules/Test"), "print");
		$this->tpl->addCss(ilUtil::getStyleSheetLocation("output", "ta.css", "Modules/Test"), "screen");
	}

	/**
	 * Returns the current tab name
	 * return string
	 */
	protected function getCurrentTab()
	{
		return "";
	}

	/**
	 * @return array
	 */
	protected function getRequirements()
	{
		require_once "Services/User/classes/class.ilUserUtil.php";
		require_once "Modules/TestQuestionPool/classes/class.assQuestion.php";

		return array(
			"tables/class.ilFormATestResultsTableGUI.php",
			"tables/class.ilFormATestSetResultTableGUI.php",
			"models/class.ilFormATestResultList.php",
			"models/class.ilFormATestSetResultList.php",
			"models/class.ilFormATestSession.php",
			"utils/class.ilObjTestGateway.php"
		);
	}

	/**
	 * @param $feedbacks
	 */
	protected function renderFeedbacks($feedbacks)
	{
		foreach($feedbacks as $feedback)
		{
			if($feedback !== NULL && strlen(trim($feedback)) > 0)
			{
				$this->template->setCurrentBlock('node_feedback');
				$this->template->setVariable('NODE_FEEDBACK', ilUtil::insertLatexImages($feedback));
				$this->template->parseCurrentBlock();
			}
		}
	}
}
 