<?php

require_once "abstract.ilXTSFControllerTableGUI.php";

/**
 * Class ilFormATestSetsGUI
 *
 * Date: 23.09.13
 * Time: 14:17
 * @author Thomas Joußen <tjoussen@databay.de>
 */ 
class ilFormATestSetsGUI extends ilXTSFControllerTableGUI
{

	public function listSets()
	{
		$this->controller->ensurePermission("write");

		$session = new ilFormATestSession($this->controller->object);
		$session->read($this->controller->object->getSetList()->getData());

		// @TODO Refactor: Remove setParameter("active_id");
		$has_participants = $this->controller->object->getParticipants()->hasParticipants();
		$table = new ilFormATestSetsTableGUI($this->controller, $this->plugin, "ilFormATestSetsGUI.listSets", true, $has_participants);
		$table->setModelList($this->controller->object->getSetList());

		$this->ctrl->setParameter($this->controller, "active_id", $session->getActiveId());
		$this->initToolbar();
		if(!$has_participants)
		{
			$this->addToolbarButton("browse_for_sets", 'ilFormATestSetsGUI.browseForSets');
		}
		$this->ctrl->setParameter($this->controller, "active_id", "");
		$this->addSessionSensitiveButton($session);

		if($has_participants)
		{
			ilUtil::sendInfo($this->plugin->txt("participants_data_exists"));
		}
		else
		{
			$table->addMultiCommand("ilFormATestSetsGUI.removeSets", $table->lng->txt("remove"));
			$table->addCommandButton("ilFormATestSetsGUI.saveSorting", $this->plugin->txt("save_sorting"));
		}

		$table->populate();

		$this->tpl->setContent($table->getHTML());
	}

	public function browseForSets()
	{
		$this->controller->ensurePermission("write");

		if($this->controller->object->getParticipants()->hasParticipants())
		{
			$this->redirect("ilFormATestSetsGUI.listSets");
		}

		$table = new ilFormATestSetsTableGUI($this->controller, $this->plugin, "ilFormATestSetsGUI.browseForSets");
		$modellist = new ilFormATestQuestionSetPoolSetList($this->controller->object);
		$table->setModelList($modellist);
		$table->addMultiCommand("ilFormATestSetsGUI.insertSets", $table->lng->txt("insert"));
		$table->populate();

		$data = $modellist->getData();
		if($data === null)
		{
			ilUtil::sendInfo($this->plugin->txt("no_access_to_sets"), true);
		}
		elseif(count($data) == 0)
		{
			ilUtil::sendInfo($this->plugin->txt("no_sets_existing"), true);
		}

		$this->tpl->setContent($table->getHTML());
	}

	public function insertSets()
	{
		$this->controller->ensurePermission("write");

		if($this->controller->object->getParticipants()->hasParticipants())
		{
			$this->redirect("ilFormATestSetsGUI.listSets");
		}

		if(!$this->dataSelected("s_id"))
		{
			ilUtil::sendInfo($this->plugin->txt("no_sets_selected"), true);
			$this->redirect("ilFormATestSetsGUI.browseForSets");
		}

		$success = $this->controller->object->getSetList()->insert($_POST["s_id"]);

		if(!$success)
		{
			ilUtil::sendFailure($this->plugin->txt("sets_added_error"), true);
			$this->redirect("ilFormATestSetsGUI.browseForSets");
		}

		ilUtil::sendSuccess($this->plugin->txt("sets_added"), true);
		$this->redirect("ilFormATestSetsGUI.listSets");
	}

	public function removeSets()
	{
		$this->redirectIfUnableToDeleteSets();

		if($_GET["confirmed"])
		{
			$this->controller->object->getSetList()->remove($_POST["s_id"]);
			$this->sortdata();

			ilUtil::sendSuccess($this->plugin->txt("sets_removed"), true);
			$this->redirect("ilFormATestSetsGUI.listSets");
		}
		else
		{
			$items = array();
			foreach($_POST["s_id"] as $set_id)
			{
				$set = new ilQuestionSetPoolSet($set_id);
				$set->read();

				$items[] = array(
					"name" => "s_id[]",
					"id" => $set_id,
					"text" => $set->getTitle()
				);
			}
			$this->renderConfirmationScreen(
				"ilFormATestSetsGUI.listSets",
				"ilFormATestSetsGUI.removeSets",
				$items,
				$this->plugin->txt("remove_selected_sets_confirm")
			);
		}
	}

	public function saveSorting()
	{
		$this->controller->ensurePermission("write");

		if($this->controller->object->getParticipants()->hasParticipants())
		{
			$this->redirect("ilFormATestSetsGUI.listSets");
		}

		$this->sortdata();
		ilUtil::sendSuccess($this->plugin->txt("saved_sorting"), true);
		$this->redirect("ilFormATestSetsGUI.listSets");
	}

	protected function sortdata()
	{
		$data = $this->controller->object->getSetList()->loadData();

		foreach($data as $key => $element)
		{
			$data[$key]["sorting"] = $_POST["sorting"][$key];
		}

		usort($data, function($a,$b){
				return $a["sorting"] - $b["sorting"];
			});

		for($i = 0; $i < count($data); $i++)
		{
			$data[$i]["sorting"] = ($i + 1)* 10;
			$this->controller->object->getSetList()->update($data[$i]);
		}

		return $data;
	}


	protected function redirectIfUnableToDeleteSets()
	{
		$this->controller->ensurePermission("write");

		if (!$this->dataSelected("s_id"))
		{
			ilUtil::sendFailure($this->plugin->txt("no_sets_selected"), true);
			$this->redirect("ilFormATestSetsGUI.listSets");
		}

		if($this->controller->object->getParticipants()->hasParticipants())
		{
			ilUtil::sendFailure($this->plugin->txt("set_not_deleteable_cause_participants"), true);
			$this->redirect("ilFormATestSetsGUI.listSets");
		}
	}

	/**
	 * Returns the current tab name
	 *
	 * return string;
	 */
	protected function getCurrentTab()
	{
		return "sets";
	}

	/**
	 * @return array
	 */
	protected function getRequirements()
	{
		require_once "Services/Utilities/classes/class.ilConfirmationGUI.php";
		require_once "Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/classes/models/class.ilQuestionSetPoolSet.php";

		return array(
			"tables/class.ilFormATestSetsTableGUI.php",
			"models/class.ilFormATestQuestionSetPoolSetList.php",
			"models/class.ilFormATestSession.php",
			"utils/abstract.ilFormATestPlayerAccessHelper.php",
		);
	}
}
