<?php

require_once 'abstract.ilXTSFControllerGUI.php';
require_once 'Modules/Test/classes/class.ilTestSkillLevelThresholdsGUI.php';

/**
 * Class ilFormATestSkillAdministrationGUI
 *
 * @author  Thomas Joußen <tjoussen@databay.de>
 * @since   23.03.15
 *
 * @ilCtrl_Calls ilFormATestSkillAdministrationGUI: ilFormATestAssQuestionSkillAssignmentsGUI
 * @ilCtrl_Calls ilFormATestSkillAdministrationGUI: ilTestSkillLevelThresholdsGUI
 */
class ilFormATestSkillAdministrationGUI extends ilXTSFControllerGUI
{

	/**
	 * @var ilObjFormATest
	 */
	protected $object;

	public function __construct(ilObjFormATestGUI $controller)
	{
		$this->object = $controller->object;

		parent::__construct($controller);
	}

	public function executeCommand()
	{
		global $ilDB, $ilPluginAdmin, $ilAccess;

		if( $this->isAccessDenied() )
		{
			$this->ilias->raiseError($this->lng->txt("permission_denied"), $this->ilias->error_obj->MESSAGE);
		}

		$nextClass = $this->ctrl->getNextClass();
		$this->setSubTabs($nextClass);
		$questionContainerId = $this->object->getId();


		switch($nextClass)
		{
			case 'ilformatestassquestionskillassignmentsgui':
				$helper = new ilFormATestSkillLevelHelper();
				$questionList = $helper->loadQuestionList($this->object);

				$gui = new ilFormATestAssQuestionSkillAssignmentsGUI($this->ctrl, $ilAccess, $this->tpl, $this->lng, $ilDB);
				$gui->setAssignmentEditingEnabled($this->isAssignmentEditingRequired());
				$gui->setQuestionContainerId($questionContainerId);
				$gui->setQuestionList($questionList);
				$gui->setQuestionOrderSequence($questionList->getOrderSequence());
				$this->ctrl->forwardCommand($gui);

				break;
			case 'iltestskilllevelthresholdsgui':
				$gui = new ilTestSkillLevelThresholdsGUI($this->ctrl, $this->tpl, $this->lng, $ilDB, $this->object->getTestId());
				$gui->setQuestionAssignmentColumnsEnabled(false);
				$gui->setQuestionContainerId($questionContainerId);
				$this->ctrl->forwardCommand($gui);
				break;
		}
	}

	private function getRequiredQuestionInstanceTypeFilter()
	{
		return ilAssQuestionList::QUESTION_INSTANCE_TYPE_DUPLICATES;
	}

	protected function isAssignmentEditingRequired()
	{
		return true;
	}

	protected function isAccessDenied()
	{
		if(!$this->object->isSkillServiceEnabled())
		{
			return true;
		}

		if(!ilObjFormATest::isSkillManagementGloballyActivated())
		{
			return true;
		}

		return false;
	}

	/**
	 * Returns the current tab name
	 * return string
	 */
	protected function getCurrentTab()
	{
		return 'tst_tab_competences';
	}

	private function setSubTabs($controllerClass)
	{
		$link = $this->ctrl->getLinkTargetByClass(
			'ilFormATestAssQuestionSkillAssignmentsGUI', ilFormATestAssQuestionSkillAssignmentsGUI::CMD_SHOW_SKILL_QUEST_ASSIGNS
		);
		$this->tabs->addSubTab(
			'ilformatestassquestionskillassignmentsgui', $this->lng->txt('qpl_skl_sub_tab_quest_assign'), $link

		);
		$link = $this->ctrl->getLinkTargetByClass(
			'ilTestSkillLevelThresholdsGUI', ilTestSkillLevelThresholdsGUI::CMD_SHOW_SKILL_THRESHOLDS
		);
		$this->tabs->addSubTab(
			'iltestskilllevelthresholdsgui', $this->lng->txt('tst_skl_sub_tab_thresholds'), $link
		);

		$this->tabs->activateSubTab($controllerClass);
	}

	/**
	 * @return array
	 */
	protected function getRequirements()
	{
		return array(
			'utils/class.ilFormATestSkillLevelHelper.php',
			'utils/class.ilFormATestAssQuestionSkillAssignmentsGUI.php',
		);
	}
}
 