<?php

/**
 * Class ilXTSFPluginDispatcher
 *
 * Date: 23.09.13
 * Time: 14:32
 * @author Thomas Joußen <tjoussen@databay.de>
 */ 
class ilXTSFPluginDispatcher {

	/**
	 * @var ilXTSFPluginDispatcher
	 */
	private static $_instance = null;

	/**
	 * @param ilObjectPluginGUI $baseController
	 *
	 * @return ilXTSFPluginDispatcher
	 */
	public static function getInstance($baseController)
	{
		if(self::$_instance == null)
		{
			self::$_instance = new ilXTSFPluginDispatcher($baseController);
		}

		return self::$_instance;
	}

	/**
	 * @param string $cmd
	 *
	 * @return mixed
	 */
	public function dispatch($cmd)
	{
		$controller = $this->getController($cmd);
		$command = $this->getCommand($cmd);

		if(!is_object($controller))
		{
			$controller = $this->instantiateController($controller);
		}

		$controller->$command();
	}

	/**
	 * @param string $cmd
	 *
	 * @return string|ilPluginControllerGUI
	 */
	public function getController($cmd)
	{
		$controller = substr($cmd, 0, strpos($cmd, "."));

		if($controller == "")
		{
			$controller = $this->baseController;
		}

		return $controller;
	}

	/**
	 * @param string $cmd
	 *
	 * @return string
	 */
	protected function getCommand($cmd)
	{
		$pos = strpos($cmd, ".");

		if($pos > 0)
		{
			$cmd = substr($cmd, $pos + 1);
		}

		return $cmd;
	}

	/**
	 * @param string $controller
	 *
	 * @return ilPluginControllerGUI
	 */
	protected function instantiateController($controller)
	{
		$this->requireController($controller);

		return new $controller($this->baseController);
	}

	/**
	 * @return string
	 */
	protected function getControllerPath()
	{
		$path = $this->baseController->plugin->getDirectory() .
			DIRECTORY_SEPARATOR .
			"classes" .
			DIRECTORY_SEPARATOR .
			"controllers" .
			DIRECTORY_SEPARATOR;

		return $path;
	}

	/**
	 * @param $controller
	 */
	protected function requireController($controller)
	{
		require_once $this->getControllerPath() . "class.$controller.php";
	}

	private function __construct($baseController){$this->baseController = $baseController;}
	private function __clone(){}
}
