<?php

require_once "abstract.ilXTSFModelList.php";

/**
 * Class ilFormATestParticipanstList
 *
 * Date: 20.01.14
 * Time: 15:57
 * @author Thomas Joußen <tjoussen@databay.de>
 */ 
class ilFormATestParticipantsList extends ilXTSFModelList
{

	/**
	 * @return array
	 */
	public function loadData()
	{
		require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/FormATest/classes/providers/class.ilFormATestParticipantsProvider.php";

		$provider = new ilFormATestParticipantsProvider();
		return $provider->loadData(array(
			"test_id" => $this->form_a_test->getTestId()
		));
	}

	public function removeAll()
	{
		$active_ids = $this->getAllActiveIds();
		$session_ids = $this->getAllSessionIdsByActiveIds($active_ids);

		$this->removeByActiveIdsAndSessionIds($active_ids, $session_ids);
	}

	/**
	 * @param array $session_ids
	 */
	public function removeBySessionIds($session_ids)
	{
		$active_ids = $this->getAllActiveIdsBySessionIds($session_ids);

		$this->removeByActiveIdsAndSessionIds($active_ids, $session_ids);
	}

	/**
	 * @return bool
	 */
	public function hasParticipants()
	{
		return count($this->getData()) > 0;
	}


	/**
	 * @param array $data
	 *
	 * @return array
	 */
	public function createObjectsFromData($data)
	{
		return $data;
	}

	/**
	 * @return array
	 */
	protected function getAllActiveIds()
	{
		$active_ids = array();

		$result = $this->db->queryF(
			"SELECT active_id FROM tst_active WHERE test_fi = %s",
			array("integer"),
			array($this->form_a_test->getTestId())
		);

		while ($row = $this->db->fetchAssoc($result))
		{
			$active_ids[] = $row["active_id"];
		}

		return $active_ids;
	}

	/**
	 * @param array $active_ids
	 *
	 * @return array
	 */
	protected function getAllSessionIdsByActiveIds($active_ids)
	{
		$session_ids = array();

		$in_active_id = $this->db->in("active_id", $active_ids, false, "integer");
		$query   = "SELECT session_id FROM rep_robj_xtsf_session WHERE $in_active_id";
		$results = $this->db->query($query);
		while ($row = $this->db->fetchAssoc($results))
		{
			$session_ids[] = $row["session_id"];
		}

		return $session_ids;
	}

	/**
	 * @param array $session_ids
	 *
	 * @return array
	 */
	protected function getAllActiveIdsBySessionIds($session_ids)
	{
		$active_ids = array();

		$in_session_id = $this->db->in("session_id", $session_ids, false, "integer");
		$query   = "SELECT active_id FROM rep_robj_xtsf_session WHERE $in_session_id";
		$results = $this->db->query($query);
		while ($row = $this->db->fetchAssoc($results))
		{
			$active_ids[] = $row["active_id"];
		}

		return $active_ids;
	}

	/**
	 * @param $active_ids
	 * @param $session_ids
	 */
	protected function removeByActiveIdsAndSessionIds($active_ids, $session_ids)
	{
		$in_active_id  = $this->db->in("active_id", $active_ids, false, "integer");
		$in_session_id = $this->db->in("session_fi", $session_ids, false, "integer");
		$in_active_fi  = $this->db->in("active_fi", $active_ids, false, "integer");

		$query = "DELETE FROM rep_robj_xtsf_session WHERE $in_active_id";
		$this->db->query($query);

		$query = "DELETE FROM rep_robj_xtsf_tracking WHERE $in_session_id";
		$this->db->query($query);

		$query = "DELETE FROM rep_robj_xtsf_trck_his WHERE $in_session_id";
		$this->db->query($query);

		$query = "DELETE FROM rep_robj_xtsf_set_trck WHERE $in_session_id";
		$this->db->query($query);

		$query = "DELETE FROM rep_robj_xtsf_solut WHERE $in_active_fi";
		$this->db->query($query);

		$query = "DELETE FROM tst_active WHERE $in_active_id";
		$this->db->query($query);

		$query = "DELETE FROM tst_result_cache WHERE $in_active_fi";
		$this->db->query($query);

		$query = "DELETE FROM tst_test_result WHERE $in_active_fi";
		$this->db->query($query);

		$query = "DELETE FROM tst_pass_result WHERE $in_active_fi";
		$this->db->query($query);

		$query = "DELETE FROM tst_solutions WHERE $in_active_fi";
		$this->db->query($query);

		$this->db->manipulate('DELETE FROM rep_robj_xtsf_fbt WHERE '.$this->db->in("session_id", $session_ids, false, "integer"));
	}
}
 