<?php

require_once "class.ilFormATestSession.php";

/**
 * Class ilFormATestSessionPosition
 *
 * Date: 25.02.14
 * Time: 09:02
 * @author Thomas Joußen <tjoussen@databay.de>
 */ 
class ilFormATestSessionSet
{

	/**
	 * @var int
	 */
	protected $set_id;

	/**
	 * @var int
	 */
	protected $node_id;

	/**
	 * @var int
	 */
	protected $step;

	/**
	 * @var int
	 */
	protected $condition_id;

	/**
	 * @var string
	 */
	protected $condition;

	/**
	 * @var int
	 */
	protected $condition_node_id;

	/**
	 * @var bool
	 */
	protected $jump;

	/**
	 * @var string
	 */
	protected $status;

	/**
	 * @var int
	 */
	protected $session_id;

	/**
	 * @var ilDB
	 */
	protected $db;

	/**
	 * @param int $set_id
	 */
	public function __construct($set_id)
	{
		global $ilDB;

		$this->set_id = $set_id;
		$this->db = $ilDB;
		$this->step = 0;
	}

	/**
	 * @return int
	 */
	public function getStep()
	{
		return $this->step;
	}

	/**
	 * @return bool
	 */
	public function isJump()
	{
		return $this->jump;
	}

	/**
	 * @param bool $is_jump
	 */
	public function setJump($is_jump)
	{
		$this->jump = $is_jump;
	}

	/**
	 * @return bool
	 */
	public function isFinished()
	{
		return $this->status == ilFormATestSession::STATUS_FINISHED;
	}

	/**
	 * @return bool
	 */
	public function isCurrent()
	{
		return $this->status == ilFormATestSession::STATUS_IN_PROGRESS;
	}

	/**
	 * @return bool
	 */
	public function isNotStarted()
	{
		return $this->status == ilFormATestSession::STATUS_NOT_STARTED;
	}

	/**
	 *
	 */
	public function read()
	{
		$result = $this->db->queryF(
			"SELECT * FROM rep_robj_xtsf_set_trck trck
			LEFT JOIN rep_robj_xtsf_tracking tracking ON trck.session_fi = tracking.session_fi AND trck.set_fi = tracking.set_fi AND tracking.step = (
				SELECT MAX(step) FROM rep_robj_xtsf_tracking WHERE set_fi = %s AND session_fi = %s
			)
			WHERE trck.set_fi = %s AND trck.session_fi = %s ",
			array("integer", "integer", "integer", "integer"),
			array($this->set_id, $this->session_id, $this->set_id, $this->session_id)
		);

		if($result->numRows())
		{
			$row = $this->db->fetchAssoc($result);
			$this->step = $row["step"];
			$this->node_id = $row["node_fi"];
			$this->condition = $row["matched_condition"];
			$this->condition_id = $row["matched_condition_id"];
			$this->condition_node_id = $row["matched_condition_node_id"];
			$this->status = $row["status"];
		}
		else
		{
			$this->step = 0;
			$this->node_id = null;
			$this->condition = null;
			$this->condition_id = null;
			$this->condition_node_id = null;
			$this->status = null;
		}
	}

	/**
	 * @param int $node_id
	 */
	public function setNodeId($node_id)
	{
		$this->node_id = $node_id;
	}

	/**
	 * @return int
	 */
	public function getNodeId()
	{
		return $this->node_id;
	}

	/**
	 * @param int $session_id
	 */
	public function setSessionId($session_id)
	{
		$this->session_id = $session_id;
	}

	/**
	 * @return int
	 */
	public function getSetId()
	{
		return $this->set_id;
	}

	/**
	 * @TODO Refactor: Rename to step()
	 */
	public function save()
	{
		$this->step = $this->getStepByNodeId($this->node_id); 

		$this->db->insert(
			"rep_robj_xtsf_tracking",
			array(
				"session_fi" => array("integer", $this->session_id),
				"step" => array("integer", $this->step),
				"node_fi" => array("integer", $this->node_id),
				"set_fi" => array("integer", $this->set_id)
			)
		);
	}

	/**
	 * @param int $node_id
	 */
	public function getStepByNodeId($node_id)
	{
		global $ilDB;
		
		$res = $ilDB->queryF('SELECT node_index FROM rep_robj_xqsp_node WHERE id = %s',
			array('integer'), array($node_id));
		
		$step_index = 0;
		while($row = $ilDB->fetchAssoc($res))
		{
			$step_index = $row['node_index'];
		}	
		
		return (int)$step_index;
	}
	
	/**
	 * Initialize the first node of the set
	 */
	public function init()
	{
		$this->db->setLimit(1, 0);

		$result = $this->db->queryF(
			"SELECT node.id FROM rep_robj_xqsp_node node
			INNER JOIN rep_robj_xqsp_path path ON path.id = node.path_fi
			WHERE path.questionset_fi = %s AND node.node_index = %s
			ORDER BY path.path_index",
			array("integer", "integer"),
			array($this->set_id, 1)
		);

		$data = $this->db->fetchAssoc($result);
		$this->node_id = $data["id"];

		$this->status = ilFormATestSession::STATUS_IN_PROGRESS;
		$this->update();
	}

	/**
	 * @param string $condition
	 */
	public function setCondition($condition)
	{
		$this->condition = $condition;
	}

	/**
	 * @param int $condition_id
	 */
	public function setConditionId($condition_id)
	{
		$this->condition_id = $condition_id;
		if($condition_id != null)
		{
			$this->jump = true;
		}
	}

	/**
	 * @param int $condition_node_id
	 */
	public function setConditionNodeId($condition_node_id)
	{
		$this->condition_node_id = $condition_node_id;
	}

	/**
	 * @param string $status
	 */
	public function setStatus($status)
	{
		$this->status = $status;
	}

	/**
	 * @return string
	 */
	public function getStatus()
	{
		return $this->status;
	}

	public function create()
	{
		$this->db->insert(
			"rep_robj_xtsf_set_trck",
			array(
				"status" => array("text", $this->status),
				"session_fi" => array("integer", $this->session_id),
				"set_fi" => array("integer", $this->set_id)
			)
		);
	}

	public function update()
	{
		$this->db->update(
			"rep_robj_xtsf_set_trck",
			array(
				"status" => array("text", $this->status)
			),
			array(
				"session_fi" => array("integer", $this->session_id),
				"set_fi" => array("integer", $this->set_id)
			)
		);

		$this->db->update(
			"rep_robj_xtsf_tracking",
			array(
				"matched_condition" => array("text", $this->condition),
				"matched_condition_id" => array("integer", $this->condition_id),
				"matched_condition_node_id" => array("integer", $this->condition_node_id)
			),
			array(
				"session_fi" => array("integer", $this->session_id),
				"set_fi" => array("integer", $this->set_id),
				"step" => array("integer", $this->step)
			)
		);
	}

	/**
	 * @param int $index
	 */
	public function removeUntil($index)
	{
		$steps = $this->getStepsByIndex($index);
		foreach($steps as $step)
		{
			$this->removeTstSolutionsByStep($step, $index);
			$this->removeTstTestResultByStep($step, $index);
			$this->removeTrackingByStep($step);
			$this->removeFeedbackByStep($step);
		}
		$this->read();
	}

	/**
	 * @param null|int $node_id
	 *
	 * @return bool
	 */
	public function hasFixatedSolution($node_id = null)
	{
		if($node_id == null)
		{
			$node_id = $this->node_id;
		}

		$result = $this->db->queryF(
			"SELECT COUNT(solution_id) num FROM rep_robj_xtsf_solut sol
			INNER JOIN rep_robj_xqsp_node node ON node.question_fi = sol.question_fi
			INNER JOIN rep_robj_xtsf_session sess ON sess.active_id = sol.active_fi AND sess.current_pass = sol.pass
			WHERE sess.session_id = %s AND node.id = %s AND sol.fixated = %s",
			array("integer", "integer", "integer"),
			array($this->session_id, $node_id, 1)
		);

		$row = $this->db->fetchAssoc($result);
		return $row["num"] > 0;
	}

	/**
	 * @return bool
	 */
	public function isDeadlockPossible()
	{
		$this->db->setLimit(1);

		$result = $this->db->queryF(
			"SELECT COUNT(his.matched_condition_id) >= con.repititions possible
				FROM rep_robj_xqsp_jmp_con con
				  LEFT JOIN rep_robj_xtsf_trck_his his ON con.id = his.matched_condition_id AND his.session_fi = %s
				  INNER JOIN rep_robj_xqsp_node node ON node.id = con.node_fi
				WHERE con.repititions IS NOT NULL AND node.question_fi = (
				  SELECT question_fi FROM
					rep_robj_xqsp_node node2
				  WHERE node2.id = %s
				)
				GROUP BY con.id, con.repititions ORDER BY possible DESC ",
			array("integer", "integer"),
			array($this->session_id, $this->node_id)
		);

		$row = $this->db->fetchAssoc($result);

		return $row != null && $row["possible"] > 0;
	}

	/**
	 * @return bool
	 */
	public function isDeadlock()
	{
		$this->db->setLimit(1);

		$result = $this->db->queryF(
			"SELECT COUNT(his.matched_condition_id) >= con.repititions possible
				FROM rep_robj_xqsp_jmp_con con
				  LEFT JOIN rep_robj_xtsf_trck_his his ON con.id = his.matched_condition_id AND his.session_fi = %s
				  INNER JOIN rep_robj_xqsp_node node ON node.id = con.node_fi
				WHERE con.repititions IS NOT NULL AND node.question_fi = (
				  SELECT question_fi FROM
					rep_robj_xqsp_node node2
				  WHERE node2.id = %s
				) AND con.id = %s
				GROUP BY con.id, con.repititions ORDER BY possible DESC ",
			array("integer", "integer", "integer"),
			array($this->session_id, $this->node_id, $this->condition_id)
		);

		$row = $this->db->fetchAssoc($result);

		return $row != null && $row["possible"] > 0;
	}

	/**
	 * @param int $node_id
	 */
	public function updateTrackNodeId($node_id)
	{
		$this->db->update(
			"rep_robj_xtsf_tracking",
			array(
				"node_fi" => array("integer", $node_id)
			),
			array(
				"session_fi" => array("integer", $this->session_id),
				"step" => array("integer", $this->step),
				"set_fi" => array("integer", $this->set_id),
				"node_fi" => array("integer", $this->node_id)
			)
		);
	}

	/**
	 * @param int $index
	 *
	 * @return array
	 */
	protected function getStepsByIndex($index)
	{
		$result = $this->db->queryF(
			"SELECT step, node.node_index n_index FROM rep_robj_xtsf_tracking track
			INNER JOIN rep_robj_xqsp_node node ON node.id = track.node_fi
			WHERE track.set_fi = %s AND track.session_fi = %s AND node.node_index >= %s",
			array("integer", "integer", "integer"),
			array($this->set_id, $this->session_id, $index)
		);

		$steps = array();
		while($row = $this->db->fetchAssoc($result))
		{
			$steps[$row["n_index"]] = $row["step"];
		}
		return $steps;
	}

	/**
	 * @param int $step
	 */
	protected function removeTstSolutionsByStep($step)
	{
		$this->db->manipulateF(
			"DELETE tst_solutions FROM tst_solutions
			INNER JOIN rep_robj_xtsf_session ON rep_robj_xtsf_session.active_id = tst_solutions.active_fi
			INNER JOIN rep_robj_xqsp_node ON rep_robj_xqsp_node.question_fi = tst_solutions.question_fi
			INNER JOIN rep_robj_xqsp_path ON rep_robj_xqsp_path.id = rep_robj_xqsp_node.path_fi
			WHERE rep_robj_xtsf_session.session_id = %s AND tst_solutions.step = %s AND rep_robj_xqsp_path.questionset_fi = %s",
			array("integer", "integer", "integer"),
			array($this->session_id, $step, $this->set_id)
		);
	}

	/**
	 * @param int $step
	 */
	protected function removeTstTestResultByStep($step)
	{
		$this->db->manipulateF(
			"DELETE tst_test_result FROM tst_test_result
			INNER JOIN rep_robj_xtsf_session ON rep_robj_xtsf_session.active_id = tst_test_result.active_fi
			INNER JOIN rep_robj_xqsp_node ON rep_robj_xqsp_node.question_fi = tst_test_result.question_fi
			INNER JOIN rep_robj_xqsp_path ON rep_robj_xqsp_path.id = rep_robj_xqsp_node.path_fi
			WHERE rep_robj_xtsf_session.session_id = %s AND tst_test_result.step = %s AND rep_robj_xqsp_path.questionset_fi = %s",
			array("integer", "integer", "integer"),
			array($this->session_id, $step, $this->set_id)
		);
	}

	/**
	 * @param int $step
	 */
	protected function removeTrackingByStep($step)
	{
		$this->db->manipulateF(
			"DELETE rep_robj_xtsf_tracking FROM rep_robj_xtsf_tracking
			WHERE session_fi = %s AND step = %s AND set_fi = %s",
			array("integer", "integer","integer"),
			array($this->session_id, $step, $this->set_id)
		);
	}

	/**
	 * @param int $step
	 */
	protected function removeFeedbackByStep($step)
	{
		$this->db->manipulateF(
			'DELETE rep_robj_xtsf_fbt FROM rep_robj_xtsf_fbt 
			WHERE step = %s
			AND session_id = %s 
			AND set_id = %s',
			array('integer', 'integer', 'integer'),
			array($step, $this->session_id, $this->set_id)
		);
	}

}
 