<?php

require_once "abstract.ilXTSFModelList.php";
require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/classes/models/class.ilQuestionSetPoolSet.php";

/**
 * Class ilFormATestSetList
 *
 * Date: 10.12.13
 * Time: 15:33
 * @author Thomas Joußen <tjoussen@databay.de>
 */ 
class ilFormATestSetList extends ilXTSFModelList
{

	/**
	 * @var ilLanguage
	 */
	protected $lng;

	public function __construct(ilObjFormATest $form_a_test)
	{
		global $lng;

		$this->lng = $lng;

		parent::__construct($form_a_test);
	}


	/**
	 * @return array
	 */
	public function loadData()
	{
		require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/FormATest/classes/providers/class.ilFormATestSetProvider.php";

		$provider = new ilFormATestSetProvider();
		return $provider->loadData(array(
				"form_a_test" => $this->form_a_test->getId()
		));
	}

	/**
	 * @param array $set_ids
	 * @return bool
	 */
	public function insert($set_ids, $do_clone = true)
	{
		require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/classes/class.ilObjQuestionSetPool.php";

		foreach($set_ids as $set_id)
		{
			$set = new ilQuestionSetPoolSet($set_id);
			$set->read();

			if($set != null )
			{
				if($do_clone)
				{
					$clone = $set->duplicate($this->form_a_test->getId());
					$container_id = $clone->getId(); 
				}
				else
				{
					$container_id = $set->getId();
				}
				$success = $this->db->queryF(
					"INSERT INTO rep_robj_xtsf_qs (obj_fi, questionset_fi, sorting)
					(SELECT %s,%s,
					CASE WHEN MAX(sorting) IS NULL
						THEN 10
						ELSE MAX(sorting) + 10
					END
					FROM rep_robj_xtsf_qs WHERE obj_fi = %s)",
					array("integer", "integer", "integer"),
					array($this->form_a_test->getId(), $container_id, $this->form_a_test->getId())
				);

				if(!$success)
				{
					return false;
				}

				$this->form_a_test->logAction($this->lng->txt("rep_robj_xtsf_log_set_added"), $set_id);
			}
		}
		return true;
	}

	public function remove($set_ids)
	{
		foreach($set_ids as $set_id)
		{
			$set = new ilQuestionSetPoolSet($set_id);
			$set->read();

			$set->remove();
			$sucess = $this->db->manipulateF(
				"DELETE FROM rep_robj_xtsf_qs WHERE obj_fi = %s AND questionset_fi = %s",
				array("integer", "integer"),
				array($this->form_a_test->getId(), $set_id)
			);

			if(!$sucess)
			{
				return false;
			}

			$this->form_a_test->logAction($this->lng->txt("rep_robj_xtsf_log_set_removed"), $set_id);
		}
		return true;
	}

	public function removeAll()
	{
		$data = $this->loadData();

		foreach($data as $item)
		{
			$set = new ilQuestionSetPoolSet($item["id"]);
			$set->read();
			$set->remove();

			$this->db->manipulateF(
				"DELETE FROM rep_robj_xtsf_qs WHERE obj_fi = %s AND questionset_fi = %s",
				array("integer", "integer"),
				array($this->form_a_test->getId(), $set->getId())
			);
		}
	}

	public function update($data)
	{
		return $this->db->update(
			"rep_robj_xtsf_qs",
			array(
				"sorting" => array("integer", $data["sorting"])
			),
			array(
				"obj_fi" => array("integer", $this->form_a_test->getId()),
				"questionset_fi" => array("integer", $data["id"])
			)
		);
	}

	public function getSet($id)
	{
		$data = $this->getData();

		foreach($data as $item)
		{
			if($item["id"] == $id)
			{
				return $item;
			}
		}

		return null;
	}

	/**
	 * @param array $data
	 *
	 * @return array
	 */
	public function createObjectsFromData($data)
	{
		return $data;
	}
}
 