<?php

require_once "interface.ilXTSFTableDataProvider.php";

/**
 * Class ilXTSFTableDatabaseDataProvider
 *
 * Date: 24.09.13
 * Time: 09:00
 * @author Thomas Joußen <tjoussen@databay.de>
 */ 
abstract class ilXTSFTableDatabaseDataProvider implements ilXTSFTableDataProvider{

	/**
	 * @var ilDB
	 */
	protected $db;

	/**
	 * @var int
	 */
	protected $limit = 0;

	/**
	 * @var int
	 */
	protected $offset = 0;

	/**
	 * @var string
	 */
	protected $query = "";

	/**
	 *
	 */
	public function __construct()
	{
		global $ilDB;

		$this->db = $ilDB;
	}

	/**
	 * @param $limit
	 *
	 * @throws InvalidArgumentException
	 */
	public function setLimit($limit)
	{
		if (!is_numeric($limit))
		{
			throw new InvalidArgumentException('Please provide a valid numerical limit.');
		}
		$this->limit = $limit;
	}

	/**
	 * @return int
	 */
	public function getLimit()
	{
		return $this->limit;
	}


	/**
	 * @param $offset
	 *
	 * @throws InvalidArgumentException
	 */
	public function setOffset($offset)
	{
		if (!is_numeric($offset))
		{
			throw new InvalidArgumentException('Please provide a valid numerical offset.');
		}

		$this->offset = $offset;
	}

	/**
	 * @return int
	 */
	public function getOffset()
	{
		return $this->offset;
	}

	/**
	 * @param array $filter
	 * @param array $orderBy
	 *
	 * @return array
	 */
	public function loadData(array $filter, array $orderBy = array())
	{
		$data = array();

		$this->db->setLimit($this->limit, $this->offset);
		$this->buildQuery($filter, $orderBy);

		$res = $this->db->query($this->query);

		while ($row = $this->db->fetchAssoc($res))
		{
			$data[] = $row;
		}

		return $data;
	}

	/**
	 * @param array $filter
	 * @param array $orderBy
	 *
	 * @return string
	 */
	protected function buildQuery(array $filter, array $orderBy)
	{
		$select = $this->getSelectPart();
		$where  = $this->getWherePart($filter);
		$from   = $this->getFromPart();
		$join   = $this->getJoinPart($filter);
		$order  = $this->getOrderByPart($orderBy);
		$group  = $this->getGroupByPart();
		$having = $this->getHavingPart($filter);

		$where = strlen($where)? 'WHERE ' . $where : '';
		$query = "SELECT {$select} FROM {$from} {$join} {$where}";

		if (strlen($group))
		{
			$query .= " GROUP BY {$group}";
		}

		if (strlen($having))
		{
			$query .= " HAVING {$having}";
		}

		if (strlen($order))
		{
			$query .= " ORDER BY {$order}";
		}

		$this->query = $query;
	}

	/**
	 * @return string
	 * @abstract
	 */
	abstract protected function getSelectPart();

	/**
	 * @return string
	 * @abstract
	 */
	abstract protected function getFromPart();

	/**
	 * @param array $filter
	 *
	 * @return string
	 * @abstract
	 */
	abstract protected function getJoinPart(array $filter);

	/**
	 * @param array $filter
	 * @return string
	 * @abstract
	 */
	abstract protected function getWherePart(array $filter);

	/**
	 * @return string
	 * @abstract
	 */
	abstract protected function getGroupByPart();

	/**
	 * @param array $filter
	 * @return string
	 * @abstract
	 */
	abstract protected function getHavingPart(array $filter);

	/**
	 * @param array $params
	 * @return string
	 * @abstract
	 */
	abstract protected function getOrderByPart(array $params);
}
