<?php

require_once "abstract.ilXTSFTableGUI.php";

/**
 * Class ilFormATestParticipantsTableGUI
 *
 * Date: 07.10.13
 * Time: 13:49
 * @author Thomas Joußen <tjoussen@databay.de>
 */ 
class ilFormATestParticipantsTableGUI extends ilXTSFTableGUI
{

	public function __construct($controller, $plugin, $action = "")
	{
		$this->setId("xtsf_participants_" . $controller->object->getId());

		parent::__construct($controller, $action, $plugin);

		$this->createTable();
	}

	private function createTable()
	{
		$this->setTitle($this->plugin->txt("participants"));

		$this->setFormName('participants');
		$this->setStyle('table', 'fullwidth');
		$this->setDefaultOrderField("title");
		$this->setDefaultOrderDirection("ASC");

		$this->addColumn('', '', '1%', true);
		$this->addColumn($this->lng->txt("username"), 'username', '');
		$this->addColumn($this->lng->txt("name"), 'name', '');
		$this->addColumn($this->plugin->txt("test_passes"), 'test_passes', '');
		$this->addColumn($this->plugin->txt("last_access"), 'last_access', '');
		$this->addColumn('', '', '15%', true);

		$this->setRowTemplate("tpl.il_xtsf_participants_row.html", $this->parent_obj->plugin->getDirectory());

		$this->setFormAction($this->ctrl->getFormAction($this->parent_obj));

		$this->setSelectAllCheckbox("s_id");
		$this->disable("sort");
		$this->enable('header');
	}

	/**
	 * Return an array of all static (always visible) data fields in a row.
	 * For each key there has to be a variable name VAL_<COLUMN_KEY> in your defined row template.
	 * Example:
	 *     return array('title', 'checkbox');
	 *     There have to be two template variables: VAL_TITLE and VAL_CHECKBOX
	 * @return array
	 */
	protected function getStaticData()
	{
		return array(
			"id",
			"session_id",
			'title',
			"login",
			"name",
			"passes",
			"tstamp",
			"show_results"
		);
	}

	/**
	 * Define a final formatting for a cell value
	 *
	 * @param mixed $column
	 * @param array $row
	 *
	 * @return mixed
	 */
	protected function formatCellValue($column, array $row)
	{
		require_once "Services/User/classes/class.ilUserUtil.php";

		if ($column == "tstamp" || $column == "created")
		{
			$row["tstamp"] = ilDatePresentation::formatDate(new ilDateTime($row[$column], IL_CAL_UNIX));
		}
		if($column == "name")
		{
			$row["name"] = ilUserUtil::getNamePresentation($row["usr_id"]);
		}
		if($column == "show_results")
		{
			$this->ctrl->setParameter($this->parent_obj, "active_id", $row["active_id"]);
			$href = $this->ctrl->getLinkTarget($this->parent_obj, "ilFormATestParticipantsGUI.showUserResults");
			$this->ctrl->setParameter($this->parent_obj, "active_id", "");
			$row["show_results"] = '<a href="'.$href.'" >'.$this->plugin->txt("show_user_results").'</a>';
		}
		return parent::formatCellValue($column, $row);
	}
}
