<?php

require_once "abstract.ilXTSFTableGUI.php";
require_once 'Services/UIComponent/AdvancedSelectionList/classes/class.ilAdvancedSelectionListGUI.php';

/**
 * Class ilFormATestResultsTableGUI
 *
 * Date: 21.01.14
 * Time: 12:36
 * @author Thomas Joußen <tjoussen@databay.de>
 */ 
class ilFormATestResultsTableGUI extends ilXTSFTableGUI
{

	/**
	 * @var string
	 */
	protected $detailed_action = "ilFormATestParticipantsGUI.showUserDetails";

	public function __construct($controller, $plugin, $action="")
	{
		$this->setId("xtsf_results");

		parent::__construct($controller, $action, $plugin);

		$this->createTable();
	}

	public function createTable()
	{
		$this->setFormName('results');
		$this->setStyle('table', 'fullwidth');
		$this->setDefaultOrderField("title");
		$this->setDefaultOrderDirection("ASC");

		$this->addColumn('', '', '1%', true);
		$this->addColumn($this->plugin->txt("pass_title"), 'pass_title', '');
		$this->addColumn($this->plugin->txt("start_date_title"), 'start_date_title', '');
		$this->addColumn($this->plugin->txt("end_date_title"), 'end_date_title', '');
		$this->addColumn($this->plugin->txt("status_title"), 'status_title', '');
		$this->addColumn($this->plugin->txt("steps_title"), 'steps_title', '');
		$this->addColumn($this->plugin->txt("reached_points_title"), 'reached_points_title', '');
		$this->addColumn($this->plugin->txt("percentage_title"), 'percentage_title', '');
		$this->addColumn('', '', '15%', true);

		$this->setPrefix('p_id');
		$this->setRowTemplate("tpl.il_xtsf_results_row.html", $this->parent_obj->plugin->getDirectory());

		$this->setFormAction($this->ctrl->getFormAction($this->parent_obj));

		$this->disable("sort");
		$this->enable('header');
	}

	/**
	 * Return an array of all static (always visible) data fields in a row.
	 * For each key there has to be a variable name VAL_<COLUMN_KEY> in your defined row template.
	 * Example:
	 *     return array('title', 'checkbox');
	 *     There have to be two template variables: VAL_TITLE and VAL_CHECKBOX
	 * @return array
	 */
	protected function getStaticData()
	{
		return array(
			"pass",
			'start_date',
			"end_date",
			"status",
			"answered_questions",
			"reached_points",
			"percentage",
			"actions",
			"scored"
		);
	}

	protected function prepareData(array &$data)
	{
		parent::prepareData($data);

		$data[\count($data)-1]["scored"] = "&otimes;";
	}


	/**
	 * Define a final formatting for a cell value
	 *
	 * @param mixed $column
	 * @param array $row
	 *
	 * @return mixed
	 */
	protected function formatCellValue($column, array $row)
	{
		require_once "Services/User/classes/class.ilUserUtil.php";

		if($column == "pass")
		{
			$row["pass"] = $row["pass"]+1;
		}
		if ($column == "tstamp" || $column == "start_date" || $column == "end_date")
		{
			$row[$column] = ilDatePresentation::formatDate(new ilDateTime($row[$column], IL_CAL_UNIX));
		}
		if($column == "reached_points")
		{
			$row["reached_points"] = sprintf($this->plugin->txt("points_of"), $row["points"], $row["max_points"]);
		}
		if($column == "percentage")
		{
			$row["percentage"] = 0;
			if($row["max_points"]>0)
			{
				$row["percentage"] = number_format(($row["points"] / $row["max_points"])*100, 2);
			}
			$row["percentage"] .= "%";
		}
		if($column == "status")
		{
			$row["status"] = $this->plugin->txt("status_".$row["status"]);
		}
		if($column == "actions")
		{
			$action = new ilAdvancedSelectionListGUI();
			$action->setId('asl_'.$row["session_id"]);
			$action->setListTitle($this->lng->txt('actions'));

			$this->ctrl->setParameter($this->parent_obj, "session_id", $row["session_id"]);
			$action->addItem($this->plugin->txt("details"),"", $this->ctrl->getLinkTarget($this->parent_obj, $this->detailed_action));
			$this->ctrl->setParameter($this->parent_obj, "session_id", "");

			$row["actions"] = $action->getHTML();
		}

		return parent::formatCellValue($column, $row);
	}

	/**
	 * @param string $detailed_action
	 */
	public function setDetailedAction($detailed_action)
	{
		if($detailed_action != "")
		{
			$this->detailed_action = $detailed_action;
		}
	}
}
 