<?php

require_once "abstract.ilXTSFTableGUI.php";
require_once 'Services/UIComponent/AdvancedSelectionList/classes/class.ilAdvancedSelectionListGUI.php';
require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/FormATest/classes/controllers/class.ilFormATestResultOuputGUI.php";

/**
 * Class ilFormATestSetResultTableGUI
 *
 * Date: 21.01.14
 * Time: 12:36
 * @author Thomas Joußen <tjoussen@databay.de>
 */
class ilFormATestSetResultTableGUI extends ilXTSFTableGUI
{

	/**
	 * @var bool
	 */
	protected $render_optional;

	/**
	 * @var int
	 */
	protected $total_max_points;

	/**
	 * @var null|string
	 */
	protected $modus = null;

	public function __construct($controller, $plugin, $render_optional = false, $modus = null)
	{
		$this->setId("xtsf_set_results");

		parent::__construct($controller, "", $plugin);

		$this->render_optional = $render_optional;
		$this->modus = $modus;
		$this->createTable();
	}

	public function createTable()
	{
		$this->setFormName('set_results');
		$this->setStyle('table', 'fullwidth');
		$this->setDefaultOrderField("header_step");
		$this->setDefaultOrderDirection("ASC");

		$this->addColumn($this->plugin->txt("header_step"), 'header_step', '1%');

		if($this->modus == null || $this->modus != ilFormATestResultOuputGUI::MODUS_PARTICIPANTS)
		{
			$this->addColumn($this->plugin->txt("header_path"), 'header_path', '25%');
			$this->addColumn($this->plugin->txt("header_question"), 'header_question', '25%');
		}


		if($this->render_optional)
		{
			$this->optionalColumns = array(
				"max_points" => "max_points",
				"reached_points" => "reached_points",
				"percentage" => "percentage"
			);
			$this->visibleOptionalColumns = array(
				"max_points" => true,
				"reached_points" => true,
				"percentage" => true
			);

			$this->addColumn($this->plugin->txt("max_points_title"), 'max_points_title', '');
			$this->addColumn($this->plugin->txt("reached_points_title"), 'reached_points_title', '');
			$this->addColumn($this->plugin->txt("percentage_title"), 'percentage_title', '');
		}

		$this->setPrefix('s_id');
		$this->setRowTemplate("tpl.il_xtsf_set_result_row.html", $this->parent_obj->plugin->getDirectory());

		$this->setFormAction($this->ctrl->getFormAction($this->parent_obj));

		$this->disable("sort");
		$this->enable('header');
	}

	/**
	 * Return an array of all static (always visible) data fields in a row.
	 * For each key there has to be a variable name VAL_<COLUMN_KEY> in your defined row template.
	 * Example:
	 *     return array('title', 'checkbox');
	 *     There have to be two template variables: VAL_TITLE and VAL_CHECKBOX
	 * @return array
	 */
	protected function getStaticData()
	{
		$data = array(
			"step",
		);

		if($this->modus == null || $this->modus != ilFormATestResultOuputGUI::MODUS_PARTICIPANTS)
		{
			$data[] = "question_title";
			$data[] = "path_title";
		}

		return $data;
	}

	protected function fillRow(array $row)
	{
		$this->prepareRow($row);

		if($this->modus == null)
		{
			$this->modus = "author";
		}

		if($this->tpl->blockExists($this->modus."_optional_column"))
		{
			$this->tpl->setCurrentBlock($this->modus.'_optional_column');
			foreach($this->optionalColumns as $index => $definition)
			{
				if(!$this->isColumnVisible($index))
				{
					continue;
				}

				$value = $this->formatCellValue($index, $row);
				if($row["step"] == 'Total')
				{
					$row["step"] = '<strong>'.$this->plugin->txt("total")."</strong>";
				}
				if((string)$value === '')
				{
					$this->tpl->touchBlock($this->modus.'_optional_column');
				}
				else
				{
					$this->tpl->setVariable('OPTIONAL_COLUMN_VAL_' . strtoupper($index), $value);
				}
			}
			$this->tpl->parseCurrentBlock();
		}

		$this->tpl->setCurrentBlock($this->modus);

		foreach($this->getStaticData() as $column)
		{
			$value = $this->formatCellValue($column, $row);
			$this->tpl->setVariable('VAL_' . strtoupper($column), $value);
		}

		$this->tpl->parseCurrentBlock();
	}

	protected function prepareData(array &$data)
	{
		$total = array(
			"step" => "Total",
			"max_points" => $this->total_max_points,
			"reached_points" => 0,
			"percentage" => 0,
		);

		foreach($data as $row)
		{
			$total["reached_points"] += $row["reached_points"];
		}

		if($total["max_points"]> 0)
		{
			$total["percentage"] = number_format(($total["reached_points"]/$total["max_points"])*100,2);
		}
		$data[] = $total;
	}


	/**
	 * Define a final formatting for a cell value
	 *
	 * @param mixed $column
	 * @param array $row
	 *
	 * @return mixed
	 */
	protected function formatCellValue($column, array $row)
	{
		if ($column == "question_title" && $row["step"] != "Total")
		{
			$row[$column] = "(Q". $row["question_index"] . ") " . $row[$column];
		}
		if($column == "percentage")
		{
			$row["percentage"] = 0;
			if($row["max_points"] >0)
			{
				$row["percentage"] = number_format(($row["reached_points"] / $row["max_points"])*100, 2);
			}
		}

		if($row["step"] == "Total")
		{
			$row[$column] = "<strong>". $row[$column]. "</strong>";
		}

		return parent::formatCellValue($column, $row);
	}

	/**
	 * @param int $total_max_points
	 */
	public function setTotalMaxPoints($total_max_points)
	{
		$this->total_max_points = $total_max_points;
	}
}
 