<?php

require_once "abstract.ilXTSFTableGUI.php";
require_once "Services/Form/classes/class.ilNumberInputGUI.php";

/**
 * Class ilFormATestSetsTableGUI
 *
 * Date: 07.10.13
 * Time: 13:49
 * @author Thomas Joußen <tjoussen@databay.de>
 */ 
class ilFormATestSetsTableGUI extends ilXTSFTableGUI
{
	/**
	 * @var bool
	 */
	protected $has_participants = false;

	/**
	 * @var bool
	 */
	protected $show_sorting = false;

	public function __construct($controller, $plugin, $action = "", $show_sorting = false, $has_participants = false)
	{
		global $ilCtrl;

		$this->ctrl = $ilCtrl;
		$this->has_participants = $has_participants;
		$this->show_sorting     = $show_sorting;
		
		$this->setId('xtsf_sets_' . $plugin->getId() . '_' . ($show_sorting ? 'list' : 'browsw'));
		parent::__construct($controller, $action, $plugin);

		$this->createTable();
	}

	private function createTable()
	{
		$this->setTitle($this->plugin->txt("setsbrowser"));

		$this->setFormName('setsbrowser');
		$this->setStyle('table', 'fullwidth');
		$this->setDefaultOrderField("title");
		$this->setDefaultOrderDirection("ASC");

		if(!$this->has_participants)
		{
			$this->addColumn('', '', '1%', true);
		}

		if($this->show_sorting)
		{
			$this->addColumn($this->plugin->txt("sorting"), 'sorting', '5%');
			$this->setDefaultOrderField("sorting");
			$this->setDefaultOrderDirection("DESC");
		}

		$this->addColumn($this->lng->txt("title"), 'title', '25%');
		$this->addColumn($this->plugin->txt('set_id'), 'id', '5%');
		$this->addColumn($this->lng->txt("description"), 'description', '55%');

		if(!$this->has_participants)
		{
			$this->setSelectAllCheckbox('s_id');
		}

		$this->setRowTemplate("tpl.il_xtsf_qs_row.html", $this->parent_obj->plugin->getDirectory());

		$this->setFormAction($this->ctrl->getFormAction($this->parent_obj));

		$this->disable("sort");
		$this->enable('header');
	}

	protected function prepareRow(array &$row)
	{
		if(!$this->has_participants)
		{
			$row['checkbox'] = ilUtil::formCheckbox(false, 's_id[]', $row['id']);
		}
		if($this->show_sorting)
		{
			$sort = new ilNumberInputGUI('', 'sorting[]');
			$sort->allowDecimals(false);
			$sort->setSize(3);
			$sort->setValue($row['sorting']);
			if($this->has_participants)
			{
				$sort->setDisabled(true);
			}
			$row['sorting'] = $sort->render();
		}
		else
		{
			unset($row['sorting']);
		}
	}

	/**
	 * Return an array of all static (always visible) data fields in a row.
	 * For each key there has to be a variable name VAL_<COLUMN_KEY> in your defined row template.
	 * Example:
	 *     return array('title', 'checkbox');
	 *     There have to be two template variables: VAL_TITLE and VAL_CHECKBOX
	 * @return array
	 */
	protected function getStaticData()
	{
		$static_columns = array(
			'checkbox',
			'id',
			'title',
			'description',
		);
		
		if($this->show_sorting)
		{
			$static_columns[] = 'sorting';
		}
		
		return $static_columns;
	}

	/**
	 * Define a final formatting for a cell value
	 *
	 * @param mixed $column
	 * @param array $row
	 *
	 * @return mixed
	 */
	protected function formatCellValue($column, array $row)
	{
		if ($column == "tstamp" || $column == "created")
		{
			return ilDatePresentation::formatDate(new ilDateTime($row[$column], IL_CAL_UNIX));
		}
		if($column == "title" && !$this->has_participants)
		{
			$this->ctrl->setParameterByClass("ilObjFormATestQuestionSetPoolGUI", "set_id", $row["id"]);
			$href = $this->ctrl->getLinkTargetByClass("ilObjFormATestQuestionSetPoolGUI", "ilQuestionSetPoolPathsGUI.listPaths");
			$this->ctrl->setParameterByClass("ilObjFormATestQuestionSetPoolGUI", "set_id", "");

			$row[$column] = '<a href="'.$href.'">'.$row[$column] . '</a>';
		}
		return parent::formatCellValue($column, $row);
	}
}
