<?php

require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/Parser/classes/ConditionParser.php";
require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/Parser/classes/CompositeEvaluator.php";
require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/classes/providers/class.ilParserQuestionProvider.php";
require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/classes/models/class.ilQuestionSetPoolJumpCondition.php";
require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/FormATest/interfaces/interface.ilConditionEvaluationSubject.php";
require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/FormATest/interfaces/interface.ilConditionEvaluationObserver.php";
require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/FormATest/interfaces/interface.ilFormATestFeedbackConstants.php";

/**
 * Class ilFormATestConditionEvaluation
 *
 * Date: 06.01.14
 * Time: 16:43
 * @author Thomas Joußen <tjoussen@databay.de>
 */ 
abstract class ilFormATestConditionEvaluation implements ilConditionEvaluationSubject, ilFormATestFeedbackConstants
{
	/**
	 * @var ilConditionEvaluationObserver[]
	 */
	public static $observers = array();

	/**
	 * @var array
	 */
	public static $state = array();

	/**
	 * @var array
	 */
	public static $fb = array();
	
	/**
	 * @param ilQuestionSetPoolNode $current_node
	 * @param ilFormATestSession $session
	 *
	 * @return null|ilQuestionSetPoolNode
	 */
	public static function getNextNode($current_node, $session)
	{
		$evaluator = self::initializeEvaluator($current_node->getPath()->getSet()->getId(), $session);
		self::resetFeedback();
		
		if(ilFormATestConditionEvaluation::isConditionTrue($evaluator, $current_node->getInitialCondition()))
		{
			$session->updateCondition($current_node->getId(), $current_node->getInitialCondition());
			// Ausgangsbedingung, Feedback true
			self::addFeedback($current_node, $current_node->getIndex(), $session, ilFormATestFeedbackConstants::INITIAL_FEEDBACK_TRUE);
			self::notify();

			return $current_node->getPath()->getNodes()->getNextNode($current_node);
		}
		else
		{
			// Ausgangsbedingung,Feedback false
			self::addFeedback($current_node, $current_node->getIndex(), $session, ilFormATestFeedbackConstants::INITIAL_FEEDBACK_FALSE);
			//self::notify();

			$jump_conditions = $current_node->getJumpConditionList()->getData();
			foreach($jump_conditions as $item)
			{
				$condition = new ilQuestionSetPoolJumpCondition($item["id"]);
				$condition->read();

				if(ilFormATestConditionEvaluation::isConditionTrue($evaluator, $condition->getJumpCondition()))
				{
					$session->updateCondition($current_node->getId(), $condition->getJumpCondition(), $condition->getId());
					if(!$session->reachedDeadlock())
					{
						$deleteIndex = null;
						// Sprungbedingung,Feedback true
						$nextNode = null;
						
						if($condition->getQuestion() != null)
						{
							$nextNode = $current_node->getPath()->getNodes()->getNodeByQuestionId($condition->getQuestion()->getId());
							
							$node_index = self::getAnchorIndex($nextNode);

							// vorher
							$deleteIndex = $node_index;
							self::addFeedback($current_node, $node_index, $session, ilFormATestFeedbackConstants::JMP_FEEDBACK_TRUE, $condition->getId());
							self::updateNodeIndex($node_index);
							//self::notify();
						}
						else
						{
							// WEnn das Set abgebrochen wird
							self::addFeedback($current_node, $current_node->getIndex(), $session, ilFormATestFeedbackConstants::JMP_FEEDBACK_TRUE, $condition->getId());
							self::updateNodeIndex($current_node->getIndex());
							//self::notify();
						}				

						if($deleteIndex !== null)
						{
							self::deleteFeedback($deleteIndex, $session->getSetId(), $session->getSessionId());	
						}	
						self::notify();
						return $nextNode;
					}
					else
					{
						//  Sprungbedingung, Feedback no_jmp
						self::addFeedback($current_node, $current_node->getIndex(), $session, ilFormATestFeedbackConstants::JMP_FEEDBACK_TRUE_JUMP, $condition->getId());
						//self::notify();
						
						$session->unsetJump();
						$session->markSolutionToFixate();
					}
				}
				else
				{
					// Sprungbedingung, Feedback false
					self::addFeedback($current_node, $current_node->getIndex(), $session, ilFormATestFeedbackConstants::JMP_FEEDBACK_FALSE, $condition->getId());
					//self::notify();
				}
			}
			return self::doPathShift($current_node, $session, $evaluator);
		}
	}
	
	/**
	 * @param CompositeEvaluator $evaluator
	 * @param string $condition
	 *
	 * @return bool
	 */
	public static function isConditionTrue($evaluator, $condition)
	{
		if(trim($condition) == "")
		{
			return false;
		}

		$parser = new ConditionParser();
		$composite = $parser->parse($condition);

		return $evaluator->evaluate($composite);
	}

	protected static function initializeEvaluator($set_id, $session)
	{
		$provider  = new ilParserQuestionProvider($set_id);
		$evaluator = new CompositeEvaluator($provider, $session);

		return $evaluator;
	}

	/**
	 * @param ilQuestionSetPoolNode $current_node
	 * @param ilFormATestSession $session
	 * @param CompositeEvaluator $evaluator
	 *
	 * @return ilQuestionSetPoolNode|null
	 */
	protected static function doPathShift($current_node, $session, $evaluator)
	{
		$paths = $current_node->getPath()->getSet()->getPathList()->getData();
		foreach($paths as $path)
		{
			if($path["id"] != $current_node->getPath()->getId())
			{
				$reset_feedback = false;

				$path = new ilQuestionSetPoolPath($path["id"]);
				$path->read();

				if($path->isEqualsUntilIndex($current_node->getPath(), $current_node->getIndex()))
				{
					$node = $path->getNodes()->getNodeByQuestionId($current_node->getQuestion()->getId());

					if($node != null && self::arePreviousConditionsValid($evaluator, $path, $node))
					{
						self::resetFeedback(true);
						$reset_feedback = true;
						
						if(ilFormATestConditionEvaluation::isConditionTrue($evaluator, $node->getInitialCondition()))
						{
							$session->updateCondition($node->getId(), $node->getInitialCondition());

							self::addFeedback($node, $node->getIndex(), $session, ilFormATestFeedbackConstants::INITIAL_FEEDBACK_TRUE);
							self::updateNodeIndex($node->getIndex());
							self::notify();

							return $node->getPath()->getNodes()->getNextNode($node);
						}
						else
						{
							self::addFeedback($node, $node->getIndex(), $session, ilFormATestFeedbackConstants::INITIAL_FEEDBACK_FALSE);
							self::updateNodeIndex($node->getIndex());
						}
					}

					$jump_conditions = $node->getJumpConditionList()->getData();
					foreach($jump_conditions as $item)
					{
						$condition = new ilQuestionSetPoolJumpCondition($item["id"]);
						$condition->read();

						if(self::arePreviousConditionsValid($evaluator, $path, $node))
						{
							if(!$reset_feedback)
							{
								self::resetFeedback(true);
								$reset_feedback = true;
							}
							if(ilFormATestConditionEvaluation::isConditionTrue($evaluator, $condition->getJumpCondition()))
							{
								$session->updateCondition($node->getId(), $condition->getJumpCondition(), $condition->getId());
								if(!$session->reachedDeadlock())
								{
									$deleteIndex = null;
									$nextNode    = null;
									if($condition->getQuestion() != null)
									{
										$nextNode = $node->getPath()->getNodes()->getNodeByQuestionId($condition->getQuestion()->getId());

										$node_index  = self::getAnchorIndex($nextNode);
										$deleteIndex = $node_index;

										self::addFeedback($node, $node_index, $session, ilFormATestFeedbackConstants::JMP_FEEDBACK_TRUE, $condition->getId());
										self::updateNodeIndex($node_index);
									}
									else
									{
										self::addFeedback($node, $node->getIndex(), $session, ilFormATestFeedbackConstants::JMP_FEEDBACK_TRUE, $condition->getId());
										self::updateNodeIndex($node->getIndex());
									}

									if($deleteIndex !== null)
									{
										self::deleteFeedback($deleteIndex, $session->getSetId(), $session->getSessionId());
									}
									self::notify();
									return $nextNode;
								}
								else
								{
									self::addFeedback($node, $node->getIndex(), $session, ilFormATestFeedbackConstants::JMP_FEEDBACK_TRUE_JUMP, $condition->getId());
									self::updateNodeIndex($node->getIndex());

									$session->unsetJump();
									$session->markSolutionToFixate();
								}
							}
							else
							{
								self::addFeedback($node, $node->getIndex(), $session, ilFormATestFeedbackConstants::JMP_FEEDBACK_FALSE, $condition->getId());
								self::updateNodeIndex($node->getIndex());
							}
						}
					}
				}
			}
		}
		self::notify();
		return null;
	}

	/**
	 * @param CompositeEvaluator $evaluator
	 * @param ilQuestionSetPoolPath $path
	 * @param ilQuestionSetPoolNode $node
	 *
	 * @return bool
	 */
	protected static function arePreviousConditionsValid($evaluator, $path, $node)
	{
		$nodes = $path->getNodes()->getData();

		foreach($nodes as $prev_node)
		{
			if($prev_node["node_index"] >= $node->getIndex())
			{
				break;
			}

			if(!ilFormATestConditionEvaluation::isConditionTrue($evaluator,$prev_node["initial_condition"]))
			{
				return false;
			}
		}

		return true;
	}

	/**
	 * @param ilConditionEvaluationObserver $observer
	 *  @throws InvalidArgumentException
	 */
	public static function attach(ilConditionEvaluationObserver $observer)
	{
		$key = array_search($observer, self::$observers);
		if($key)
		{
			throw new InvalidArgumentException('Observer already attached to subject.');
		}
		self::$observers[] = $observer;
	}

	/**
	 * @param ilConditionEvaluationObserver $observer
	 * @throws InvalidArgumentException
	 */
	public static function detach(ilConditionEvaluationObserver $observer)
	{
		$key = array_search($observer, self::$observers, true);
		if($key === false)
		{
			throw new InvalidArgumentException('Observer not found in collection.');
		}
		unset(self::$observers[$key]);
	}
	
	public static function notify()
	{
		foreach(self::$observers as $observer)
		{
			foreach(self::$fb as $fb) {
				$observer->updateConditionEvaluation($fb);
			}
		}
		
		self::resetFeedback();
	}

	public static function addFeedback($current_node, $node_index, $session, $feedback_type, $condition_id = NULL)
	{
		self::$fb[] = array(
			'node'          => $current_node,
			'node_index'	=> $node_index,
			'session'       => $session,
			'feedback_type' => $feedback_type,
			'condition_id'  => $condition_id
		);
	}

	/**
	 * @param bool|false $path_sift
	 */
	public static function resetFeedback($path_sift = false)
	{
		if($path_sift == true)
		{
			$tmp_fb = array();

			foreach(self::$fb as $fb)
			{
				if($fb['feedback_type'] == ilFormATestFeedbackConstants::JMP_FEEDBACK_TRUE_JUMP)
				{
					$tmp_fb[] = $fb; 
				}
			}
			
			self::$fb = $tmp_fb;
		}	
		else
		{
			self::$fb = array();
		}
	}
	
	public static function updateNodeIndex($node_index) 
	{
		foreach(self::$fb as $key => $item) 
		{
			self::$fb[$key]['node_index'] = $node_index;
		}
	}
	

	public static function deleteFeedback($node_index, $set_id, $session_id)
	{
		foreach(self::$observers as $observer)
		{
			foreach(self::$fb as $fb) {
				$observer->deleteFeedback($node_index, $set_id, $session_id);
			}
		}
	}

	/**
	 * @param $nextNode
	 *
	 * @return int
	 */
	public static function getAnchorIndex($nextNode)
	{
		$feedbackAnchorNode = $nextNode->getPath()->getNodes()->getPreviousNode($nextNode);
		if(is_object($feedbackAnchorNode))
		{
			$node_index = $feedbackAnchorNode->getIndex();

			return $node_index;
		}
		else
		{
			// Wenn kein Vorgänger vorhanden, dann muss Knotenindex immer 0 sein!
			return 0;
		}
	}
	
}
 