<?php

/**
 * Class ilFormATestPlayerAccessHelper
 *
 * Date: 17.01.14
 * Time: 17:16
 * @author Thomas Joußen <tjoussen@databay.de>
 */ 
abstract class ilFormATestPlayerAccessHelper
{

	/**
	 * Check whether it is possible to start a test.
	 * Checks the test settings and the user access level
	 *
	 * @param ilObjFormATestGUI $controller
	 *
	 * @return bool
	 */
	public static function canDoTestAction($controller, ilFormATestSession $session)
	{
		if(!$controller->object->isOnline())
		{
			ilUtil::sendInfo($controller->plugin->txt("test_is_offline"));
			return false;
		}

		if(
			$controller->object->isStartTimeEnabled() &&
			ilDateTime::_after($controller->object->getStartTime(), new ilDateTime(time(), IL_CAL_UNIX))
		){

			ilUtil::sendInfo(sprintf($controller->plugin->txt("test_start_future"), $controller->object->getStartTime()));
			return false;
		}

		if(
			$controller->object->isEndTimeEnabled() &&
			ilDateTime::_before($controller->object->getEndTime(), new ilDateTime(time(), IL_CAL_UNIX))
		)
		{
			ilUtil::sendInfo(sprintf($controller->plugin->txt("test_start_past"), $controller->object->getEndTime()));
			return false;
		}

		// @todo @Thomas: Respect max passes setting
		if(
			$controller->object->getMaxPasses() > 0 &&
			$session->activeIDExists() &&
			(
				$session->getPass() + 1 >= $controller->object->getMaxPasses()
			)
		)
		{
			if(
				($session->getPass() + 1 == $controller->object->getMaxPasses() && $session->isPassFinished()) ||
				$session->getPass() + 1 > $controller->object->getMaxPasses()
			)
			{
				ilUtil::sendInfo($controller->lng->txt('maximum_nr_of_tries_reached'), true);
				return false;
			}
		}
		
		return true;
	}
}
 