<?php

require_once "./Services/InfoScreen/classes/class.ilInfoScreenGUI.php";

/**
 * Class ilFormATestInfoScreen
 *
 * Date: 28.01.14
 * Time: 13:47
 * @author Thomas Joußen <tjoussen@databay.de>
 *
 * @ilCtrl_Calls ilFormATestInfoScreenGUI: ilNoteGUI, ilColumnGUI, ilPublicUserProfileGUI
 * @ilCtrl_Calls ilFormATestInfoScreenGUI: ilCommonActionDispatcherGUI
 */ 
class ilFormATestInfoScreenGUI extends ilInfoScreenGUI {

	/**
	 * @var string
	 */
	protected $intro = "";

	/**
	 * @param string $intro
	 */
	public function setIntro($intro)
	{
		$this->intro = $intro;
	}

	/**
	 * get html
	 */
	function getHTML()
	{
		global $lng, $ilSetting, $tree, $ilAccess, $ilCtrl, $ilUser;

		$tpl= new ilTemplate("tpl.infoscreen.html", true, true, "Customizing/global/plugins/Services/Repository/RepositoryObject/FormATest");

		if($this->intro != "")
		{
			$this->addSection($this->lng->txt("tst_introduction"));
			$this->addProperty("", $this->intro);
		}

		// other class handles form action (@todo: this is not implemented/tested)
		if ($this->form_action == "")
		{
			$this->setFormAction($ilCtrl->getFormAction($this));
		}

		if($this->hidden)
		{
			$tpl->touchBlock("hidden_js");
			if($this->show_hidden_toggle)
			{
				$this->addButton($lng->txt("toggle_hidden_sections"), "JavaScript:toggleSections();");
			}
		}
		// add top buttons
		if (count($this->top_buttons) > 0)
		{
			$tpl->addBlockfile("TOP_BUTTONS", "top_buttons", "tpl.buttons.html");

			foreach($this->top_buttons as $button)
			{
				// view button
				$tpl->setCurrentBlock("btn_cell");
				$tpl->setVariable("BTN_LINK", $button["link"]);
				$tpl->setVariable("BTN_TARGET", $button["target"]);
				$tpl->setVariable("BTN_TXT", $button["title"]);
				if($button["primary"])
				{
					$tpl->setVariable("BTN_CLASS", " emphsubmit");
				}
				$tpl->parseCurrentBlock();
			}
		}

		// add top formbuttons
		if ((count($this->top_formbuttons) > 0) && (strlen($this->form_action) > 0))
		{
			$tpl->addBlockfile("TOP_FORMBUTTONS", "top_submitbuttons", "tpl.submitbuttons.html", "Services/InfoScreen");

			foreach($this->top_formbuttons as $button)
			{
				// view button
				$tpl->setCurrentBlock("btn_submit_cell");
				$tpl->setVariable("BTN_COMMAND", $button["command"]);
				$tpl->setVariable("BTN_NAME", $button["title"]);
				$tpl->parseCurrentBlock();
			}
		}

		// add form action
		if (strlen($this->form_action) > 0)
		{
			if($this->open_form_tag)
			{
				$tpl->setCurrentBlock("formtop");
				$tpl->setVariable("FORMACTION", $this->form_action);
				$tpl->parseCurrentBlock();
			}

			if($this->close_form_tag)
			{
				$tpl->touchBlock("formbottom");
			}
		}

		if (count($this->hiddenelements))
		{
			foreach ($this->hiddenelements as $hidden)
			{
				$tpl->setCurrentBlock("hidden_element");
				$tpl->setVariable("HIDDEN_NAME", $hidden["name"]);
				$tpl->setVariable("HIDDEN_VALUE", $hidden["value"]);
				$tpl->parseCurrentBlock();
			}
		}

		// learning progress
		if($this->learning_progress_enabled and $html = $this->showLearningProgress($tpl))
		{
			$tpl->setCurrentBlock("learning_progress");
			$tpl->setVariable("LP_TABLE",$html);
			$tpl->parseCurrentBlock();
		}

		// notes section
		if ($this->private_notes_enabled && !$ilSetting->get('disable_notes'))
		{
			$html = $this->showNotesSection();
			$tpl->setCurrentBlock("notes");
			$tpl->setVariable("NOTES", $html);
			$tpl->parseCurrentBlock();
		}

		// tagging
		if (is_object($this->gui_object->object))
		{
			$tags_set = new ilSetting("tags");
			if ($tags_set->get("enable") && $ilUser->getId() != ANONYMOUS_USER_ID)
			{
				$this->addTagging();
			}
		}

		if(is_object($this->gui_object->object))
		{
			$this->addObjectSections();
		}

		// render all sections
		for($i = 1; $i <= $this->sec_nr; $i++)
		{
			if (is_array($this->section[$i]["properties"]))
			{
				// section properties
				foreach($this->section[$i]["properties"] as $property)
				{
					if ($property["name"] != "")
					{
						if ($property["link"] == "")
						{
							$tpl->setCurrentBlock("pv");
							$tpl->setVariable("TXT_PROPERTY_VALUE", $property["value"]);
							$tpl->parseCurrentBlock();
						}
						else
						{
							$tpl->setCurrentBlock("lpv");
							$tpl->setVariable("TXT_PROPERTY_LVALUE", $property["value"]);
							$tpl->setVariable("LINK_PROPERTY_VALUE", $property["link"]);
							$tpl->parseCurrentBlock();
						}
						$tpl->setCurrentBlock("property_row");
						$tpl->setVariable("TXT_PROPERTY", $property["name"]);
						$tpl->parseCurrentBlock();
					}
					else
					{
						$tpl->setCurrentBlock("property_full_row");
						$tpl->setVariable("TXT_PROPERTY_FULL_VALUE", $property["value"]);
						$tpl->parseCurrentBlock();
					}
				}

				// section header
				if($this->section[$i]["hidden"])
				{
					$tpl->setVariable("SECTION_HIDDEN", " style=\"display:none;\"");
					$tpl->setVariable("SECTION_ID", "hidable_".$i);
				}
				else
				{
					$tpl->setVariable("SECTION_ID", $i);
				}
				$tpl->setVariable("TCLASS", $this->getTableClass());
				$tpl->setVariable("TXT_SECTION", $this->section[$i]["title"]);
				$tpl->touchBlock("row");
			}
		}

		return $tpl->get();
	}
}
 