<?php

require_once './Customizing/global/plugins/Services/Repository/RepositoryObject/FormATest/classes/providers/class.ilFormATestQuestionListProvider.php';
require_once './Customizing/global/plugins/Services/Repository/RepositoryObject/FormATest/classes/providers/class.ilFormATestSetProvider.php';
require_once './Customizing/global/plugins/Services/Repository/RepositoryObject/FormATest/classes/models/class.ilFormATestSetResultList.php';
require_once './Customizing/global/plugins/Services/Repository/RepositoryObject/FormATest/classes/utils/class.ilFormATestSkillAssQuestionList.php';

/**
 * Class ilFormATestSkillTriggerHelper
 *
 * @author  Thomas Joußen <tjoussen@databay.de>
 * @since   01.04.15
 */
class ilFormATestSkillLevelHelper
{

	/**
	 * @var ilDB
	 */
	protected $db;

	/**
	 * @var ilLanguage
	 */
	protected $lng;

	/**
	 * @var ilPluginAdmin
	 */
	protected $pluginAdmin;

	/**
	 * @var ilSetting
	 */
	protected $ilSettings;

	/**
	 * @var int
	 */
	protected $testId;

	/**
	 * @var ilFormATestSession
	 */
	protected $session;

	/**
	 * @var int
	 */
	protected $refId;

	public function __construct()
	{
		global $ilDB, $lng, $ilPluginAdmin, $ilSetting;

		$this->db = $ilDB;
		$this->lng = $lng;
		$this->pluginAdmin = $ilPluginAdmin;
		$this->ilSetting = $ilSetting;
	}

	/**
	 * @param ilObjFormATest $object
	 */
	public function trigger($object)
	{
		$skillEvaluation 	= $this->prepareSkillEvaluation();
		$questionList 		= $this->loadQuestionList($object);
		$testResults 		= $this->getTestResults($object);

		$skillEvaluation->init($questionList);
		$skillEvaluation->evaluate($testResults);

		$skillEvaluation->handleSkillTriggering();
	}

	/**
	 * @param \ilFormATestSession $session
	 */
	public function setSession($session)
	{
		$this->session = $session;
	}

	/**
	 * @param int $testId
	 */
	public function setTestId($testId)
	{
		$this->testId = $testId;
	}

	/**
	 * @param int $refId
	 */
	public function setRefId($refId)
	{
		$this->refId = $refId;
	}

	/**
	 * @param ilObjFormATest
	 *
	 * @return ilFormATestSkillAssQuestionList
	 */
	public function loadQuestionList(ilObjFormATest $object)
	{
		$provider = new ilFormATestQuestionListProvider();

		$questionList = new ilFormATestSkillAssQuestionList($this->db, $this->lng, $this->pluginAdmin, $object->getId());
		$questionList->setQuestionInstanceTypeFilter(ilAssQuestionList::QUESTION_INSTANCE_TYPE_DUPLICATES);
		$questionList->load($provider);

		return $questionList;
}

	/**
	 * @return ilTestSkillEvaluation
	 */
	protected function prepareSkillEvaluation()
	{
		require_once 'Modules/Test/classes/class.ilTestSkillEvaluation.php';
		$skillEvaluation = new ilTestSkillEvaluation($this->db, $this->testId, $this->refId);

		$skillEvaluation->setUserId($this->session->getUserId());
		$skillEvaluation->setActiveId($this->session->getActiveId());
		$skillEvaluation->setPass($this->session->getPass());

		$skillEvaluation->setNumRequiredBookingsForSkillTriggering(
			$this->ilSetting->get(
				'ass_skl_trig_num_answ_barrier',
				ilObjAssessmentFolder::DEFAULT_SKL_TRIG_NUM_ANSWERS_BARRIER
			)
		);

		return $skillEvaluation;
	}

	/**
	 * @param ilObjFormATest $object
	 *
	 * @return array
	 */
	public function getTestResults(ilObjFormATest $object)
	{
		$setProvider = new ilFormATestSetProvider();
		$sets = $setProvider->loadData(array('form_a_test' => $object->getId()));

		$results = array();
		foreach($sets as $set)
		{
			$provider = new ilFormATestSetResultList($object ,$set['id'], $this->session->getSessionId());
			$data = $provider->getData();
			$results = array_merge($results, $data);
		}

		/**
		 * Manipulate results array to be usable by class.ilTestSkillLevelEvaluation.php
		 */
		foreach($results as $key => $result)
		{
			$results[$key]['reached'] = $result['reached_points'];
			$results[$key]['points'] = $result['max_points'];
			$results[$key]['qid'] = $result['question_fi'];
			$results[$key]['workedthrough'] = true;
		}

		return $results;
	}
}
 