<?php

/**
 * Class ilObjTestGateway
 *
 * Date: 13.01.14
 * Time: 12:21
 * @author Thomas Joußen <tjoussen@databay.de>
 */ 
class ilObjTestGateway {

	/**
	 * @param int $active_id
	 * @param int $pass
	 *
	 * @return array
	 */
	public function getQuestionCountAndPointsForPassOfParticipant($active_id, $pass)
	{
		/** @var ilDB $ilDB */
		global $ilDB;

		$data = array("count" => 0, "points" => 0);
		$points = $this->getMaxPointsPerSet($active_id);

		$result = $ilDB->queryF(
			"SELECT COUNT(tracking.step) qcount
			FROM rep_robj_xtsf_tracking tracking
			INNER JOIN rep_robj_xtsf_session sess ON sess.session_id = tracking.session_fi
			WHERE sess.active_id = %s AND sess.current_pass = %s",
			array("integer", "integer"),
			array($active_id, $pass)
		);

		$row = $ilDB->fetchAssoc($result);
		foreach($points as $element)
		{
			$data["points"] += $element;
		}

		if(is_array($row))
		{
			$data["count"] = $row["qcount"];
		}

		return $data;
	}

	public function getWorkingTimeOfParticipantForPass($active_id, $pass)
	{
		/** @var ilDB $ilDB */
		global $ilDB;

		$result = $ilDB->queryF(
			"SELECT start_date FROM rep_robj_xtsf_session WHERE active_id = %s AND current_pass = %s",
			array("integer", "integer"),
			array($active_id, $pass)
		);

		$row = $ilDB->fetchAssoc($result);

		$now = time();

		return $now - $row["start_date"];
	}

	public function getMaxPointsPerSet($active_id)
	{
		/** @var ilDB $ilDB */
		global $ilDB;

		$points = array();

		$result = $ilDB->queryF(
			"SELECT SUM(
 				CASE
 					WHEN node.dominant_scoring IS NOT NULL
 					THEN node.dominant_scoring
 					ELSE qst.points
 				END
 			) qsum, qs.id set_id
 			FROM qpl_questions qst
 			INNER JOIN rep_robj_xqsp_node node ON node.question_fi = qst.question_id
 			INNER JOIN rep_robj_xqsp_path path ON path.id = node.path_fi
 			INNER JOIN rep_robj_xqsp_qs qs ON qs.id = path.questionset_fi
 			INNER JOIN tst_tests test ON test.obj_fi = qs.questionsetpool_fi
 			INNER JOIN tst_active active ON active.test_fi = test.test_id
 			WHERE active.active_id = %s
 			GROUP BY path.id, set_id",
			array("integer"),
			array($active_id)
		);

		while(($row = $ilDB->fetchAssoc($result)) != null)
		{
			if(array_key_exists($row["set_id"], $points))
			{
				if($points[$row["set_id"]] < $row["qsum"])
				{
					$points[$row["set_id"]] = $row["qsum"];
				}
			}
			else
			{
				$points[$row["set_id"]] = $row["qsum"];
			}
		}

		return $points;
	}

}
 